      SUBROUTINE DADIO
     I                ( LUPRT, LUDSR, LUTDDB, LISTN, DSRNAM, TDDBNM,
     O                  IRETCD )
C
C     + + + PURPOSE + + +
C     Initialize logical unit numbers and other variables and open
C     the TDDB and DSR file
C
C     + + + HISTORY + + +
C     MODIFIED June 28, 1995 -- RSR
C       set Pdate in DADIO, not in DADSUM, put Iyr, Imo, Ida, Ihr, Imn
C       in DADBLK.CMN
C     MODIFIED January 20, 1995 -- RSR
C       removed printing of summaries to DADSUM
C     MODIFIED December 1, 1994 -- RSR
C       code clean-up, arguments changed to follow implicit typing,
C       DADO and DADI names switched, irdpdy moved from after idata
C       to before in DADI and DADO argument lists.
C     MODIFIED September 1, 1994 -- RSR
C       added use storage of floating point data
C       parameters may be stored in any data type
C     MODIFIED October 8, 1993 -- RSR
C       removed use of I*2
C     MODIFIED July 16, 1993 -- RSR
C       removed PUNCH dummy argument since not used.
C     VERSION 06/01/89  16-DIGIT STATION IDENTIFIERS
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER LUPRT, LUDSR, LUTDDB, LISTN, IRETCD
      CHARACTER*(*) DSRNAM, TDDBNM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LUPRT  - Unit number of printed output
C     LUDSR  - Unit number of DSR file
C     LUTDDB - Unit number of TDDB
C     LISTN  - Summary list option (-1=calendar, 0=none, 1=directory,
C              2=conventions summary)
C     DSRNAM - Filename of DSR file
C     TDDBNM - Filename of TDDB
C     IRETCD - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + INCLUDES + + +
      INCLUDE 'dadblk.cmn'
      INCLUDE 'datdio.cmn'
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
      INCLUDE 'datyps.cmn'
      INCLUDE 'dastyp.cmn'
      INCLUDE 'dstrf.cmn'
C     + + + COMMONS + + +
      CHARACTER*60 Versn
      COMMON /VERSION/ Versn
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, listab, iret
      CHARACTER rvers*60
C
C     + + + EXTERNALS + + +
      EXTERNAL RDINDX, INTNDX, WRTNDX, RDDSRF, ERRDAD, DADSUM, OPENDB,
     &         SYSDATE, SYSTIME, DADINI, DTCODE
C
C     + + + END SPECIFICATIONS + + +
C
      Versn =
     &     '@(#) libdadio - Storage/retrieval routines to access a TDDB'
      Versn = '@(#) libdadio - Contact: h2osoft@usgs.gov'
      Versn = '@(#) libdadio - Version: 6.0x 1997/02/05'
      rvers =
     &  '$Id: dadio.f,v 6.0 1997/02/05 20:03:59 rsregan Exp rsregan $'
C this string is used for labeling printouts.
      Versn = 'DADIO - VERSION:'//rvers(15:29)
C
      Printr = LUPRT
      Tddata = LUTDDB
      Dsref = LUDSR
      listab = LISTN
      CALL DADINI()
      CALL SYSDATE(Iyr,Imo,Ida)
      CALL SYSTIME(Ihr,Imn,i)
      CALL DTCODE(Iyr,Imo,Ida,Ihr,Imn,Pdate,i,iret)
      IF (iret.NE.0) THEN
        IRETCD = -3
        GOTO 30
      ENDIF
C
      IF (Tddata.EQ.Printr .OR. Tddata.EQ.Dsref) THEN
        IRETCD = -7
        GOTO 30
      ENDIF
C
C WHEN IRETCD=999, DADIO IS BEING CALLED FOR THE FIRST TIME BY A TDDS
C UTILITY PROGRAM (I.E. DAFILE, DAGET, BACKUP).  THE FOLLOWING FLAGS
C MUST BE SET TO THE VALUES SPECIFIED TO KEEP THE TDDB INDEX PROPERLY
C SORTED, AND THE DSRFILE INDEX UP TO DATE. WHEN OTHER PROGRAMS SUCH AS
C BRANCH CALL DADIO, THE FIRST FLAG ALLOWS THE VALUES TO BE SET CORRECTLY.
C IF SYSTEMS OF PROGRAMS ARE LINKED TOGETHER, SUCH THAT ONE PROGRAM IS
C CALLED, USES THE TDDB, STOPS, AND ANOTHER PROGRAM IS CALLED WITHOUT
C STOPPING THE ORIGINAL PROGRAM, IRETCD SHOULD BE SET TO 999 FOR EACH FIRST
C CALL TO DADIO, SO THAT THE FLAGS BELOW ARE SET TO THE CORRECT VALUES.
C
      IF (IRETCD.EQ.999) THEN
        IRETCD = 0
        Opend = .FALSE.
        Sorted = .FALSE.
        Ltrkno = 0
        Apydtm = .TRUE.
C
C Open the TDDB and DSR file
C
      ELSE
        CALL OPENDB(Tddata,Dsref,Printr,TDDBNM,DSRNAM,IRETCD)
        IF (IRETCD.NE.0) GOTO 20
      ENDIF
C
C Produce conventions summary
C
      IF (listab.EQ.2) THEN
        CALL DADSUM(listab,IRETCD)
        GOTO 20
      ENDIF
C
      DO 10 i = 1, NELEHF
        Dati4(i) = 0
   10 CONTINUE
C
C CHECK IF FILE HAS BEEN OPENED - IF NOT, READ IN INDEX.
C
      IF (.NOT.Opend) THEN
        Opend = .TRUE.
        CALL RDINDX(Tddata,IRETCD)
        IF (IRETCD.EQ.-5) THEN
C
C NEW FILE - INITIALIZE INDEX
C
          IF (listab.LT.3) GOTO 30
          IRETCD = 0
          CALL INTNDX()
C
C WRITE INITIALIZED DATA TO FILE
C
          CALL WRTNDX(Tddata,Printr,IRETCD)
          IF (IRETCD.NE.0) GOTO 30
          WRITE (Printr,9005) Versn(:31), MAXENT, MAXFLD, MAXDYS, 
     &                        MAXYRS, LENREC, MAXREC, LENIND
        ENDIF
        IF (IRETCD.NE.0) GOTO 30
C
C allocate sets listab to 999 so that the dsr file is not read
C
        IF (listab.EQ.999) GOTO 20
        Nexent = Noent + 1
        Ptime(Nexent) = Pdate
C
C READ REQUIRED DATA FROM THE DATA STATION REFERENCE FILE
C
        CALL RDDSRF(Dsref,Printr,IRETCD)
        IF (IRETCD.NE.0) GOTO 20
      ENDIF
C
C FILE IS NOW OPEN - CHECK IF LIST OPTION REQUESTED
C
C remove flag to allow creation of new data base for dafile, backup
      IF (listab.GT.2) listab = listab - 5
      IF (listab.NE.0) THEN
        IF (Noent.EQ.0) THEN
          IRETCD = -5
          GOTO 30
        ENDIF
        CALL DADSUM(listab,IRETCD)
      ENDIF
   20 RETURN
   30 CALL ERRDAD(IRETCD,Printr)
      GOTO 20
C
C FORMATS
C
 9005 FORMAT (/,' A TDDB was created or initialized by ',A,/,
     &' with the following attributes as defined in include file dimpar.
     &cmn:',//' MAXENT =',I6,' Maximum number of index entries',/,
     &' MAXFLD =',I6,' Maximum number of station in DSR file',/,
     &' MAXDYS =',I6,' Maximum number of days per data set',/,
     &' MAXYRS =',I6,' Maximum number of years per TDDB',/,' LENREC =',
     &I6,' Length of each record in the TDDB, in bytes',/,' MAXREC =',
     &I6,' TDDB record length based on Fortran I/O',/,' LENIND =',I6,
     &' Length of each index entry, in bytes',/)
      END
C
C
C
      SUBROUTINE DADSUM
     I                 ( LISTN,
     O                   IRETCD )
C
C     + + + PURPOSE + + +
C     Print TDDB index summaries
C
C     + + + HISTORY + + +
C     MODIFIED January 20, 1995 -- RSR
C       removed from DADIO
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER LISTN, IRETCD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LISTN  - Summary list option (-1=calendar, 0=none, 1=directory,
C              2=conventions summary)
C     IRETCD - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + INCLUDES + + +
      INCLUDE 'dadblk.cmn'
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
      INCLUDE 'datyps.cmn'
      INCLUDE 'dastyp.cmn'
      INCLUDE 'dstrf.cmn'
C
C     + + + COMMONS + + +
      CHARACTER*60 Versn
      COMMON /VERSION/ Versn
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, j, k, noy, kk, py(MAXYRS), idmy, pyr, ibelt, ieelt,
     &        rdpday, isub, isub15, ioff, iexp, irds, lrdpdy, ixl, ixm, 
     &        iertst, eltiby, m, mm, mrds, dperm(12), ierval, iby, ibmo, 
     &        ibd, ibh, ibm, iey, iemo, ied, ieh, iem, bjdayn, ejdayn, 
     &        errors(15), ipy, ipmo, ipd, iph, ipm, listab, jj, mmm
      CHARACTER*4 irdcod, rv(366)
      CHARACTER cfldst*16
      LOGICAL nodata
C
C     + + + EXTERNALS + + +
      INTEGER DAYFEB, MNYEAR
      EXTERNAL DTCODE, STPASS, SORTIN, SORTYR, DECODE, DAYFLG, ERRDAD, 
     &         INCELT, DAYFEB, MNYEAR
C
C     + + + INTRINSICS + + +
      INTRINSIC MOD
C
C     + + + SAVE VARIABLES + + +
      INTEGER mxln
      CHARACTER*4 ichar(858), linseg(2), month(24), ibar, symbol(6), 
     &            ircode(NORPD), yesno(2)
      SAVE dperm, ichar, mxln, yesno, ircode, month, linseg, symbol, 
     &     ibar
C
C     + + + DATA + + +
      DATA dperm/31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
      DATA ichar/' MON', 'TH  ', ' 1  ', ' 2  ', ' 3  ', ' 4  ', ' 5  ', 
     &     ' 6  ', ' 7  ', ' 8  ', ' 9  ', ' 10 ', ' 11 ', ' 12 ', 
     &     ' 13 ', ' 14 ', ' 15 ', ' 16 ', ' 17 ', ' 18 ', ' 19 ', 
     &     ' 20 ', ' 21 ', ' 22 ', ' 23 ', ' 24 ', ' 25 ', ' 26 ', 
     &     ' 27 ', ' 28 ', ' 29 ', ' 30 ', ' 31 ', '----', 32*'---+', 
     &     792*'    '/
      DATA mxln/55/, ibar/'   |'/, yesno/'YES ', ' NO '/
      DATA ircode/' Z |', ' Y |', ' X |', ' W |', ' V |', ' U |', 
     &     ' T |', ' S |', ' R |', ' Q |', ' P |'/
      DATA linseg/'----', '---+'/, symbol/'   |', ' * |', ' # |', 
     &     ' % |', ' $ |', ' - |'/
      DATA month/' JAN', '.  |', ' FEB', '.  |', ' MAR', 'CH |', ' APR', 
     &     'IL |', ' MAY', '   |', ' JUN', 'E  |', ' JUL', 'Y  |', 
     &     ' AUG', '.  |', ' SEP', 'T. |', ' OCT', '.  |', ' NOV', 
     &     '.  |', ' DEC', '.  |'/
C
C     + + + END SPECIFICATIONS + + +
C
      IRETCD = 0
      listab = LISTN
      IF (listab.EQ.2) THEN
        WRITE (Printr,9045) Stypes, Rpday
        WRITE (Printr,9050) (Types(i),Ctyps(i),i=1,NOTYPS)
        GOTO 190
      ENDIF
      iby = Iyr
      ibmo = Imo
      ibd = Ida
      ibh = Ihr
      ibm = Imn
C
C FILE IS NOW OPEN - CHECK IF LIST OPTION REQUESTED
C
C remove flag to allow creation of new data base for dafile, backup
      IF (listab.GT.2) listab = listab - 5
      IF (listab.EQ.0) GOTO 190
      IF (Noent.EQ.0) THEN
        IRETCD = -5
        GOTO 200
      ENDIF
C
C LIST OPTION REQUESTED --
C -1 : PRINT BOTH CALENDAR-YEAR AND DIRECTORY SUMMARIES
C  0 : DO NOT PRINT INDEX SUMMARIES
C +1 : PRINT ONLY THE DIRECTORY SUMMARY
C
C SET UP THE 'PASS' ARRAY FOR THIS DATA STATION REFERENCE
      CALL STPASS()
C
      CALL SORTIN
      Sorted = .TRUE.
      IF (listab.LT.1) THEN
C
C FIND AND SORT ALL YEARS COVERED BY DATA ON FILE
C
        CALL SORTYR(Printr,py,noy,IRETCD)
        IF (IRETCD.EQ.-2) GOTO 190
C
C INITIALIZE THE CALENDAR-YEAR SUMMARY OUTPUT BUFFER
C
        DO 20 m = 1, 2
          DO 10 k = 2, 24, 2
            ichar(m+33*k) = month(m+k-2)
            ichar(m+33*(k+1)) = linseg(m)
   10     CONTINUE
   20   CONTINUE
        lrdpdy = 0
C
C GENERATE CALENDAR-YEAR SUMMARIES FOR EACH DATA TYPE, FOR EACH
C YEAR, FOR EACH FIELD STATION.
C
        DO 160 i = 1, Istk
          cfldst = Fldsta(i)
          DO 150 j = 1, noy
            pyr = py(j)
            dperm(2) = DAYFEB(pyr)
            DO 140 kk = 1, NOTYPS
              DO 130 jj = 1, 4
                nodata = .TRUE.
                DO 80 k = 1, Noent
                  IF (Pass(k) .OR. Statno(k).NE.cfldst .OR. Dtypei(k)
     &               .NE.Types(kk) .OR. Stypei(k).NE.Stypes(jj)) GOTO 80
                  CALL DECODE(Btime(k),iby,ibmo,ibd,ibh,ibm,bjdayn,
     &                        ibelt)
                  CALL DECODE(Etime(k),iey,iemo,ied,ieh,iem,ejdayn,
     &                        ieelt)
                  IF (iby.EQ.pyr .OR. iey.EQ.pyr) THEN
                    IF (nodata) THEN
                      DO 30 m = 1, 366
                        rv(m) = ibar
   30                 CONTINUE
                    ENDIF
                    nodata = .FALSE.
                    rdpday = Rperd(k)
                    IF (lrdpdy.EQ.0) lrdpdy = rdpday
                    Irdint = Rdint(k)
                    DO 70 m = 1, NORPD
                      IF (rdpday.NE.Rpday(m)) GOTO 70
                      irdcod = ircode(m)
                      isub = 0
                      isub15 = 0
                      ioff = 1
                      IF (iby.NE.iey) THEN
                        eltiby = MNYEAR(iby)
                        IF (iby.NE.pyr) THEN
                          ioff = (eltiby-ibelt)/1440 + 1
                          ibelt = Irdint
                          bjdayn = 1
                        ELSE
                          ejdayn = eltiby/1440
                          ieelt = eltiby
                        ENDIF
                      ENDIF
                      DO 60 mmm = bjdayn, ejdayn
                        IF (isub.EQ.0 .OR. MOD(ioff+14,15).EQ.0) THEN
                          isub = (ioff+14)/15
                          isub15 = (isub-1)*15
                          DO 40 mm = 1, 15
                            errors(mm) = 0
   40                     CONTINUE
                          ierval = Erflag(isub,mmm)
                          iexp = 14
   50                     iertst = 2**iexp
                          IF (ierval.GE.iertst) THEN
                            errors(iexp+1) = 1
                            ierval = ierval - iertst
                          ENDIF
                          iexp = iexp - 1
                          IF (iexp.GE.0) GOTO 50
                        ENDIF
                        idmy = mmm*1440
                        IF (ieelt.LT.idmy) idmy = ieelt
                        irds = (idmy-ibelt)/Irdint + 1
                        ibelt = ibelt + irds*Irdint
C Determine if data are partial, marked, complete, flagged, dual density
                        CALL DAYFLG(irds,rdpday,lrdpdy,
     &                              errors(ioff-isub15),irdcod,symbol,
     &                              rv(mmm))
                        ioff = ioff + 1
   60                 CONTINUE
                      lrdpdy = rdpday
                      GOTO 80
   70               CONTINUE
                    IRETCD = -9
                    GOTO 200
                  ENDIF
   80           CONTINUE
                IF (.NOT.nodata) THEN
                  DO 90 k = 1, 366
                    READ (rv(k),'(I4)',ERR=90) mrds
C PARTIAL DAY, NO FLAGS
                    rv(k) = symbol(2)
C PARTIAL DAY, FLAGS
                    IF (mrds.LT.0) rv(k) = symbol(4)
   90             CONTINUE
                  mm = 1
                  DO 120 k = 1, 12
                    ixl = 2 + 66*k
                    ixm = ixl + 33
                    DO 100 m = 1, dperm(k)
                      ichar(ixl+m) = rv(mm)
                      ichar(ixm+m) = linseg(2)
                      mm = mm + 1
  100               CONTINUE
                    idmy = 31 - dperm(k)
                    DO 110 m = 1, idmy
                      ichar(ixl+dperm(k)+m) = symbol(6)
                      ichar(ixm+dperm(k)+m) = linseg(2)
  110               CONTINUE
  120             CONTINUE
                  WRITE (Printr,9030)
                  WRITE (Printr,9035) Versn(:31), Iyr, Imo, Ida, Ihr, 
     &                                Imn
                  WRITE (Printr,9040) Ctyps(kk), Stypes(jj), pyr, 
     &                                cfldst, ichar
                ENDIF
  130         CONTINUE
  140       CONTINUE
  150     CONTINUE
  160   CONTINUE
      ENDIF
C
C GENERATE THE DIRECTORY SUMMARY
C
      WRITE (Printr,9005)
      WRITE (Printr,9035) versn(:31), Iyr, Imo, Ida, Ihr, Imn
      WRITE (Printr,9010)
      mm = 10
      DO 180 i = 1, Noent
        IF (Pass(i)) GOTO 180
        mm = mm + 1
        CALL DECODE(Btime(i),iby,ibmo,ibd,ibh,ibm,bjdayn,ibelt)
        CALL DECODE(Etime(i),iey,iemo,ied,ieh,iem,ejdayn,ieelt)
        CALL DECODE(Ptime(i),ipy,ipmo,ipd,iph,ipm,ejdayn,idmy)
        IF (iby.LT.iey) CALL INCELT(iby,iey,ieelt)
        irds = (ieelt-ibelt)/Rdint(i) + 1
        m = 2
        DO 170 j = 1, 6
          IF (Erflag(j,i).NE.0) m = 1
  170   CONTINUE
        WRITE (Printr,9015) i, Statno(i), Dtypei(i), iby, ibmo, ibd, 
     &                      ibh, ibm, iey, iemo, ied, ieh, iem, 
     &                      Rperd(i), irds, Stypei(i), yesno(m), ipy, 
     &                      ipmo, ipd, iph, ipm, Trkno(i), Bytno(i)
        IF (mm.GE.mxln .AND. i.NE.Noent) THEN
          WRITE (Printr,9020)
          WRITE (Printr,9010)
          mm = 6
        ENDIF
  180 CONTINUE
      WRITE (Printr,9025)
  190 RETURN
  200 CALL ERRDAD(IRETCD,Printr)
      GOTO 190
C
C FORMATS
C
 9005 FORMAT ('1',/,T34,'INDEX SUMMARY OF DATA SETS STORED IN THE TDDB')
 9010 FORMAT (1X,113('='),/,'  DATA|     STATION    |PARM|',
     &        ' START OF ENTRY |  END OF ENTRY  |RDS/|NO OF|STOR|',
     &        'DATA| PROCESSING TIME|FILE ADDRESS',/,
     &        '  SET |   IDENTIFIER   |CODE| ',2('YR MO DY HR MN | '),
     &        'DAY| RDS |TYPE|FLAG| YR MO DY HR MN |RECORD  BYTE',/,1X,
     &        113('='))
 9015 FORMAT (I5,' |',A16,'| ',A2,2(' |',I3.2,2('/',I2.2),I3.2,':',I2.2)
     &        ,' |',I4,'|',I5,'| ',A2,' |',A3,' |',I3.2,2('/',I2.2),
     &        I3.2,':',I2.2,' |',I4,' |',I6)
 9020 FORMAT ('1')
 9025 FORMAT (1X,113('='),//)
 9030 FORMAT ('1',/,T32,
     &        'CALENDAR-YEAR SUMMARY OF DATA STORED IN THE TDDB')
 9035 FORMAT (T34,'PROCESSED BY ',A,/,T37,'PROCESSING DATE AND TIME',
     &        I3.2,2('/',I2.2),I3.2,':',I2.2,/)
 9040 FORMAT (/,T51,A,'(',A2,')',/,' CALENDAR YEAR  -  19',I2.2,T102,
     &        'STATION ID. ',A16,/,T65,'DAY',/,26(1X,32A4,A3/),/,T64,
     &        'CODES',/,
     & ' * = PARTIAL RECORD;  % = PARTIAL RECORD,CONTAINS FLAGGED DATA;'
     & ,'  # = DUAL DENSITY RECORD;',
     & '  $ = COMPLETE RECORD,CONTAINS FLAGGED DATA',/,11X,
     &'P = 1440 READINGS/DAY; Q = 720 READINGS/DAY; R = 288 READINGS/DAY
     &; S = 240 READINGS/DAY; T = 144 READINGS/DAY;',/,1X,
     &'U =   96 READINGS/DAY; V =  48 READINGS/DAY; W =  24 READINGS/DAY
     &; X =  12 READINGS/DAY; Y =   8 READINGS/DAY; Z =   1 READINGS/DAY
     &')
 9045 FORMAT (6X,'Valid values associated with data stored in a TDDB',
     &        //' Storage-types:',/,4X,'Code',8X,'Description',/,3X,A4,
     &        5X,'two-byte integer',/,3X,A4,5X,'four-byte integer',/,3X,
     &        A4,5X,'four-byte floating-point',/,3X,A4,5X,
     &        'eight-byte floating-point',//,
     &        ' Recording densities (readings per day):',/,3X,2I3,4I4,
     &        5I5,//,'Data-parameters:',/,'   Parameter',7X,
     &        'Description',11X,'Parameter',7X,'Description')
 9050 FORMAT (2(5X,A2,6X,A26))
      END
C
C
C
      SUBROUTINE DADO
     I               ( STANUM, TYPE, STYP, BY, BMO, BD, BH, BMN, EY, 
     I                 EMO, ED, EH, EMN, RDPDY,
     O                 IDATA,
     I                 STRIP, PRTMSG,
     O                 IRETCD )
C
C     + + + PURPOSE + + +
C     Retrieve data from a TDDB
C     RESTRICTION: Data may not span a recording density change
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STANUM*16, TYPE*2, STYP*2
      INTEGER BY, BMO, BD, BH, BMN, EY, EMO, ED, EH, EMN, RDPDY, STRIP, 
     &        IRETCD
      LOGICAL PRTMSG
      INTEGER*2 IDATA(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STANUM - Station identifier
C     TYPE   - Parameter type
C     STYP   - Data storage type
C     BY     - Beginning year of request
C     BMO    - Beginning month
C     BD     - Beginning day
C     BH     - Beginning hour
C     BMN    - Beginining minute
C     EY     - Ending year
C     EMO    - Ending month
C     ED     - Ending day
C     EH     - Ending hour
C     EMN    - Ending minute
C     RDPDY  - Number of readings per day 
C     IDATA  - Array that will contain retrieved data (must be dimensioned
C              appropriately by calling routine
C     STRIP  - Flag to signal what data status codes are to be stripped
C     PRTMSG - Flag to allow print of DADIO warning messages
C     IRETCD - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER*2 iblk2
      INTEGER kele, iele, jele, itrkno, ibelt, ieelt, dt, nexrec, nsub,
     &        i, j, ibytno, iby, ibmo, ibd, ibh, ibm, iey, iemo, ied,
     &        ieh, iem, bjdayn, ejdayn, bysav, eysav, beltsv, eeltsv,
     &        icuret
      LOGICAL nodata, error, flags, prtref
C
C     + + + INCLUDES + + +
      INCLUDE 'dadblk.cmn'
      INCLUDE 'datdio.cmn'
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
C
C     + + + EXTERNALS + + +
      EXTERNAL DADCHK, DECODE, ERRDAD, STRPI2, INCELT, PRTDDF
C
C     + + + DATA + + +
      DATA iblk2/-32768/
C     USE SMALLEST POSSIBLE I*2 AS NO VALUE MARKER, USED TO BE INTEGER
C     VALUE OF A BLANK.
C
C     + + + END SPECIFICATIONS + + +
C
      prtref = .TRUE.
      IF (Noent.EQ.0) THEN
        IRETCD = -5
        GOTO 120
      ENDIF
      CALL DADCHK(STANUM,TYPE,STYP,BY,BMO,BD,BH,BMN,EY,EMO,ED,EH,EMN,
     &            RDPDY,PRTMSG,IRETCD)
      IF (IRETCD.NE.0) GOTO 130
      nodata = .TRUE.
C
C SET UP DATA BUFFER OF ARGUMENT LIST - INITIALIZE ERROR FLAGS.
C
      kele = ((Eelt-Belt)/Irdint+1)*Nstep
      DO 10 i = 1, kele
        IDATA(i) = iblk2
   10 CONTINUE
      error = .FALSE.
      flags = .FALSE.
      bysav = BY
      eysav = EY
      beltsv = Belt
      eeltsv = Eelt
      icuret = Belt
      nsub = 1
C
C SEQUENCE THROUGH THE INDEX TO RETRIEVE THE REQUESTED DATA.
C
      DO 70 i = 1, Noent
        IF (STANUM.NE.Statno(i) .OR. TYPE.NE.Dtypei(i) .OR. 
     &      STYP.NE.Stypei(i) .OR. Et.LT.Btime(i) .OR. Bt.GT.Etime(i))
     &      GOTO 70
        IF (Rperd(i).NE.RDPDY) THEN
          IRETCD = 6
          GOTO 120
        ENDIF
C
C INDEX ENTRY #I CONTAINS AT LEAST PART OF REQUESTED DATA.
C
        itrkno = Trkno(i)
        ibytno = Bytno(i)
        nodata = .FALSE.
        CALL DECODE(Btime(i),iby,ibmo,ibd,ibh,ibm,bjdayn,ibelt)
        CALL DECODE(Etime(i),iey,iemo,ied,ieh,iem,ejdayn,ieelt)
        IF (BY.LT.iby) THEN
          CALL INCELT(BY,iby,ibelt)
          CALL INCELT(BY,iey,ieelt)
        ELSE
          IF (iby.NE.iey) CALL INCELT(iby,iey,ieelt)
          IF (BY.GT.iby) THEN
            CALL INCELT(iby,BY,Belt)
C           already incremented Eelt from Ey-By
            CALL INCELT(iby,BY,Eelt)
          ENDIF
        ENDIF
C
C READ DATA FROM D/A FILE - COMPUTE BUFFER SUBSCRIPTS NEEDED TO
C OBTAIN THE REQUESTED DATA STORED ON THIS TRACK.
C
   20   READ (Tddata,REC=itrkno) Dati4
        nexrec = itrkno + 1
        Ltrkno = itrkno
        iele = (ibytno+1)/2
        IF (icuret.LE.ibelt) GOTO 40
        iele = iele + ((icuret-ibelt)/Irdint)*Nstep
        IF (iele.LE.NELE) GOTO 40
   30   ibelt = ibelt + (NELE-(ibytno-1)/2)/Nstep*Irdint
        ibytno = 1
        itrkno = nexrec
        GOTO 20
   40   jele = (ibytno+1)/2 + ((ieelt-ibelt)/Irdint+1)*Nstep - 1
        IF (Eelt.LT.ieelt) jele = jele - (ieelt-Eelt)/Irdint*Nstep
        IF (jele.GT.NELE) jele = NELE
C
C PROCESS DATA ACCORDING TO STORAGE TYPE
C
C INTEGER*2 STAGE DATA
C
        IF (TYPE.EQ.' Z' .AND. STYP.EQ.'I2') THEN
          DO 50 j = iele, jele
            dt = dati2(j)
            IF (dt.GT.9999 .OR. dt.LT.-999) THEN
C 21001 = 22000 + (-999) = smallest value plus offset
              IF (dt.LT.21001) THEN
                error = .TRUE.
              ELSE
C dt >= 22000 (update flag only)
                flags = .TRUE.
              ENDIF
C Strip flags if Integer*2 data
              IF (STRIP.NE.0) CALL STRPI2(STRIP,dt)
            ENDIF
            IDATA(nsub) = dt + Izcorr
            nsub = nsub + 1
   50     CONTINUE
          IF (prtref .AND. Izcorr.NE.0 .AND. TYPE.EQ.' Z') THEN
            WRITE (Printr,9005) STANUM, Izcorr/100.0
            prtref = .FALSE.
          ENDIF
        ELSE
C
C All other data (I4,R4,R8) can be put in IDATA due to equivalence
C relationship
C
          DO 60 j = iele, jele
            IDATA(nsub) = dati2(j)
            nsub = nsub + 1
   60     CONTINUE
        ENDIF
C
C END OF PROCESSING LOOP
C
        icuret = icuret + ((jele-iele+1)/Nstep)*Irdint
        IF (jele.GE.NELE) THEN
C found data see if it was all there
          IF (icuret.GT.Eelt .OR. icuret.GT.ieelt) GOTO 80
          GOTO 30
        ENDIF
        BY = bysav
        EY = eysav
        Belt = beltsv
        Eelt = eeltsv
   70 CONTINUE
C
C ALL RELATED INDEX ENTRIES HAVE BEEN PROCESSED - CHECK IF ALL DATA
C REQUESTED WERE RETRIEVED AND SET THE CORRECT RETURN CODE.
C
   80 IF (nodata) THEN
        IRETCD = 7
        GOTO 120
      ENDIF
      IF (Nstep.EQ.1) THEN
        DO 90 j = 1, kele
          IF (IDATA(j).EQ.iblk2) THEN
            IRETCD = 11
            GOTO 110
          ENDIF
   90   CONTINUE
      ELSE
        DO 100 j = 1, kele-1
          IF (IDATA(j).EQ.iblk2 .AND. IDATA(j+1).EQ.iblk2) THEN
            IRETCD = 11
            GOTO 110
          ENDIF
  100   CONTINUE
      ENDIF
  110 IF (.NOT.error) THEN
        IF (.NOT.flags) THEN
          IF (IRETCD.NE.11) GOTO 130
          GOTO 120
        ENDIF
        IF (IRETCD.NE.11) THEN
          IRETCD = 10
        ELSE
          IRETCD = 12
        ENDIF
      ELSEIF (IRETCD.NE.11) THEN
        IRETCD = 4
        IF (flags) IRETCD = 17
      ELSE
        IRETCD = 13
        IF (flags) IRETCD = 18
      ENDIF
C
C ERROR : PRINT ARGUMENT LIST IF ERROR IRETCD ENCOUNTERED.
C
  120 IF (PRTMSG) THEN
        CALL ERRDAD(IRETCD,Printr)
        CALL PRTDDF(STANUM,TYPE,STYP,BY,BMO,BD,BH,BMN,EY,EMO,ED,EH,EMN,
     &              RDPDY,Printr)
      ENDIF
  130 RETURN
 9005 FORMAT (/,' DADIO message: Reference adjustment added to retrieved
     & values for station ',A,' is',F10.2,/)
      END
C
C
C
      SUBROUTINE DADI
     I               ( STANUM, TYPE, STYP, BY, BMO, BD, BH, BMN, EY, 
     I                 EMO, ED, EH, EMN, RDPDY, IDATA, STRIP, PRTMSG,
     O                 IRETCD )
C
C     + + + PURPOSE + + +
C     Store data in a TDDB
C     RESTRICTION: Data may not span a recording density change
C     **update to allow any time period to be stored (multiple data sets)**
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STANUM*16, TYPE*2, STYP*2
      INTEGER BY, BMO, BD, BH, BMN, EY, EMO, ED, EH, EMN, RDPDY, IRETCD, 
     &        STRIP
      LOGICAL PRTMSG
      INTEGER*2 IDATA(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STANUM - Station identifier
C     TYPE   - Parameter type
C     STYP   - Data storage type
C     BY     - Beginning year of request
C     BMO    - Beginning month
C     BD     - Beginning day
C     BH     - Beginning hour
C     BMN    - Beginining minute
C     EY     - Ending year
C     EMO    - Ending month
C     ED     - Ending day
C     EH     - Ending hour
C     EMN    - Ending minute
C     RDPDY  - Number of readings per day
C     IDATA  - Array that contains data to be stored (must be dimensioned
C              appropriately by calling routine
C     STRIP  - Dummy argument, used so that arg list for dadi & dado match
C     PRTMSG - Flag to allow print of DADIO warning messages
C     IRETCD - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + INCLUDES + + +
      INCLUDE 'dadblk.cmn'
      INCLUDE 'datdio.cmn'
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, j, ltent, nxent, nbyte, lbyte, ibyte, itrkno, lele, 
     &        ibelt, ieelt, lstep, kele, imod, ll, iele, jele, n, ioff, 
     &        nexrec, iby, ibmo, ibd, ibh, ibm, iey, iemo, ied, ieh, 
     &        iem, bjdayn, ejdayn, iret, nentrk, nintrk, ierval, isub,
     &        iexp
      LOGICAL update, error
      INTEGER*2 iblk2, i2one, i2dt
C
C     + + + EXTERNALS + + +
      INTEGER DAYNYR
      EXTERNAL DADCHK, ERRDAD, WRTNDX, SETNDX, DECODE, CHKTYP, CHKSTP, 
     &         GETSTP, INCELT, DAYNYR, PRTDDF
C
C     + + + INTRINSICS + + +
      INTRINSIC MOD
C
C     + + + SAVE VARIABLES + + +
      CHARACTER blk2*2
      SAVE blk2
C
C     + + + DATA + + +
      DATA blk2/'  '/, i2one/1/
C
C     + + + END SPECIFICATIONS + + +
C
      READ (blk2,'(A2)') iblk2
      IF (Noent.GE.MAXENT) THEN
        IRETCD = -1
        GOTO 110
      ENDIF
      CALL DADCHK(STANUM,TYPE,STYP,BY,BMO,BD,BH,BMN,EY,EMO,ED,EH,EMN,
     &            RDPDY,PRTMSG,IRETCD)
      IF (IRETCD.NE.0) GOTO 120
C
      ltent = Noent
      nxent = ltent + 1
      update = .FALSE.
C
C WRITE INDEX TO THE DATA BASE BEFORE STORING NEW DATA IN THE FILE IF
C THE INDEX WAS SORTED IN CORE FOR PRINTOUT PURPOSES
C
      IF (Sorted) THEN
        Sorted = .FALSE.
        Ptime(Nexent) = 0
        CALL WRTNDX(Tddata,Printr,IRETCD)
        IF (IRETCD.NE.0) GOTO 120
      ENDIF
      IF (ltent.NE.0) THEN
C
C CHECK FOR UPDATE PROCESSING:
C
        IF (Btime(ltent).NE.Bt .OR. Etime(ltent).NE.Et .OR. Ptime(ltent)
     &      .NE.Bt) THEN
C
C CHECK FOR PREVIOUSLY STORED DATA FOR GIVEN TIME SPAN.
C
          DO 10 i = 1, Noent
            IF (STANUM.EQ.Statno(i) .AND. TYPE.EQ.Dtypei(i) .AND. 
     &          STYP.EQ.Stypei(i) .AND. Et.GE.Btime(i) .AND. 
     &          Bt.LE.Etime(i)) THEN
              IRETCD = 8
              GOTO 110
            ENDIF
   10     CONTINUE
C
C FIND THE INDEX ENTRY (LTENT) WHICH WAS STORED LAST ON THE FILE -
C AND THUS FIND THE LAST TRACK USED AND THE NUMBER OF BYTES USED.
C
          IF (ltent.NE.1) THEN
            ltent = 1
            lbyte = Trkno(1)*LENREC + Bytno(1)
            DO 20 i = 2, Noent
              ibyte = Trkno(i)*LENREC + Bytno(i)
              IF (ibyte.GE.lbyte) THEN
                ltent = i
                lbyte = ibyte
              ENDIF
   20       CONTINUE
          ENDIF
          GOTO 50
        ELSE
C
          update = .TRUE.
          nxent = ltent
          nbyte = Trkno(nxent)*LENREC + Bytno(nxent)
          lbyte = 0
          ltent = 0
          DO 30 i = 1, MAXENT
            ibyte = Trkno(i)*LENREC + Bytno(i)
            IF (ibyte.EQ.0) GOTO 40
            IF (ibyte.LT.nbyte .AND. ibyte.GT.lbyte) THEN
              ltent = i
              lbyte = ibyte
            ENDIF
   30     CONTINUE
   40     IF (ltent.NE.0) GOTO 50
        ENDIF
      ENDIF
C
C NO DATA SETS ARE CURRENTLY STORED - THUS, THIS IS FIRST ENTRY.
C
      CALL SETNDX(nentrk,nintrk,i,j)
      Trkno(1) = nintrk + 1
      Bytno(1) = i2one
      itrkno = nintrk + 1
      lele = 0
C
C CHECK FOR TIME SPAN GREATER THAN DEFINED LIMIT.
C
   50 IF (.NOT.update .AND. Nexent.GT.Noent) Ptime(Nexent) = 0
      CALL DECODE(Bt,iby,ibmo,ibd,ibh,ibm,bjdayn,ibelt)
      CALL DECODE(Et,iey,iemo,ied,ieh,iem,ejdayn,ieelt)
      IF (iby.LT.iey) ejdayn = ejdayn + DAYNYR(iby)
      n = ejdayn - bjdayn + 1
      IF (n.GT.MAXDYS) THEN
        IRETCD = 9
        GOTO 110
      ENDIF
C
C CALCULATE PERTINENT PARAMETERS FOR THE 'LTENT' INDEX ENTRY.
C
      IF (ltent.NE.0) THEN
        CALL CHKTYP(Dtypei(ltent),IRETCD)
        CALL CHKSTP(STYP,iret)
        IF (iret.NE.0 .AND. IRETCD.NE.0) IRETCD = 16
        IF (iret.NE.0 .AND. IRETCD.NE.16) IRETCD = iret
        IF (IRETCD.NE.0) GOTO 110
        CALL GETSTP(Stypei(ltent),lstep)
        CALL DECODE(Btime(ltent),iby,ibmo,ibd,ibh,ibm,bjdayn,ibelt)
        CALL DECODE(Etime(ltent),iey,iemo,ied,ieh,iem,ejdayn,ieelt)
        IF (iby.LT.iey) CALL INCELT(iby,iey,ieelt)
        IF (.NOT.update) THEN
C
C FIND TRACK AND BYTE NUMBER WHERE DATA IS TO BE STORED.
C
          kele = ((ieelt-ibelt)/Rdint(ltent)+1)*lstep + Bytno(ltent)/2
          Trkno(nxent) = Trkno(ltent) + kele/NELE
          lele = MOD(kele,NELE)
          imod = MOD((NELE-lele),Nstep)
          IF (imod.NE.0) THEN
            kele = ((Eelt-Belt)/Irdint+1)*Nstep + lele
            IF (kele.GT.NELE) THEN
              ll = lele + 1
              n = ll + imod - 1
              DO 60 i = ll, n
                dati2(i) = iblk2
   60         CONTINUE
              lele = lele + imod
              IF ((lele+Nstep).GT.NELE) THEN
                Trkno(nxent) = Trkno(nxent) + i2one
                lele = 0
              ENDIF
            ENDIF
          ENDIF
C
          Bytno(nxent) = lele*2 + 1
          itrkno = Trkno(nxent)
          IF (lele.EQ.0 .OR. itrkno.EQ.Ltrkno) GOTO 70
        ELSE
          itrkno = Trkno(nxent)
          lele = Bytno(nxent)/2
        ENDIF
        READ (Tddata,REC=itrkno) Dati4
        nexrec = itrkno + 1
      ENDIF
C
C PLACE DATA ON FILE
C
   70 iele = lele + 1
      ioff = (RDPDY-(BH*60+BMN)/Irdint) + 1
      kele = ((Eelt-Belt)/Irdint+1)*Nstep + lele
      j = 1
      ierval = 0
      error = .FALSE.
      isub = 1
      iexp = 0
C
   80 jele = kele
      IF (jele.GT.NELE) jele = NELE
C
C PROCESS DATA ACCORDING TO DATA TYPE
C
C INTEGER*2 DATA STAGE DATA
      IF (TYPE.EQ.' Z' .AND. STYP.EQ.'I2') THEN
        DO 85 i = iele, jele
          i2dt = IDATA(j)
C if >22000 only an update flag has been set (used to not set flag
C if only an update flag)
          IF (i2dt.LT.-999 .OR. i2dt.GT.9999) error = .TRUE.
          dati2(i) = i2dt
          IF (j.EQ.ioff) THEN
            ioff = ioff + rdpdy
            IF (error) ierval = ierval + 2**iexp
            error = .FALSE.
            iexp = iexp + 1
            IF (iexp.GT.14) THEN
              Erflag(isub,nxent) = ierval
              isub = isub + 1
              iexp = 0
              ierval = 0
            ENDIF
          ENDIF
          j = j + 1
   85   CONTINUE
      ELSE
C
C INTEGER*2, INTEGER*4, REAL*4, and REAL*8 data put in dati2 array
        DO 90 i = iele, jele
          dati2(i) = IDATA(j)
          j = j + 1
   90   CONTINUE
      ENDIF
C
C CHECK IF NEXT TRACK IS TO BE USED WITH THIS DATA SET.
C
      IF (jele.NE.NELE .AND. .NOT.update) THEN
        ll = jele + 1
        DO 100 i = ll, NELE
          dati2(i) = iblk2
  100   CONTINUE
      ENDIF
C
C WRITE TRACK WITH NEW DATA BACK ONTO FILE.
C
      WRITE (Tddata,REC=itrkno) Dati4
      nexrec = itrkno + 1
      IF (jele.GE.NELE) THEN
        kele = kele - NELE
        IF (kele.NE.0) THEN
          iele = 1
          itrkno = nexrec
          IF (update .AND. kele.LT.NELE) THEN
            READ (Tddata,REC=itrkno) Dati4
            nexrec = itrkno + 1
          ENDIF
          GOTO 80
        ENDIF
      ENDIF
C
C DATA STORAGE COMPLETED - ADD NEW INDEX ENTRY.
C
      Ltrkno = itrkno
      IF (error) ierval = ierval + 2**iexp
      IF (isub.LT.7) Erflag(isub,nxent) = ierval
      Statno(nxent) = STANUM
      Btime(nxent) = Bt
      Etime(nxent) = Et
      Ptime(nxent) = Pdate
      Rperd(nxent) = RDPDY
      Rdint(nxent) = Irdint
      Dtypei(nxent) = TYPE
      Stypei(nxent) = STYP
      Dummy(nxent) = blk2
      CALL WRTNDX(Tddata,Printr,IRETCD)
      IF (IRETCD.EQ.0) Noent = Noent + 1
      GOTO 120
C ERROR : PRINT ARGUMENT LIST IF ERROR IRETCD ENCOUNTERED.
  110 IF (PRTMSG) THEN
        CALL ERRDAD(IRETCD,Printr)
        CALL PRTDDF(STANUM,TYPE,STYP,BY,BMO,BD,BH,BMN,EY,EMO,ED,EH,EMN,
     &              RDPDY,Printr)
      ENDIF
  120 RETURN
      END
C
C
C
      SUBROUTINE OPENDB
     I                 ( TDDATA, DSREF, PRINTR, TDDNAM, DSRNAM,
     O                   IRETCD )
C
C     + + + PURPOSE + + +
C     Open the TDDB and DSR file to specified logical units.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER TDDATA, DSREF, PRINTR, IRETCD
      CHARACTER*48 TDDNAM, DSRNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     TDDATA - Unit number of TDDB
C     DSREF  - Unit number of DSR file
C     PRINTR - Unit number of printed output file
C     TDDNAM - File name of TDDB
C     DSRNAM - File name of DSR file
C     IRETCD - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ios
      LOGICAL exist
C
C     + + + EXTERNALS + + +
      EXTERNAL OPNRDR, ERRDAD
C
C     + + + END SPECIFICATIONS + + +
C
      CALL OPNRDR(DSREF,DSRNAM,0,IRETCD)
      IF (IRETCD.NE.0) THEN
        IRETCD = -13
        GOTO 20
      ELSE
        INQUIRE (FILE=TDDNAM,EXIST=exist)
        IF (.NOT.exist) THEN
          OPEN (TDDATA,FILE=TDDNAM,ACCESS='DIRECT',RECL=MAXREC,ERR=10,
     &          IOSTAT=ios,STATUS='NEW')
        ELSE
          OPEN (TDDATA,FILE=TDDNAM,ACCESS='DIRECT',RECL=MAXREC,ERR=10,
     &          IOSTAT=ios,STATUS='OLD')
        ENDIF
        IF (ios.EQ.0) GOTO 30
      ENDIF
   10 IRETCD = -12
   20 CALL ERRDAD(IRETCD,PRINTR)
   30 RETURN
      END
C
C
C
      SUBROUTINE LINDEX
     I                 ( TDDUNT, DSRUNT, PRTUNT,
     M                   ISTART, ISTOP,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     List the TDDB index in a brief format (up to 10 data sets per screen)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER TDDUNT, DSRUNT, PRTUNT, ISTART, ISTOP, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     TDDUNT - Unit number of TDDB
C     DSRUNT - Unit number of DSR file
C     PRTUNT - Unit number of printed output
C     ISTART - First data set to list
C     ISTOP  - Last data set to list
C     RETC   - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER istanm*16
      INTEGER size, ibelt, ieelt, i, noy, py(25), ln, by, bmo, bd, bh, 
     &        bmn, ey, emo, ed, eh, emn, ijd
      LOGICAL found
C
C     + + + INCLUDES + + +
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
C
C     + + + SAVE VARIABLES + + +
      INTEGER lastfd
      LOGICAL opend
      SAVE opend, lastfd
C
C     + + + EXTERNALS + + +
      EXTERNAL RDINDX, ERRDAD, RDDSRF, SORTYR, STPASS, SORTIN, DECODE, 
     &         INCELT
C
C     + + + DATA + + +
      DATA opend/.FALSE./, lastfd/0/
C
C     + + + END SPECIFICATIONS + + +
C
C CHECK IF FILE HAS BEEN OPENED - IF NOT, READ IN INDEX.
      IF (.NOT.opend) THEN
        RETC = 0
        opend = .TRUE.
        CALL RDINDX(TDDUNT,RETC)
        IF (RETC.LT.0) CALL ERRDAD(RETC,PRTUNT)
        IF (RETC.EQ.-10) GOTO 30
C READ REQUIRED DATA FROM THE DATA STATION REFERENCE FILE
        IF (RETC.NE.-5) CALL RDDSRF(DSRUNT,PRTUNT,RETC)
        IF (RETC.LT.-1) GOTO 30
      ENDIF
      IF (ISTART.GT.Noent) THEN
        ISTART = ISTART - 10
        ISTOP = Noent
      ENDIF
C FIND AND SORT ALL YEARS COVERED BY DATA ON FILE
      CALL SORTYR(PRTUNT,py,noy,RETC)
      IF (RETC.NE.-2) THEN
C SET UP THE 'PASS' ARRAY FOR THIS DATA STATION REFERENCE
        CALL STPASS()
C
C GENERATE THE DIRECTORY SUMMARY -
C much of code below is to account for searching a data base using
C a dsr file that does not cover all stations.
C
        CALL SORTIN()
        WRITE (*,9005)
        found = .FALSE.
   10   IF (ISTART.LT.1) ISTART = 1
        IF (ISTOP.GT.Noent) ISTOP = Noent
        ln = 0
        DO 20 i = ISTART, ISTOP
          IF (.NOT.Pass(i)) THEN
            found = .TRUE.
            CALL DECODE(Btime(i),by,bmo,bd,bh,bmn,ijd,ibelt)
            CALL DECODE(Etime(i),ey,emo,ed,eh,emn,ijd,ieelt)
            IF (by.NE.ey) CALL INCELT(by,ey,ieelt)
            size = (ieelt-ibelt)/Rdint(i) + 1
            istanm = Statno(i)
            WRITE (*,9010) i, istanm, Dtypei(i), by, bmo, bd, bh, bmn, 
     &                     ey, emo, ed, eh, emn, Rperd(i), size, 
     &                     Stypei(i)
            lastfd = i
          ENDIF
          IF (.NOT.found) ln = ln + 1
   20   CONTINUE
        IF (found) ISTOP = lastfd
        ISTART = ISTART + ln
        IF (.NOT.found) THEN
          IF (ISTOP.NE.Noent) THEN
            ISTOP = ISTART + 9
          ELSE
            IF (lastfd.EQ.0) THEN
              RETC = -14
              CALL ERRDAD(RETC,PRTUNT)
              GOTO 30
            ENDIF
            ISTOP = lastfd
            ISTART = ISTOP - 9
          ENDIF
          GOTO 10
        ENDIF
      ENDIF
   30 RETURN
C
 9005 FORMAT (/,' DATA|',5X,'STATION',4X,
     &        '|PARM| START OF ENTRY |  END OF ENTRY  |RDS/|NO OF|STOR|'
     &        ,/,' SET |   IDENTIFIER   |CODE',2('| YR MO DY HR MN '),
     &        '| DAY|  RDS|TYPE|',/,1X,77('='))
 9010 FORMAT (I4,' |',A16,'| ',A2,' |',
     &    2(I3.2,2('/',I2.2),I3.2,':',I2.2,' |'),I4,'|',I5,'| ',A2,' |')
      END
C
C
C
      SUBROUTINE SORTIN
C
C     + + + PURPOSE + + +
C     Sort the TDDB index stored in common INDEX and CINDEX
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER sti*16, ch2*2
      LOGICAL psi
      INTEGER bti, eti, pti, ip1, n, k, l, i, j, m
      INTEGER*2 rpd, rdi, err, trk, byt
C
C     + + + INCLUDES + + +
      INCLUDE 'dadblk.cmn'
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
      INCLUDE 'datyps.cmn'
C
C     + + + END SPECIFICATIONS + + +
C
C INITIALIZATIONS
      IF (Noent.GT.1) THEN
        n = Noent - 1
        k = n
        Sorted = .FALSE.
C
C SORTING ORDER:  STATION NUMBER - PARAMETER CODE - STORAGE TYPE -
C BEGIN TIME
C
        DO 60 l = 1, n
          IF (Sorted) GOTO 70
          Sorted = .TRUE.
          DO 50 i = 1, k
            ip1 = i + 1
C
C SORT ON STATION NUMBER
C
            IF (Statno(ip1).GT.Statno(i)) GOTO 50
            IF (Statno(ip1).EQ.Statno(i)) THEN
C
C EQUAL STATION NUMBERS - SORT ON PARAMETER CODE
C
              IF (Dtypei(i).NE.Dtypei(ip1)) THEN
                DO 10 m = 1, NOTYPS
                  IF (Dtypei(i).EQ.Types(m)) GOTO 50
                  IF (Dtypei(ip1).EQ.Types(m)) GOTO 30
   10           CONTINUE
              ENDIF
C
C EQUAL PARAMETER CODES - SORT ON STORAGE TYPE
C
              IF (Stypei(i).NE.Stypei(ip1)) THEN
                DO 20 m = 1, NOSTYP
                  IF (Stypei(i).EQ.Stypes(m)) GOTO 50
                  IF (Stypei(ip1).EQ.Stypes(m)) GOTO 30
   20           CONTINUE
              ENDIF
C
C EQUAL STORAGE TYPE - SORT ON BEGIN TIME
C
              IF (Btime(ip1).GT.Btime(i)) GOTO 50
            ELSE
C
C SWITCH THE ORDER OF I AND IP1 INDEX ENTRIES.
C
              sti = Statno(i)
              Statno(i) = Statno(ip1)
              Statno(ip1) = sti
            ENDIF
   30       ch2 = Dtypei(i)
            Dtypei(i) = Dtypei(ip1)
            Dtypei(ip1) = ch2
            ch2 = Stypei(i)
            Stypei(i) = Stypei(ip1)
            Stypei(ip1) = ch2
            bti = Btime(i)
            Btime(i) = Btime(ip1)
            Btime(ip1) = bti
            eti = Etime(i)
            Etime(i) = Etime(ip1)
            Etime(ip1) = eti
            pti = Ptime(i)
            Ptime(i) = Ptime(ip1)
            Ptime(ip1) = pti
            ch2 = Dummy(i)
            Dummy(i) = Dummy(ip1)
            Dummy(ip1) = ch2
            rpd = Rperd(i)
            Rperd(i) = Rperd(ip1)
            Rperd(ip1) = rpd
            rdi = Rdint(i)
            Rdint(i) = Rdint(ip1)
            Rdint(ip1) = rdi
            trk = Trkno(i)
            Trkno(i) = Trkno(ip1)
            Trkno(ip1) = trk
            byt = Bytno(i)
            Bytno(i) = Bytno(ip1)
            Bytno(ip1) = byt
            psi = Pass(i)
            Pass(i) = Pass(ip1)
            Pass(ip1) = psi
            DO 40 j = 1, 6
              err = Erflag(j,i)
              Erflag(j,i) = Erflag(j,ip1)
              Erflag(j,ip1) = err
   40       CONTINUE
            Sorted = .FALSE.
C
C COMPARE NEXT TWO ENTRIES
C
   50     CONTINUE
          k = k - 1
   60   CONTINUE
      ENDIF
C
C SORT COMPLETE
C
   70 RETURN
      END
C
C
C
      SUBROUTINE CHKRPD
     I                 ( IRPD,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Verify that a valid readings per day is specified
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IRPD, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IRPD   - Data type
C     RETC   - Return code, rtcode=1 for invalid recording density
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + INCLUDES + + +
      INCLUDE 'dastyp.cmn'
C
C     + + + END SPECIFICATIONS + + +
C
      RETC = 0
      DO 10 i = 1, NORPD
        IF (IRPD.EQ.Rpday(i)) GOTO 20
   10 CONTINUE
      RETC = 1
   20 RETURN
      END
C
C
C
      SUBROUTINE CHKSTP
     I                 ( STYPE,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Verify that a valid data type code is specified
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*2 STYPE
      INTEGER RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STYPE  - Data type
C     RETC   - Return code, rtcode=15 for invalid storage type
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + INCLUDES + + +
      INCLUDE 'datyps.cmn'
C
C     + + + END SPECIFICATIONS + + +
C
      RETC = 0
C     Stypes(1) = I2
C     Stypes(2) = I4
C     Stypes(3) = R4
C     Stypes(4) = R8
      DO 10 i = 1, NOSTYP
        IF (STYPE.EQ.Stypes(i)) GOTO 20
   10 CONTINUE
      RETC = 15
   20 RETURN
      END
C
C
C
      SUBROUTINE CHKTYP
     I                 ( TYPE,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Verify that a valid parameter code is specified
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*2 TYPE
      INTEGER RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     TYPE   - Parameter code
C     RETC   - Return code, rtcode=5 for invalid parameter code
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + INCLUDES + + +
      INCLUDE 'datyps.cmn'
C
C     + + + END SPECIFICATIONS + + +
C
      RETC = 0
      DO 10 i = 1, NOTYPS
        IF (TYPE.EQ.Types(i)) GOTO 20
   10 CONTINUE
      RETC = 5
   20 RETURN
      END
C
C
C
      SUBROUTINE CKSTYP
     I                 ( DTYPE,
     M                   STYPE )
C
C     + + + PURPOSE + + +
C     Set STORAGE type, if not set to one of valid types (I2, I4, R4, or
C     R8), based on traditional meaning of parameter codes:  " Z", " P",
C     "PH", " S", " T", "XV", and "XU" were I*2 data, the rest were I*4
C     data, now any parameter code may be any storage type
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*2 STYPE, DTYPE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DTYPE  - Parameter type (defined in bldtyp.f)
C     STYPE  - Data type (I2, I4, R4, or R8)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + INCLUDES + + +
      INCLUDE 'datyps.cmn'
C
C     + + + END SPECIFICATIONS + + +
C
      IF (STYPE.NE.'  ') THEN
        DO 10 i = 1, NOSTYP
          IF (STYPE.EQ.Stypes(i)) GOTO 20
   10   CONTINUE
      ENDIF
      STYPE = 'I4'
      IF (DTYPE.EQ.' Z' .OR. DTYPE.EQ.' P' .OR. DTYPE.EQ.'PH' .OR. 
     &    DTYPE.EQ.' S' .OR. DTYPE.EQ.' T' .OR. DTYPE.EQ.'XV' .OR. 
     &    DTYPE.EQ.'XU') STYPE = 'I2'
C
   20 RETURN
      END
C
C
C
      SUBROUTINE GETPLB
     I                 ( TYPE,
     O                   CTYPE )
C
C     + + + PURPOSE + + +
C     Determine the parameter label based on convention in bldtyp.f
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER TYPE*2, CTYPE*26
C
C     + + + ARGUMENT DEFINITIONS + + +
C     TYPE   - Parameter type
C     CTYPE  - Parameter label string
C
C     + + + INCLUDES + + +
      INCLUDE 'datyps.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + END SPECIFICATIONS + + +
C
      DO 10 i = 1, NOTYPS
        IF (TYPE.EQ.Types(i)) THEN
          CTYPE = Ctyps(i)
          GOTO 20
        ENDIF
   10 CONTINUE
      CTYPE = 'NOT FOUND'
   20 RETURN
      END
C
C
C
      SUBROUTINE GETSTP
     I                 ( STYPE,
     O                   NSTEP )
C
C     + + + PURPOSE + + +
C     Determine the step increment based on storage type
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*2 STYPE
      INTEGER NSTEP
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STYPE  - Storage type
C     NSTEP  - Step increment, I2 = 1, I4 & R4 = 2, R8 = 4
C
C     + + + END SPECIFICATIONS + + +
C
      NSTEP = 2
      IF (STYPE.EQ.'I2') NSTEP = 1
      IF (STYPE.EQ.'R8') NSTEP = 4
      RETURN
      END
C
C
C
      SUBROUTINE ERRDAD
     I                 ( ICODE, PRINTR )
C
C     + + + PURPOSE + + +
C     Print standard DADIO error messages
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER ICODE, PRINTR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ICODE  - Error code
C     PRINTR - Unit of printed output
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
      INCLUDE 'datyps.cmn'
      INCLUDE 'dastyp.cmn'
C
C     + + + END SPECIFICATIONS + + +
C
      IF (ICODE.EQ.10 .OR. ICODE.EQ.4) THEN
        WRITE (PRINTR,9145) ICODE
      ELSE
        WRITE (PRINTR,9005) ICODE
      ENDIF
      IF (ICODE.EQ.1) THEN
        WRITE (PRINTR,9075) Rpday
      ELSEIF (ICODE.EQ.2) THEN
        WRITE (PRINTR,9080)
      ELSEIF (ICODE.EQ.3) THEN
        WRITE (PRINTR,9085)
      ELSEIF (ICODE.EQ.4) THEN
        WRITE (PRINTR,9090)
      ELSEIF (ICODE.EQ.5) THEN
        WRITE (PRINTR,9095)
        WRITE (PRINTR,9160) Types
      ELSEIF (ICODE.EQ.6) THEN
        WRITE (PRINTR,9100)
      ELSEIF (ICODE.EQ.7) THEN
        WRITE (PRINTR,9105)
      ELSEIF (ICODE.EQ.8) THEN
        WRITE (PRINTR,9110)
      ELSEIF (ICODE.EQ.9) THEN
        WRITE (PRINTR,9115) MAXDYS
      ELSEIF (ICODE.EQ.10) THEN
        WRITE (PRINTR,9120)
      ELSEIF (ICODE.EQ.11) THEN
        WRITE (PRINTR,9125)
      ELSEIF (ICODE.EQ.12) THEN
C---codes 10 and 11
        WRITE (PRINTR,9120)
        WRITE (PRINTR,9125)
      ELSEIF (ICODE.EQ.13) THEN
C---codes 4 and 11
        WRITE (PRINTR,9090)
        WRITE (PRINTR,9125)
      ELSEIF (ICODE.EQ.14) THEN
        WRITE (PRINTR,9130)
      ELSEIF (ICODE.EQ.15) THEN
        WRITE (PRINTR,9150)
        WRITE (PRINTR,9160) Stypes
      ELSEIF (ICODE.EQ.16) THEN
C---codes 5 and 15
        WRITE (PRINTR,9155)
        WRITE (PRINTR,9160) Types
        WRITE (PRINTR,9160) Stypes
      ELSEIF (ICODE.EQ.17) THEN
C---codes 4 and 10
        WRITE (PRINTR,9090)
        WRITE (PRINTR,9120)
      ELSEIF (ICODE.EQ.18) THEN
C---codes 4, 10, and 11
        WRITE (PRINTR,9090)
        WRITE (PRINTR,9120)
        WRITE (PRINTR,9125)
      ELSEIF (ICODE.EQ.-1) THEN
        WRITE (PRINTR,9010) MAXENT
      ELSEIF (ICODE.EQ.-2) THEN
        WRITE (PRINTR,9015) MAXYRS
      ELSEIF (ICODE.EQ.-3) THEN
        WRITE (PRINTR,9020)
      ELSEIF (ICODE.EQ.-4) THEN
        WRITE (PRINTR,9025) MAXFLD
      ELSEIF (ICODE.EQ.-5) THEN
        WRITE (PRINTR,9030)
      ELSEIF (ICODE.EQ.-6) THEN
        WRITE (PRINTR,9035)
      ELSEIF (ICODE.EQ.-7) THEN
        WRITE (PRINTR,9040)
      ELSEIF (ICODE.EQ.-8) THEN
        WRITE (PRINTR,9045)
      ELSEIF (ICODE.EQ.-9) THEN
        WRITE (PRINTR,9050)
      ELSEIF (ICODE.EQ.-10) THEN
        WRITE (PRINTR,9055)
      ELSEIF (ICODE.EQ.-11) THEN
        WRITE (PRINTR,9060)
      ELSEIF (ICODE.EQ.-12) THEN
        WRITE (PRINTR,9065)
      ELSEIF (ICODE.EQ.-13) THEN
        WRITE (PRINTR,9070)
      ELSEIF (ICODE.EQ.-14) THEN
        WRITE (PRINTR,9140)
      ELSE
        WRITE (PRINTR,9135)
      ENDIF
      RETURN
 9005 FORMAT (/,10X,'***DADIO ERROR, RETURN CODE =',I3,'***')
 9010 FORMAT (
     & ' TDDB INDEX RECORDS ARE FULL; RECONSTRUCT THE TDDB, MAXENT=',I4)
 9015 FORMAT (' MAXIMUM NUMBER OF YEARS (',I3,
     &        ') ALLOWED IN THE TDDB EXCEEDED; CREATE A NEW TDDB')
 9020 FORMAT (' SYSTEM SUPPLIED DATE AND (OR) TIME IS ERRANT')
 9025 FORMAT (
     &   ' MAXIMUM NUMBER OF STATION IDENTIFIERS IN DSR FILE EXCEEDED ('
     &   ,I4,')')
 9030 FORMAT (' TDDB INDEX IS EMPTY; BE SURE DADIO IS CALLED BEFORE',/,
     &        ' DADI OR DADO OR CHECK ASSIGNED FILE NAMES')
 9035 FORMAT (' TDDB INDEX HAS NOT BEEN READ; BE SURE DADIO IS CALLED',
     &        ' BEFORE DADI OR DADO')
 9040 FORMAT (
     & ' LUTDDB EQUALS LUDSR OR LUPRT; CHECK VALUES OF DADIO ARGUMENTS')
 9045 FORMAT (' THE DSR FILE IS EMPTY')
 9050 FORMAT (
     &  ' INVALID NUMBER OF READINGS PER DAY ENCOUNTERED FOR A DATA SET'
     &  ,/,' POSSIBLY INVALID PROGRAM VERSION')
 9055 FORMAT (' ERROR READING THE TDDB INDEX')
 9060 FORMAT (' ERROR WRITING THE TDDB INDEX')
 9065 FORMAT (' ERROR OPENING THE TDDB; CHECK FOR CORRECT FILE NAME')
 9070 FORMAT (' ERROR OPENING THE DSR FILE;',/,
     &        ' A VALID FILE MUST EXIST BEFORE ACCESSING THE TDDB')
 9075 FORMAT (
     &       ' NUMBER OF READINGS PER DAY IS INVALID, VALID VALUES ARE:'
     &       ,/,12I5)
 9080 FORMAT (' INVALID STATION IDENTIFIER; NOT FOUND IN DSR FILE')
 9085 FORMAT (' INVALID DATE AND (OR) TIME SPECIFIED')
 9090 FORMAT (' REQUESTED DATA CONTAINS PREVIOUSLY FLAGGED VALUES',/,
     &        ' (WARNING CODES 11000, -11000, or -22000)')
 9095 FORMAT (' INVALID PARAMETER CODE, VALID CODES ARE:')
 9100 FORMAT (' NUMBER OF READINGS PER DAY SPECIFIED DOES NOT MATCH THE'
     &        ,/,' VALUE FOR THE DATA STORED IN THE TDDB')
 9105 FORMAT (' NO DATA AVAILABLE FOR TIME SPAN OF REQUEST')
 9110 FORMAT (' DATA PREVIOUSLY STORED FOR SPECIFIED TIME SPAN')
 9115 FORMAT (' MAXIMUM NUMBER OF DAYS (',I4,
     &        ') OF RECORD ALLOWED PER DATA SET EXCEEDED BY TIME SPAN')
 9120 FORMAT (' REQUESTED DATA CONTAINS PREVIOUSLY UPDATED VALUES',/,
     &        ' (WARNING CODE 22000 ASSIGNED BY DAFIX)')
 9125 FORMAT (' INSUFFICIENT DATA AVAILABLE FOR TIME SPAN SPECIFIED')
 9130 FORMAT (
     & ' SPECIFIED TIME IS NOT AN EVEN INCREMENT OF RECORDING INTERVAL')
 9135 FORMAT (' Invalid error code')
 9140 FORMAT (
     &' NO DATA SETS SELECTED BASED ON STATIONS IDENTIFIED IN THE DSR FI
     &LE',/,' TDDB NOT EMPTY, PERHAPS WRONG DSR FILE SPECIFIED')
 9145 FORMAT (/,10X,'***DADIO WARNING, RETURN CODE =',I3,'***')
 9150 FORMAT (' INVALID STORAGE CODE, VALID CODES ARE:')
 9155 FORMAT (
     &' INVALID PARAMETER CODE AND STORAGE-TYPE CODE, VALID CODES ARE:')
 9160 FORMAT (25(1X,A2))
      END
