      SUBROUTINE DADCHK
     I                 ( STANUM, TYPE, STYPE, BY, BMO, BD, BH, BMN, EY,
     I                   EMO, ED, EH, EMN, RDPDY, PRTMSG,
     O                   RTCODE )
C
C     + + + PURPOSE + + +
C     Valid data-definintion variable values
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STANUM*16, TYPE*2, STYPE*2
      INTEGER RTCODE, RDPDY, BY, BMO, BD, BH, BMN, EY, EMO, ED, EH, EMN
      LOGICAL PRTMSG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STANUM - Station identifier
C     TYPE   - Parameter type
C     STYPE  - Data storage type
C     BY     - Beginning year of request
C     BMO    - Beginning month
C     BD     - Beginning day
C     BH     - Beginning hour
C     BMN    - Beginining minute
C     EY     - Ending year
C     EMO    - Ending month
C     ED     - Ending day
C     EH     - Ending hour
C     EMN    - Ending minute
C     RDPDY  - Number of readings per day
C     PRTMSG - Flag to allow print of DADIO warning messages
C     RTCODE - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, irdh, iret
C
C     + + + INCLUDES + + +
      INCLUDE 'dadblk.cmn'
      INCLUDE 'dstrf.cmn'
C
C     + + + INTRINSICS + + +
      INTRINSIC IFIX, SIGN
C
C     + + + EXTERNALS + + +
      EXTERNAL DTCODE, CHKSTA, CHKTYP, ERRDAD, CHKRPD, CHKSTP, GETSTP,
     &         INCELT, PRTDDF
C
C     + + + END SPECIFICATIONS + + +
C
      RTCODE = 0
      IF (.NOT.Opend) THEN
        RTCODE = -6
      ELSE
        CALL CHKRPD(RDPDY,RTCODE)
        IF (RTCODE.EQ.0) THEN
          Irdint = 1440/RDPDY
          IF ((BMN-BMN/Irdint*Irdint).EQ.0 .AND. (EMN-EMN/Irdint*Irdint)
     &        .EQ.0) GOTO 10
          IF (Irdint.GT.60) THEN
            irdh = Irdint/60
            IF ((BH-BH/irdh*irdh.EQ.0) .AND. (EH-EH/irdh*irdh.EQ.0))
     &          GOTO 10
          ENDIF
          RTCODE = 14
        ENDIF
      ENDIF
      GOTO 40
   10 CALL DTCODE(BY,BMO,BD,BH,BMN,Bt,Belt,iret)
      IF (iret.EQ.0) THEN
        CALL DTCODE(EY,EMO,ED,EH,EMN,Et,Eelt,iret)
        IF (iret.EQ.0 .AND. BY.LE.EY) THEN
          CALL INCELT(BY,EY,Eelt)
          IF (Belt.LE.Eelt) THEN
            CALL CHKSTA(STANUM)
            DO 30 i = 1, Istk
              IF (STANUM.NE.Fldsta(i)) GOTO 30
C
C CHECK FOR INVALID DATA TYPE
C
              CALL CHKTYP(TYPE,RTCODE)
              CALL CHKSTP(STYPE,iret)
              IF (iret.NE.0 .AND. RTCODE.NE.0) RTCODE = 16
              IF (iret.NE.0 .AND. RTCODE.NE.16) RTCODE = iret
              IF (RTCODE.NE.0) GOTO 40
C
C NSTEP DETERMINED BY GETSTP THE 'STEP' VALUE FOR THE DATA TYPE.
C   1, INTEGER*2 DATA
C   2, INTEGER*4 OR REAL*4 DATA
C   4, REAL*8 DATA
C
              CALL GETSTP(STYPE,Nstep)
              Izcorr = 0
              IF (TYPE.EQ.' Z' .AND. Apydtm)
     &           Izcorr = IFIX(Datum(i)*100.0+SIGN(0.5,Datum(i)))
              GOTO 50
   30       CONTINUE
            RTCODE = 2
C
C ERROR : PRINT ARGUMENT LIST IF ERROR RTCODE ENCOUNTERED.
C
            GOTO 40
          ENDIF
        ENDIF
      ENDIF
      RTCODE = 3
   40 IF (PRTMSG) THEN
        CALL ERRDAD(RTCODE,Printr)
        CALL PRTDDF(STANUM,TYPE,STYPE,BY,BMO,BD,BH,BMN,EY,EMO,ED,EH,EMN,
     &              RDPDY,Printr)
      ENDIF
C
   50 RETURN
      END
C
C
C
      SUBROUTINE PRTDDF
     I                 ( STANUM, TYPE, STYP, BY, BMO, BD, BH, BMN, EY,
     I                   EMO, ED, EH, EMN, RDPDY, PRINTR )
C
C     + + + PURPOSE + + +
C     Print time-series data definition parameters in standard form.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STANUM*16, TYPE*2, STYP*2
      INTEGER BY, BMO, BD, BH, BMN, EY, EMO, ED, EH, EMN, RDPDY, PRINTR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STANUM - Station identifier
C     TYPE   - Parameter type
C     STYP   - Data storage type
C     BY     - Beginning year of request
C     BMO    - Beginning month
C     BD     - Beginning day
C     BH     - Beginning hour
C     BMN    - Beginining minute
C     EY     - Ending year
C     EMO    - Ending month
C     ED     - Ending day
C     EH     - Ending hour
C     EMN    - Ending minute
C     RDPDY  - Number of readings per day
C     PRINTR - Logical unit number for printed output
C
C     + + + END SPECIFICATIONS + + +
C
      WRITE (PRINTR,9005) STANUM, BY, BMO, BD, BH, BMN, EY, EMO, ED, EH, 
     &                    EMN, TYPE, RDPDY, STYP
      RETURN
 9005 FORMAT (' STATION IDENTIFIER = "',A16,'"',/,' TIME SPAN =',I3.2,
     &        2('/',I2.2),I3.2,':',I2.2,' TO',I3.2,2('/',I2.2),I3.2,':',
     &        I2.2,/,' DATA-PARAMETER CODE = "',A2,'"    READINGS/DAY=',
     &        I4,4X,'STORAGE-TYPE CODE = "',A2,'"',/)
      END
C
C
C
      SUBROUTINE RDDSRF
     I                 ( DSRUNT, PRTUNT,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Read required data from the DSR file
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER DSRUNT, PRTUNT, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DSRUNT - Unit number of DSR file
C     PRTUNT - Unit number of printed output file
C     RETC   - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + INCLUDES + + +
      INCLUDE 'dstrf.cmn'
C
C     + + + EXTERNALS + + +
      EXTERNAL CHKSTA, ERRDAD
C
C     + + + END SPECIFICATIONS + + +
C
      Istk = 0
      DO 10 i = 1, MAXFLD
        READ (DSRUNT,9005,END=20) Fldsta(i), Datum(i)
        CALL CHKSTA(Fldsta(i))
        Istk = i
   10 CONTINUE
C     Exceeded maximum number of station identifiers
      RETC = -4
      CALL ERRDAD(RETC,PRTUNT)
C
   20 IF (Istk.EQ.0) THEN
        RETC = -8
        CALL ERRDAD(RETC,PRTUNT)
      ENDIF
      REWIND DSRUNT
      RETURN
C
 9005 FORMAT (1X,A16,8X,F8.2)
      END
C
C
C
      SUBROUTINE RDINDX
     I                 ( TDDUNT,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Read the TDDB index into common INDEX and CINDEX
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER TDDUNT, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     TDDUNT - Unit number of TDDB (IN)
C     RETC   - Return code, non-zero if error or warning condition (OUT)
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER nentrk, nintrk, iind, nind, j, i, k, ios
C
C     + + + INCLUDES + + +
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
C
C     + + + EXTERNALS + + +
      EXTERNAL SETNDX, CKSTYP
C
C     + + + END SPECIFICATIONS + + +
C
      CALL SETNDX(nentrk,nintrk,iind,nind)
C
      Noent = 0
      DO 30 j = 1, nintrk
        IF (nind.GT.MAXENT) nind = MAXENT
        READ (TDDUNT,REC=j,IOSTAT=ios) (Statno(i),Btime(i),Etime(i),
     &        Rperd(i),Dtypei(i), (Erflag(k,i),k=1,6),Dummy(i), 
     &        Stypei(i),Ptime(i),Trkno(i),Bytno(i),i=iind,nind)
        IF (ios.EQ.0) THEN
          DO 10 i = iind, nind
            CALL CKSTYP(Dtypei(i),Stypei(i))
            IF (Rperd(i).EQ.0) GOTO 20
            Noent = i
            Rdint(i) = 1440/Rperd(i)
   10     CONTINUE
   20     IF (nind.EQ.MAXENT) GOTO 40
          iind = iind + nentrk
          nind = nind + nentrk
          IF (iind.GT.MAXENT) iind = MAXENT
        ELSEIF (ios.LT.0) THEN
          RETC = -5
          GOTO 50
        ELSE
          RETC = -10
C
C Add check for end of file of new file, as Lahey sets ios>0 for EOF
C of direct access file and can't use REC= and END= in read of DA file
C 18446=Lahey iostat for "data transfer beyond end of file"
C 18525=Lahey 90 iostat for "data transfer beyond end of file"
C 168=Silicon Graphics Fortran for "data transfer beyond end of file"
C
          IF (j.EQ.1) THEN
            READ (TDDUNT,IOSTAT=ios) Dummy(1)
            IF (ios.LT.0 .OR. ios.EQ.18446 .OR. ios.EQ.168 .OR.
     &          ios.EQ.18525) RETC = -5
          ENDIF
          GOTO 50
        ENDIF
   30 CONTINUE
   40 IF (Noent.GT.MAXENT) RETC = -1
   50 RETURN
      END
C
C
C
      SUBROUTINE SETNDX
     O                 ( NENTRK, NINTRK, IIND, NIND )
C
C     + + + PURPOSE + + +
C     Set TDDB index counters
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NENTRK, NINTRK, IIND, NIND
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NENTRK - Number of index entries per TDDB record
C     NINTRK - Number of records for the TDDB index
C     IIND   - Index number of first data set in current index record
C     NIND   - Index number of last data set in current index record
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + INTRINSICS + + +
      INTRINSIC MOD
C
C     + + + END SPECIFICATIONS + + +
C
      NENTRK = LENREC/LENIND
      NINTRK = MAXENT/NENTRK
      IF (MOD(MAXENT,NENTRK).NE.0) NINTRK = NINTRK + 1
      IIND = 1
      NIND = NENTRK
C
      RETURN
      END
C
C
C
      SUBROUTINE INTNDX
C
C     + + + PURPOSE + + +
C     Initialize TDDB index arrays
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER k, n
      CHARACTER blank*2, blnk16*16
      INTEGER*2 erf(6*MAXENT), i2zero
C
C     + + + INCLUDES + + +
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
C
C     + + + EQUIVALENCES + + +
      EQUIVALENCE (Erflag(1,1),erf(1))
C
C     + + + DATA + + +
      DATA blank/'  '/, blnk16/'                '/, i2zero/0/
C
C     + + + END SPECIFICATIONS + + +
C
      DO 10 k = 1, MAXENT
        Statno(k) = blnk16
        Btime(k) = 0
        Etime(k) = 0
        Ptime(k) = 0
        Rperd(k) = i2zero
        Dtypei(k) = blank
        Stypei(k) = blank
        Trkno(k) = i2zero
        Bytno(k) = i2zero
        Dummy(k) = blank
   10 CONTINUE
      n = MAXENT*6
      DO 20 k = 1, n
        erf(k) = i2zero
   20 CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE WRTNDX
     I                 ( TDDUNT, PRTUNT,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Write index to the TDDB
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER TDDUNT, PRTUNT, RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     TDDUNT - Logical unit number of TDDB
C     PRTUNT - Logical unit number for printed output
C     RETC   - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, j, nentrk, nintrk, iind, nind, k, ios
C
C     + + + INCLUDES + + +
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
C
C     + + + EXTERNALS + + +
      EXTERNAL SETNDX, ERRDAD
C
C     + + + END SPECIFICATIONS + + +
C
      CALL SETNDX(nentrk,nintrk,iind,nind)
C
      DO 10 j = 1, nintrk
        IF (nind.GT.MAXENT) nind = MAXENT
        WRITE (TDDUNT,REC=j,IOSTAT=ios) (Statno(i),Btime(i),Etime(i),
     &                                  Rperd(i),Dtypei(i),
     &                                  (Erflag(k,i),k=1,6),Dummy(i),
     &                                  Stypei(i),Ptime(i),Trkno(i),
     &                                  Bytno(i),i=iind,nind)
        IF (ios.GT.0) THEN
          RETC = -11
          CALL ERRDAD(RETC,PRTUNT)
          GOTO 20
        ENDIF
        IF (nind.EQ.MAXENT) GOTO 20
        iind = iind + nentrk
        nind = nind + nentrk
        IF (iind.GT.MAXENT) iind = MAXENT
   10 CONTINUE
C
   20 RETURN
      END
C
C
C
      SUBROUTINE SORTYR
     I                 ( PRTUNT,
     O                   PY, NOY, RETC )
C
C     + + + PURPOSE + + +
C     Find and sort all years covered by data in TDDB
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER PRTUNT, PY(*), RETC, NOY
C
C     + + + ARGUMENT DEFINITIONS + + +
C     PRTUNT - Unit number of printed output
C     PY     - Sorted array of each year in the TDDB
C     NOY    - Number of years with data in the TDDB
C     RETC   - Return code, non-zero if error or warning condition
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, j, k, by, ey, iy, n
C
C     + + + INCLUDES + + +
      INCLUDE 'index.cmn'
C
C     + + + EXTERNALS + + +
      EXTERNAL ERRDAD
C
C     + + + END SPECIFICATIONS + + +
C
      by = Btime(1)/10000000
      ey = Etime(1)/10000000
      NOY = 1
      PY(1) = by
      IF (by.NE.ey) THEN
        NOY = 2
        PY(2) = ey
      ENDIF
      DO 40 i = 2, Noent
        by = Btime(i)/10000000
        ey = Etime(i)/10000000
        iy = by
   10   DO 20 j = 1, NOY
          IF (iy.EQ.PY(j)) GOTO 30
   20   CONTINUE
        NOY = NOY + 1
        IF (NOY.GT.MAXYRS) THEN
C
          RETC = -2
          CALL ERRDAD(RETC,PRTUNT)
          GOTO 70
        ENDIF
        PY(NOY) = iy
   30   IF (iy.NE.ey) THEN
          iy = ey
          GOTO 10
        ENDIF
   40 CONTINUE
C
      IF (NOY.GT.1) THEN
        n = NOY
   50   k = 0
        n = n - 1
        DO 60 j = 1, n
          IF (PY(j).GT.PY(j+1)) THEN
            k = 1
            iy = PY(j)
            PY(j) = PY(j+1)
            PY(j+1) = iy
          ENDIF
   60   CONTINUE
        IF (k.EQ.1) GOTO 50
      ENDIF
C
   70 RETURN
      END
C
C
C
      SUBROUTINE STPASS
C
C     + + + PURPOSE + + +
C     Set up the 'PASS' array for the DSR file
C
C     + + + PARAMETERS + + +
      INCLUDE 'dimpar.cmn'
C
C     + + + INCLUDES + + +
      INCLUDE 'index.cmn'
      INCLUDE 'cindex.cmn'
      INCLUDE 'dstrf.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, j
      CHARACTER*16 istanm
C
C     + + + END SPECIFICATIONS + + +
C
      DO 10 i = 1, MAXENT
        Pass(i) = .TRUE.
   10 CONTINUE
      DO 30 i = 1, Istk
        istanm = Fldsta(i)
        DO 20 j = 1, Noent
          IF (Statno(j).EQ.istanm) Pass(j) = .FALSE.
   20   CONTINUE
   30 CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE DAYFLG
     I                 ( IRDS, RDPDAY, LRDPDY, ERRORS, IRDCOD, SYMBOL,
     M                   RV )
C
C     + + + PURPOSE + + +
C     Check on status of data for a day for callendar year summary
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IRDS, RDPDAY, LRDPDY, ERRORS
      CHARACTER*4 IRDCOD, SYMBOL(6), RV
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IRDS   - Number of readings available in a day for the data set
C     RDPDAY - Number of readings per day
C     LRDPDY - Number of readings per day of last data set
C     ERRORS - Flag to signal if suspect readings are present
C     IRDCOD - Code corresponding to number of rds/day ' U |' = 96/day
C     SYMBOL - Array of day symbol flags
C     RV     - Code for day in calendar-summary matrix
C
C     + + + LOCAL VARIABLES + + +
      INTEGER iabrvl, lnrds, mrds
C
C     + + + INTRINSICS + + +
      INTRINSIC IABS
C
C     + + + END SPECIFICATIONS + + +
C
      IF (RV.EQ.'   |') THEN
        lnrds = 0
        IF (IRDS.NE.RDPDAY) THEN
C PARTIAL DAY
          mrds = IRDS
C MARK ERROR FLAG
          IF (ERRORS.NE.0) THEN
            mrds = -IRDS
            IF (mrds.LT.-999) THEN
              lnrds = mrds
              mrds = -999
            ENDIF
          ENDIF
          WRITE (RV,'(I4)') mrds
C COMPLETE DAY
        ELSEIF (ERRORS.EQ.0) THEN
C COMPLETE DAY, NO FLAGS
          RV = IRDCOD
        ELSE
C COMPLETE DAY, FLAGS
          RV = SYMBOL(5)
        ENDIF
      ELSEIF (LRDPDY.NE.RDPDAY) THEN
C DUAL DENSITY RECORD
        RV = SYMBOL(3)
      ELSE
C MULTIPLE DATA SETS ON SAME DAY
        READ (RV,'(I4)') mrds
        IF (mrds.EQ.-999 .AND. lnrds.NE.0) mrds = lnrds
        iabrvl = IABS(mrds) + IRDS
        IF (iabrvl.NE.RDPDAY) THEN
C PARTIAL DAY, MARK ERROR FLAG
          IF (ERRORS.NE.0 .OR. mrds.LT.0) THEN
            mrds = -iabrvl
            IF (mrds.LT.-999) THEN
              lnrds = mrds
              mrds = -999
            ENDIF
          ENDIF
          WRITE (RV,'(I4)') mrds
C COMPLETE DAY
        ELSEIF (ERRORS.EQ.0 .AND. mrds.GT.0) THEN
C COMPLETE DAY, NO FLAGS
          RV = IRDCOD
        ELSE
C COMPLETE DAY, FLAGS
          RV = SYMBOL(5)
        ENDIF
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE STRPI2
     I                 ( STRIP,
     M                   IDAT )
C
C     + + + PURPOSE + + +
C     Strip status flags from I*2 data value
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER STRIP, IDAT
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRIP  - Flag to determine to strip status code
C     IDAT   - Data value
C
C     + + + END SPECIFICATIONS + + +
C
C put if in calling program
      IF (STRIP.NE.-1) THEN
C       STRIP = 1
C STRIP ONLY +22000 ERROR CODE (22000-999=21001):
        IF (IDAT.GT.21000) IDAT = IDAT - 22000
C DATUM IS ERRANT; DETERMINE THE ERROR CONDITION AND STRIP CODE.
      ELSEIF (IDAT.GT.0) THEN
C ERROR CODE +11000 OR +22000
        IF (IDAT.GT.21000) IDAT = IDAT - 11000
        IDAT = IDAT - 11000
      ELSE
C ERROR CODE -11000 OR -22000 (-22000+9999=12001):
        IF (IDAT.LT.-12000) IDAT = IDAT + 11000
        IDAT = IDAT + 11000
      ENDIF
C
      RETURN
      END
C
C
C
      CHARACTER*26 FUNCTION PARMNM
     I                            ( DTYPE )
C
C     + + + PURPOSE + + +
C     TO GET THE DEFINITION OF A PARAMETER CODE.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*2 DTYPE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DTYPE  - Parameter code
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + INCLUDES + + +
      INCLUDE 'datyps.cmn'
C
C     + + + END SPECIFICATIONS + + +
C
      PARMNM = ' '
      DO 10 i = 1, NOTYPS
        IF (DTYPE.EQ.Types(i)) THEN
          PARMNM = Ctyps(i)
          GOTO 20
        ENDIF
   10 CONTINUE
   20 RETURN
      END
