C
C     SYMBOL.FOR
C
      SUBROUTINE SYMBOL(X1,Y1,HEIGHT,STRING,ANGLE,NCHAR)
 
C     + + + PURPOSE + + +
C     Plots annotation and special symbols.
C     Called by the main program and routines AXIS, NUMBER, and LINE.
C     There are three calls to subroutine SYMBOL:
C     (1) The call to draw text such as titles and legends.
C     (2) The call to draw a single character referenced by its integer
C         index.
C     (3) The call to draw a plot, with a marker centered at X,Y.
C
C     + + + HISTORY + + +
C       AUTHOR: Steve Regan
C     LANGUAGE: Fortran 77
C      VERSION: June 17, 1993
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NCHAR
      REAL X1, Y1, HEIGHT, ANGLE
      CHARACTER*(*) STRING
C
C     + + +INPUT ARGUMENT DEFINITIONS + + +
C     X1,Y1    - If NCHAR > 0, these are the coordinates in inches of the
C                              lower left hand corner (before rotation) of
C                              the first character being plotted.
C              - If NCHAR <= 0, these are the coordinates of the geometric
C                               center of the marker being plotted.
C                To continue annotation from the current X and/or Y position,
C                the corresponding X1 and/or Y1 coordinates should equal 999.
C     HEIGHT   - The height in inches of the character(s) being plotted.
C                The width of the character, including spacing, is normally
C                the same as the height.
C     STRING   - If NCHAR > 0, this contains a string array of NCHAR
C                              character(s) being plotted.
C                If NCHAR <= 0, the integer index of the marker to be plotted
C                               is contained in STRING.
C     ANGLE    - The angle, in degrees from the x-axis, at which the
C                character(s) are to be plotted.  At ANGLE=0.0, the
C                character(s) will be plotted right side up and parallel
C                to the x-axis.
C     NCHAR    - If > 0,   NCHAR is the number of charcaters in STRING to be
C                          plotted, imncluding any trailing blanks.
C                If = 0/-1,A marker with integer index contained in
C                          STRING is centered at X1,Y1.
C                If < -1,  A marker is produced, where the move to X1,Y1
C                          is made with the line status the same as if
C                          the absolute value of NCHAR were the IPEN
C                          argument of subroutine PLOT.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ipen, intgr, nc
      REAL c, s, ra, x, y, xm, ym, hgt
      CHARACTER*1 sym
C
C     + + + INTRINSICS + + +
      INTRINSIC IABS, COS, SIN, ICHAR
C
C     + + + EXTERNALS + + +
      EXTERNAL PLOT, GMARK1, GMKSTR
C
C     + + + END SPECIFICATIONS + + +
C
C---- X,Y is the current unfactored location:
      x = X1
      y = Y1
      hgt = HEIGHT
C
C---- Determine whether or not a line is to be drawn
      IF (NCHAR.GT.-2) THEN
        ipen = 3
      ELSE
        ipen = IABS(NCHAR)
      ENDIF
      nc = NCHAR
C
C---- Move to character location
      CALL PLOT(x,y,ipen)
C
C---- Compute location of symbol:
C---- Set absolute direction for character plotting (convert to radians):
      ra = .0174533*ANGLE
      c = COS(ra)
      s = SIN(ra)
C
C---- Plot text string:
C
      IF (nc.EQ.0) THEN
C
C---- A single right-justified character produced.
C     This character may be passed from the calling routine as a single
C     character in quotes, or as the "char", the ASCII integer index of
C     the character; for example, char(65) is the same as 'A'.
C     (NCHAR is 0):
C
        sym = STRING(1:1)
C---- DIGS software generated character
        xm = hgt*0.5
        ym = xm*s
        xm = xm*c
        CALL GMARK1(ICHAR(sym),hgt,x-xm,y-ym)
C
      ELSEIF (nc.GT.0) THEN
C---- NCHAR is greater than 0 -- use standard characters stored in STRING:
        CALL GMKSTR(STRING,nc,hgt,x,y,ANGLE)
      ELSE
C
C  Special call to SYMBOL for a centered symbol (NCHAR is less than 0):
C  The calling routine should pass the index of the symbol desired
C  through STRING as the "char" of the symbol index number desired.
C  For example, for symbol number 1 (a dot), pass char(1);
C  this is then converted back to the integer 0 here using "ichar".
C
        intgr = ICHAR(STRING)
C
        CALL GMARK1(intgr,hgt,x,y)
      ENDIF
C
      RETURN
      END
C
C
C
      SUBROUTINE NXTPLT()
C
C     + + + PURPOSE + + +
C     Beeps to signal the end of screen plotting, and then waits for a
C     carriage return from the user to continue.  Finally, this routine
C     clears the screen.  Lahey dependent routine
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER dummy*1, vers*80
      LOGICAL kpress
C
C     + + + INTRINSICS + + +
      INTRINSIC CHAR
C
C     + + + EXTERNALS + + +
      LOGICAL ISKEY
      EXTERNAL PLOT, ISKEY, GTEXT, NEWPEN
C
C     + + + END SPECIFICATIONS + + +
C
      vers =
     &      '@(#) libcal - Supplement to Lahey CalComp graphics library'
      vers = '@(#) libcal - Contact: h2osoft@usgs.gov'
      vers = '@(#) libcal - Version: 6.0x 1997/02/05'
      vers =
     & '$Id: digsublh.f,v 6.0 1997/03/13 17:34:25 rsregan Exp rsregan $'

C---- Beep
      WRITE (*,*) CHAR(7)
      CALL NEWPEN(4)
C---- Lahey dependent calls to print message to screen
      CALL GTEXT(1,55,'PRESS ENTER TO CONTINUE')
C
C---- Clear screen
   10 kpress = ISKEY()
      IF (.NOT.(kpress)) GOTO 10
      READ (*,'(A)') dummy
C---- Lahey dependent call to clear screen
      CALL PLOT(0.0,0.0,-999)
      CALL FACTOR(1.0)
      RETURN
      END
