      SUBROUTINE OPNOTF
     I                 ( INSTRM, LEN, IFORM, OKFLAG, OUTFIL, IOUNT,
     I                   SUFFIX, FILTYP,
     M                   OTNAME )
C
C     + + + PURPOSE + + +
C     Generate a file name based on an input file name and given
C     suffix, then call a routine to open the file.
C     User is given the choice of a default name, either the name
C     that appeared in the master file or the name generated from
C     the base name, or a new name may be entered.  If the file
C     names were said to be OK in the master file, they are used
C     as is.  If a file name is not in the master file, one is generated
C     using the base name and given suffix.
C
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER LEN, IOUNT, IFORM
      LOGICAL OKFLAG, OUTFIL
      CHARACTER*(*) OTNAME, INSTRM, FILTYP
      CHARACTER SUFFIX*4
C
C     + + + ARGUMENT DEFINITIONS + + +
C     INSTRM - The name of the base file, used to build name from.
C     LEN    - The length in characters of the base name.
C     IFORM  - Flag to specify format of file (0=formatted, 1=unformatted,
C              2=unformatted, do not truncate)
C     OKFLAG - Logical flag to specify file names as they appear in the
C              master file are to be used.
C     OUTFIL - Logical flag to specify file names are to be generated using
C              the base name.
C     IOUNT  - Logical unit number of file to be opened.
C     SUFFIX - 4 character file name suffix, appended to base name.
C     FILTYP - character discriptor of file used to prompt user.
C     OTNAME - Generated file name, length assigned from calling pgm.
C
C     + + + EXTERNALS + + +
      EXTERNAL IOPRMP, OPNFIL, OPNFLU, OPNFUR
C
C     + + + END SPECIFICATIONS + + +
C
C---ASSIGN THE OUTPUT FILE NAME AND OPEN
      IF (OTNAME.EQ.'    ' .OR. OUTFIL) THEN
        OTNAME = '    '
        OTNAME(1:LEN) = INSTRM(1:LEN)
        OTNAME(LEN+1:LEN+4) = SUFFIX
        IF (OKFLAG) WRITE (*,9005) FILTYP, OTNAME
      ENDIF
      IF (.NOT.OKFLAG .AND. .NOT.OUTFIL) CALL IOPRMP(FILTYP,OTNAME)
      IF (IFORM.EQ.0) THEN
        CALL OPNFIL(IOUNT,OTNAME)
      ELSEIF (IFORM.EQ.2) THEN
        CALL OPNFUR(IOUNT,OTNAME)
      ELSE
        CALL OPNFLU(IOUNT,OTNAME)
      ENDIF
      RETURN
 9005 FORMAT (' *WARNING* ',A,' output file unspecified.',
     &        /,' *** Default file name assigned: ',A,/)
      END
C
C
C
      SUBROUTINE OPNFIL
     I                 ( IUNIT, FILNAM )
C
C     + + + PURPOSE + + +
C     Open a sequential formatted file for program output,
C     truncate if old file.
C     Use of FORTRAN statement OPEN and CLOSE to delete existing files.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT
      CHARACTER*(*) FILNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT  - The logical file unit number of the file
C     FILNAM - The name of the file, with length given from calling pgm
C
C     + + + LOCAL VARIABLES + + +
      LOGICAL exists
C
C     + + + END SPECIFICATIONS + + +
C
      IF (FILNAM.EQ.'prn ' .OR. FILNAM.EQ.'com1' .OR. FILNAM.EQ.'lpt1'
     &    .OR. FILNAM.EQ.'PRN ' .OR. FILNAM.EQ.'lpt1' .OR. 
     &    FILNAM.EQ.'PRN ' .OR. FILNAM.EQ.'COM1' .OR. FILNAM.EQ.'LPT1' 
     &    .OR. FILNAM.EQ.'com2' .OR. FILNAM.EQ.'lpt2' .OR.
     &    FILNAM.EQ.'lpt3' .OR. FILNAM.EQ.'COM2' .OR. FILNAM.EQ.'LPT2' 
     &    .OR. FILNAM.EQ.'LPT3' .OR. FILNAM.EQ.'com3' .OR. 
     &    FILNAM.EQ.'lpt4' .OR. FILNAM.EQ.'COM3' .OR. FILNAM.EQ.'LPT4') 
     &   THEN
        OPEN (IUNIT,FILE=FILNAM,STATUS='OLD',FORM='FORMATTED',
     &        ACCESS='SEQUENTIAL')
      ELSE
        INQUIRE (FILE=FILNAM,EXIST=exists)
        IF (exists) THEN
          OPEN(IUNIT,FILE=FILNAM)
          CLOSE(IUNIT,STATUS='DELETE')
        ENDIF
        OPEN (IUNIT,FILE=FILNAM,STATUS='NEW',FORM='FORMATTED',
     &        ACCESS='SEQUENTIAL')
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE OPNFLU
     I                 ( IUNIT, FILNAM )
C
C     + + + PURPOSE + + +
C     Open a sequential unformatted file for program output,
C     truncate if old file.
C     Use of FORTRAN statement OPEN and CLOSE with DELETE to
C     truncate existing files.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT
      CHARACTER*(*) FILNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT  - The logical file unit number of the file
C     FILNAM - The name of the file, with length given from calling pgm
C
C     + + + LOCAL VARIABLES + + +
      LOGICAL exists
C
C     + + + END SPECIFICATIONS + + +
C
      INQUIRE (FILE=FILNAM,EXIST=exists)
      IF (exists) THEN
C
C---TRUNCATE OUTPUT FILE
        OPEN (IUNIT,FILE=FILNAM)
        CLOSE (IUNIT,STATUS='DELETE')
      ENDIF
      OPEN (IUNIT,FILE=FILNAM,STATUS='NEW',FORM='UNFORMATTED',
     &      ACCESS='SEQUENTIAL')
      RETURN
      END
C
C
C
      SUBROUTINE OPNRDF
     I                 ( MSG, IUNIT,
     M                   FILNAM, OKFLAG )
C
C     + + + PURPOSE + + +
C     Open a sequential formatted file for program input.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT
      CHARACTER*(*) MSG, FILNAM
      LOGICAL OKFLAG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MSG    - Character string array describing value needed
C     IUNIT  - The logical file unit number of the file
C     FILNAM - The name of the file, with length given from calling pgm
C     OKFLAG - Flag to indicate file names as specified in master
C              file were correct by user, false means new master file or update
C
C     + + + LOCAL VARIABLES + + +
      INTEGER retc
C
C     + + + EXTERNALS + + +
      EXTERNAL IOPRMP, CHKIFL
C
C     + + + END SPECIFICATIONS + + +
C
   10 IF (.NOT.OKFLAG) CALL IOPRMP(MSG,FILNAM)
C
      CALL CHKIFL(FILNAM,retc)
      IF (retc.NE.0) THEN
        OKFLAG = .FALSE.
        GOTO 10
      ENDIF
      OPEN (IUNIT,FILE=FILNAM,STATUS='OLD',FORM='FORMATTED',
     &      ACCESS='SEQUENTIAL',BLANK='ZERO')
      RETURN
      END
C
C
C
      SUBROUTINE OPNRDR
     I                 ( IUNIT, FILNAM, IFLG,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Open a formatted file for progrm input.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT, IFLG, RETC
      CHARACTER*(*) FILNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT  - The logical file unit number of the file.
C     FILNAM - The name of the file, with length given from
C              the calling program.
C     IFLG   - Flag to indicate to open file if it does not
C              already exist.  If iflg=0, open only if file already
C              exists, if iflg=1, open if new or old.
C     RETC   - Flag to indicate file was opened successfully, if retc=0
C              an existing file was opened successfully, if retc=1 a new
C              file was opened successfully, if retc=-2 an error was
C              encountered opening the file, if retc=-3 an error was
C              encountered inquiring on file name, if retc=-1, a
C              required input file did not exist.
C
C     + + + LOCAL VARIABLES + + +
      LOGICAL exists
C
C     + + + EXTERNAL ERRFIL + + +
      EXTERNAL ERRFIL
C
C     + + + END SPECIFICATIONS + + +
C
      INQUIRE (FILE=FILNAM,EXIST=exists,err=10)
      IF (exists) THEN
        OPEN (IUNIT,FILE=FILNAM,STATUS='OLD',FORM='FORMATTED',
     &        ACCESS='SEQUENTIAL',BLANK='ZERO',ERR=20)
        RETC = 0
      ELSEIF (IFLG.EQ.0) THEN
        CALL ERRFIL('Specified file not found',FILNAM)
        RETC = -1
      ELSE
        OPEN (IUNIT,FILE=FILNAM,STATUS='NEW',FORM='FORMATTED',
     &        ACCESS='SEQUENTIAL',BLANK='ZERO',ERR=20)
        RETC = 1
      ENDIF
      GOTO 30
   10 CALL ERRFIL('Occurred while looking for file',FILNAM)
      RETC = -3
      GOTO 30
   20 CALL ERRFIL('Occurred while opening file',FILNAM)
      RETC = -2
   30 RETURN
      END
C
C
C
      SUBROUTINE OPNRDU
     I                 ( MSG, IUNIT,
     M                   FILNAM, OKFLAG )
C
C     + + + PURPOSE + + +
C     Opens an unformatted file for program input.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) MSG, FILNAM
      INTEGER IUNIT
      LOGICAL OKFLAG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MSG    - Character string array describing value needed
C     IUNIT  - The logical file unit number of the file
C     FILNAM - The name of the file, with length given from calling pgm
C     OKFLAG - Flag to indicate file names as specified in master
C              file were correct by user, false means new master file or update
C
C     + + + LOCAL VARIABLES + + +
      INTEGER retc
C
C     + + + EXTERNALS + + +
      EXTERNAL IOPRMP, CHKIFL
C
C     + + + END SPECIFICATIONS + + +
C
   10 IF (.NOT.OKFLAG) CALL IOPRMP(MSG,FILNAM)
C
      CALL CHKIFL(FILNAM,retc)
      IF (retc.NE.0) THEN
        OKFLAG = .FALSE.
        GOTO 10
      ENDIF
      OPEN (IUNIT,FILE=FILNAM,STATUS='OLD',FORM='UNFORMATTED',
     &      ACCESS='SEQUENTIAL')
      RETURN
      END
C
C
C
      SUBROUTINE OPNSCR
     I                 ( IUNIT )
C
C     + + + PURPOSE + + +
C     Open a scratch file for progrm input.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT  - The logical file unit number of the file.
C
C     + + + END SPECIFICATIONS + + +
C
      CLOSE (IUNIT)
      OPEN (IUNIT,STATUS='SCRATCH',FORM='FORMATTED',ACCESS='SEQUENTIAL')
      RETURN
      END
C
C
C
      SUBROUTINE OPNFUR
     I                 ( IUNIT, FILNAM )
C
C     + + + PURPOSE + + +
C     Open an unformatted file for progrm input.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT
      CHARACTER*(*) FILNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT  - The logical file unit number of the file.
C     FILNAM - The name of the file, with length given by calling 
C              program.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER iret
C
C     + + + EXTERNALS + + +
      EXTERNAL CHKIFL, OPNUFL
C
C     + + + END SPECIFICATIONS + + +
C
      CALL CHKIFL(FILNAM,iret)
      IF (iret.NE.0) STOP 'in OPNFUR'
      CALL OPNUFL(IUNIT,FILNAM)
      RETURN
      END
C
C
C
      SUBROUTINE OPNUFL
     I                 ( IUNIT, FILNAM )
C
C     + + + PURPOSE + + +
C     Open a sequential unformatted file for program input.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT
      CHARACTER*(*) FILNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT  - The logical file unit number of the file
C     FILNAM - The name of the file, with length given from calling pgm
C
C     + + + LOCAL VARIABLES + + +
      LOGICAL exists
C
C     + + + END SPECIFICATIONS + + +
C
      INQUIRE (FILE=FILNAM,EXIST=exists)
      IF (exists) THEN
        OPEN (IUNIT,FILE=FILNAM,STATUS='OLD',FORM='UNFORMATTED',
     &        ACCESS='SEQUENTIAL')
      ELSE
        OPEN (IUNIT,FILE=FILNAM,STATUS='NEW',FORM='UNFORMATTED',
     &        ACCESS='SEQUENTIAL')
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE CHKIFL
     M                 ( FILNAM,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Check whether a file of that name already exists, set retc to 1 if
C     file does not exist, set retc to 2 if error inquiring about file.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) FILNAM
      INTEGER RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     FILNAM - The name of the file, with length given from calling pgm
C     RETC   - Return code
C
C     + + + LOCAL VARIABLES + + +
      LOGICAL exists
C
C     + + + EXTERNALS + + +
      EXTERNAL ERRFIL
C
      RETC = 0
      INQUIRE (FILE=FILNAM,EXIST=exists,ERR=10)
      IF (.NOT.exists) THEN
        CALL ERRFIL('Specified file not found',FILNAM)
        RETC = 1
      ENDIF
      GOTO 20
   10 CALL ERRFIL('Occurred while looking for file',FILNAM)
      FILNAM = 'bad_file_name'
      RETC = 2
   20 RETURN
      END
C
C
C
      SUBROUTINE IOPRMP
     I                 ( MSG,
     M                   FILNAM )
C
C     + + + PURPOSE + + +
C     Prompt the user for a file name, giving the user a default name
C     and showing whether a file of that names already exists.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) FILNAM, MSG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MSG    - Character string array describing value needed.
C     FILNAM - The name of the file, character string with length from
C              calling pgm.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER retc
C
C     + + + EXTERNALS + + +
      EXTERNAL PRMPFL
C
C     + + + END SPECIFICATIONS + + +
C
      CALL PRMPFL(MSG,FILNAM,retc)
      IF (retc.NE.0) STOP
      RETURN
      END
C
C
C
      SUBROUTINE PRMPFL
     I                 ( MSG,
     M                   FILNAM,
     O                   RETC )
C
C     + + + PURPOSE + + +
C     Prompt the user for a file name, giving the user a default name
C     and showing whether a file of that names already exists, returns
C     to calling program if a quit or exit condition is encountered.
C     Remarks:
C     Units Used:  MESSGE (internal write)
C     Sets variables in the include file, which are used by FMUSER.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) FILNAM, MSG
      INTEGER RETC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MSG    - Character string array describing value needed.
C     FILNAM - The name of the file, character string with length from
C              calling pgm.
C     RETC   - Return code.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER n
      LOGICAL exists
      CHARACTER*6 ctmp
C
C     + + + INCLUDES + + +
      INCLUDE 'codmes.cmn'
C
C     + + + EXTERNALS + + +
      INTEGER LENCHR
      EXTERNAL FMUSER, ERRFIL, LENCHR
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + END SPECIFICATIONS + + +
C
      Num = 2
      Rtype = 2
      Messge(1) = 'ENTER THE FILE NAME OF THE '//MSG(:LENCHR(MSG))
   10 INQUIRE (FILE=FILNAM,EXIST=exists,err=20)
      ctmp = '(NEW) '
      IF (exists) ctmp = '(OLD) '
      n = LENCHR(FILNAM)
      IF (n.LT.1) n = LEN(FILNAM)
      Messge(2) = ctmp//FILNAM(1:n)
      CALL FMUSER(RETC)
      IF (RETC.EQ.0 .AND. Answer(:4).NE.'    ') FILNAM = Answer
      RETURN
   20 CALL ERRFIL('Invalid file name specified',FILNAM)
      FILNAM = 'badfile.nam'
      GOTO 10
      END
C
C
C
      SUBROUTINE ERRFIL
     I                 ( MSG, FILNAM )
C     + + + PURPOSE + + +
C     Display mesage about an errant file name.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) MSG, FILNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MSG    - Character string array describing value needed.
C     FILNAM - The name of the file, character string with length from
C              calling pgm.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER l, n
C
C     + + + EXTERNALS + + +
      INTEGER LENCHR
      EXTERNAL LENCHR
C
C     + + + END SPECIFICATIONS + + +
C
      l = LENCHR(MSG)
      n = LENCHR(FILNAM)
      WRITE (*,9005) MSG(:l), FILNAM(:n)
      RETURN
 9005 FORMAT (/,' **ERROR**',/,1X,A,': "',A,'"')
      END
