C
C     pltsubs.f
C
      SUBROUTINE PLTDAT
     I                 ( XP, YP, SIZE, ANGLE, IYR, IMO, IDA )
C
C     + + + PURPOSE + + +
C     Plots a date string at the given coordinates.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL XP, YP, SIZE, ANGLE
      INTEGER IYR, IMO, IDA
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XP     - X coordinate of lower left corner of date, in inches
C     YP     - Y coordinate of lower left corner of date, in inches
C     SIZE   - Size of characters to plto
C     ANGLE  - Angle in degrees to plot date
C     IYR    - Year
C     IMO    - Month of year
C     IDA    - Day of month
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER*8 date
C
C     + + + EXTERNALS + + +
      EXTERNAL SYMBOL
C
C     + + + END SPECIFICATIONS + + +
C
      WRITE (date,9005) IYR, IMO, IDA
      CALL SYMBOL(XP,YP,SIZE,date,ANGLE,8)
      RETURN
 9005 FORMAT (2(I2.2,'/'),I2.2)
      END
C
C
C
      SUBROUTINE PLTTIM
     I                 ( XP, YP, SIZE, ANGLE, IHR, IMN, ISEC )
C
C     + + + PURPOSE + + +
C     Plots a time string at the given coordinates.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL XP, YP, SIZE, ANGLE
      INTEGER IHR, IMN, ISEC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XP     - X coordinate of lower left corner of date
C     YP     - Y coordinate of lower left corner of date
C     SIZE   - Size of characters to plto
C     ANGLE  - Angle in degrees to plot date
C     IHR    - Hour
C     IMN    - Minute of hour
C     ISEC   - Second of minute
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER*8 time0
C 
C     + + + EXTERNALS + + +
      EXTERNAL SYMBOL
C
C     + + + END SPECIFICATIONS + + +
C
      WRITE (time0,9005) IHR, IMN, ISEC
      CALL SYMBOL(XP,YP,SIZE,time0,ANGLE,8)
      RETURN
 9005 FORMAT (2(I2.2,':'),I2.2)
      END
C
C
C
      SUBROUTINE PLTLAX
     I                 ( XP, YP, MESS, NCHARS, AXLEN, ANGLE, FIRVAL,
     I                   DELVAL, MSGSIZ, NDECIM, ISIDE, TICDIS, TICHGT,
     I                   NTICS)
C
C     + + + PURPOSE + + +
C     Plots a labeled axis at the given coordinates.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL XP, YP, AXLEN, ANGLE, TICDIS, TICHGT, FIRVAL, DELVAL, MSGSIZ
      INTEGER NTICS, NCHARS, NDECIM, ISIDE
      CHARACTER*(*) MESS
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XP     - X coordinate of lower left corner of date, in inches
C     YP     - Y coordinate of lower left corner of date, in inches
C     MESS   - Axis title
C     NCHARS - Number of characters in axis title
C     AXLEN  - Axis lenght in inches
C     ANGLE  - Angle in degrees to plot date
C     FIRVAL - Initial data value of axis, in data units
C     DELVAL - Number of data units per axis increment
C     MSGSIZ - Size of axis title, in inches
C     NDECIM - Number of decimal places in axis labeling (0=integer value)
C     ISIDE  - Side of axis to put labeling (-1 Lable on clockwise side,
C              1 label on counter clockwise sied)
C     TICDIS - Distance between major tic marks, in inches
C     TICHGT - Height of tic marks, in inches (<0=tic marks on clockwise
C              side, >0= tic marks on counter clockwise side).
C     NTICS  - Number of minor tic marks between major tic marks
C
C     + + + LOCAL VARIABLES + + +
      REAL xx, yy, x1, y1, disx, disy, radian, sizlbl, rangl, cosang, 
     &     xtic, ytic, sinang, sinan2, cosan2, value, sizhf, rangl2, 
     &     angl2, disyx
      INTEGER i, n, ndigit, numflg
      LOGICAL lside
C 
C     + + + EXTERNALS + + +
      EXTERNAL PLTBAX, PLOT, NUMBER, SYMBOL
C 
C     + + + INTRINSICS + + +
      INTRINSIC ABS, IFIX, COS, SIN, ALOG10
C 
C     + + + DATA + + +
      DATA radian/.01745329/
C
C     + + + END SPECIFICATIONS + + +
C
      xx = XP
      yy = YP
      numflg = NDECIM
      IF (NDECIM.EQ.0) numflg = -1
      CALL PLTBAX(xx,yy,AXLEN,ANGLE,TICDIS,TICHGT,NTICS)
      CALL PLOT(xx,yy,3)
      sizlbl = 0.9*ABS(TICHGT)
      IF (sizlbl.EQ.0.0) sizlbl=0.105
      n = IFIX(AXLEN/TICDIS+1.0005)
      rangl = radian*ANGLE
      cosang = COS(rangl)
      sinang = SIN(rangl)
      angl2 = ANGLE - 90.0
      rangl2 = radian*angl2
      sinan2 = SIN(rangl2)
      cosan2 = COS(rangl2)
      value = FIRVAL
      xtic = TICDIS*cosang
      ytic = TICDIS*sinang
      sizhf = sizlbl*0.5
      lside = ISIDE.EQ. - 1
      disyx = 0.0
      DO 10 i = 1, n
        ndigit = 1
        IF (ABS(value).GT.1) ndigit = IFIX(ALOG10(ABS(value)+0.05))
     &                                + ndigit
        IF (value.LT.0.0) ndigit = ndigit + 1
        IF (NDECIM.GT.0) ndigit = ndigit + NDECIM + 1
        IF (lside) THEN
          disx = ndigit*sizhf
          IF (TICHGT.GT.0.0) THEN
            disy = sizlbl*1.7
          ELSE
            disy = -TICHGT + 1.7*sizlbl
          ENDIF
          IF (disy.GT.disyx) disyx = disy
          x1 = xx - disx*cosang + disy*cosan2
          y1 = yy - disx*sinang + disy*sinan2
          CALL NUMBER(x1,y1,sizlbl,value,ANGLE,numflg)
        ELSE
          IF (TICHGT.GT.0.0) THEN
            disx = TICHGT + ndigit*sizlbl + sizhf
          ELSE
            disx = ndigit*sizlbl + sizhf
          ENDIF
          IF (disx.GT.disyx) disyx = disx
          disy = sizhf
          x1 = xx - disx*cosan2 + disy*ABS(cosang)
          y1 = yy - disx*sinan2 - disy*sinang
          CALL NUMBER(x1,y1,sizlbl,value,angl2,numflg)
        ENDIF
        value = value + DELVAL
        xx = xx + xtic
        yy = yy + ytic
   10 CONTINUE
C
      xx = XP
      yy = YP
      IF (lside) THEN
        disx = (AXLEN-NCHARS*MSGSIZ)*0.5
        disy = 1.7*MSGSIZ + disyx
        x1 = xx + disx*cosang + disy*cosan2
        y1 = yy + disy*sinan2 + disx*sinang
      ELSE
        disx = 0.7*MSGSIZ + disyx
        disy = (AXLEN-NCHARS*MSGSIZ)*0.5
        x1 = xx - disx*cosan2 + disy*cosang
        y1 = yy - disx*sinan2 + disy*sinang
      ENDIF
      CALL SYMBOL(x1,y1,MSGSIZ,MESS,ANGLE,NCHARS)
      RETURN
      END
C
C
C
      SUBROUTINE PLTLIN
     I                 ( XARR, YARR, NPTS, KCYCLE, CRVTYP, IPEN, ISYM,
     I                   XMIN, YMIN, XSCL, YSCL )
C
C     + + + PURPOSE + + +
C     Plots a line of a particular curve type.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NPTS, KCYCLE, CRVTYP, ISYM, IPEN
      REAL XARR(NPTS), YARR(NPTS), XMIN, YMIN, XSCL, YSCL
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XARR   - Array contianing data values for the x coordinate
C     YARR   - Array contianing data values for the y coordinate
C     NPTS   - Number of data points examined in XARR or YARR
C     KCYCLE - Increment used in gathering data from the two arrays,
C              default is 1
C     CRVTYP - Type of curve drawn.  (0 = straight line connects every
C              KCYCLE points without markers, >0 = straight line connects
C              every kcycle points with a marker drawn every CRVTYP point,
C              <0 = only markers drawn every CRVTYP point, i.e. no 
C              connecting lines between markers.
C     IPEN  - Pen number, the same meaning as IPEN argument to routine PLOT.
C     ISYM  - Index number of marker to be drawn.
C     XMIN  - Minimum data value of plot in X direction, same as in PLTLAX.
C     YMIN  - Minimum data value of plot in Y direction, same as in PLTLAX.
C     XSCL  - Scale factor in X direction (number of values/inch).
C     YSCL  - Scale factor in Y direction (number of values/inch).
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, inc, incsym, ipts, j, ip
      REAL x, y, xf, yf, xo, yo
C 
C     + + + EXTERNALS + + +
      EXTERNAL PLOT, SYMBOL
C 
C     + + + INTRINSICS + + +
      INTRINSIC IABS
C 
C     + + + END SPECIFICATIONS + + +
C
      inc = KCYCLE
      IF (inc.EQ.0) inc = 1
      ipts = NPTS
      ip = IPEN
      IF (IABS(ip).LT.2 .OR. IABS(ip).GT.9) ip = 2
      IF (XSCL.EQ.0) THEN
        i = ipts + 1
        xo = XARR(i)
        yo = YARR(i)
        i = i + 1
        xf = 1.0/XARR(i)
        yf = 1.0/YARR(i)
      ELSE
        xo = XMIN
        yo = YMIN
        xf = 1.0/XSCL
        yf = 1.0/YSCL
      ENDIF
      incsym = 0
      IF (CRVTYP.LT.0) THEN
        ip = 3
        incsym = -CRVTYP
      ELSEIF (CRVTYP.GT.0) THEN
        incsym = CRVTYP
      ENDIF
      x = (XARR(1)-xo)*xf
      y = (YARR(1)-yo)*yf
      CALL PLOT(x,y,3)
      IF (incsym.EQ.0) THEN
        DO 10 i = 1 + inc, ipts, inc
          x = (XARR(i)-xo)*xf
          y = (YARR(i)-yo)*yf
          CALL PLOT(x,y,ip)
   10   CONTINUE
      ELSE
        IF (incsym.GT.0) CALL SYMBOL(999.,999.,.08,ISYM,0.,-1)
        j = 1
        DO 20 i = 1 + inc, ipts, inc
          x = (XARR(i)-xo)*xf
          y = (YARR(i)-yo)*yf
          CALL PLOT(x,y,ip)
          IF (j.EQ.incsym) THEN
            CALL SYMBOL(999.,999.,.08,ISYM,0.,-1)
            j = 0
          ENDIF
          j = j + 1
   20   CONTINUE
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE PLTBAX(XP,YP,AXLEN,ANGLE,TICDIS,TICHGT,NTICS)
C
C     + + + PURPOSE + + +
C     Plots an unlabeled axis at the given coordinates.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL XP, YP, AXLEN, ANGLE, TICDIS, TICHGT
      INTEGER NTICS
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XP     - X coordinate of lower left corner of date, in inches
C     YP     - Y coordinate of lower left corner of date, in inches
C     AXLEN  - Axis lenght in inches
C     ANGLE  - Angle in degrees to plot date
C     TICDIS - Distance between major tic marks, in inches
C     TICHGT - Height of tic marks, in inches (<0=tic marks on clockwise
C              side, >0= tic marks on counter clockwise side).
C     NTICS  - Number of minor tic marks between major tic marks
C
C     + + + LOCAL VARIABLES + + +
      REAL xx, yy, x1, y1, tick, radian, ticang, rangl, trangl, xtic, 
     &     ytic, xtich, ytich, xtichf, ytichf
      INTEGER i, n, j, nstics
C 
C     + + + EXTERNALS + + +
      EXTERNAL PLOT
C 
C     + + + INTRINSICS + + +
      INTRINSIC IFIX, SIGN, ABS, COS, SIN
C 
C     + + + DATA + + +
      DATA radian/.01745329/
C
C     + + + END SPECIFICATIONS + + +
C
      xx = XP
      yy = YP
      nstics = NTICS + 1
      CALL PLOT(xx,yy,3)
      tick = TICHGT
      ticang = SIGN(90.0,tick) + ANGLE
      tick = ABS(tick)
      n = IFIX(AXLEN/TICDIS+0.0005)
      rangl = radian*ANGLE
      trangl = radian*ticang
      xtic = (TICDIS*COS(rangl))/nstics
      ytic = (TICDIS*SIN(rangl))/nstics
      xtich = tick*COS(trangl)
      ytich = tick*SIN(trangl)
      xtichf = xtich*0.5
      ytichf = ytich*0.5
      x1 = xx + xtich
      y1 = yy + ytich
      CALL PLOT(x1,y1,2)
      CALL PLOT(xx,yy,2)
      DO 20 i = 1, n
        xx = xx + xtic
        yy = yy + ytic
        CALL PLOT(xx,yy,2)
        DO 10 j = 1, NTICS
          x1 = xx + xtichf
          y1 = yy + ytichf
          CALL PLOT(x1,y1,2)
          CALL PLOT(xx,yy,2)
          xx = xx + xtic
          yy = yy + ytic
          CALL PLOT(xx,yy,2)
   10   CONTINUE
        x1 = xx + xtich
        y1 = yy + ytich
        CALL PLOT(x1,y1,2)
        CALL PLOT(xx,yy,2)
   20 CONTINUE
      RETURN
      END
