      SUBROUTINE DATCOD
     M                 ( ETIME, YR,
     O                   MO, DA, HR, MN )
C
C     + + + PURPOSE + + +
C     Decode an encoded date and time to its components.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER ETIME, YR, MO, DA, HR, MN
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ETIME  - Elapsed time in minutes from Jan 1 of YR
C     YR     - Year
C     MO     - Month
C     DA     - Day
C     HR     - Hour
C     MN     - Minute
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, irem, jdayn, dperm(12)
C
C     + + + INTRINSICS + + +
      INTRINSIC MOD
C
C     + + + EXTERNALS + + +
      INTEGER DAYFEB, DAYNYR
      EXTERNAL DAYFEB, DAYNYR
C
C     + + + DATA + + +
      SAVE dperm
      DATA dperm/31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
C
C     + + + END SPECIFICATIONS + + +
C
      jdayn = (ETIME-1)/1440 + 1
   10 irem = DAYNYR(YR)
      IF (jdayn.GT.irem) THEN
        jdayn = jdayn - irem
        ETIME = ETIME - irem*1440
        YR = YR + 1
        IF (YR.GT.99) YR = YR - 100
        GOTO 10
      ENDIF
      irem = jdayn
      dperm(2) = DAYFEB(YR)
      DO 20 i = 1, 12
        IF (irem.GT.dperm(i)) THEN
          irem = irem - dperm(i)
        ELSE
          MO = i
          GOTO 30
        ENDIF
   20 CONTINUE
   30 DA = irem
      irem = ETIME - (jdayn-1)*1440
      HR = irem/60
      MN = MOD(irem,60)
      IF (HR.EQ.0 .AND. MN.EQ.0) THEN
        HR = 24
        DA = DA - 1
        IF (DA.EQ.0) THEN
          MO = MO - 1
          IF (MO.EQ.0) THEN
            YR = YR - 1
            IF (YR.LT.0) YR = 99
            MO = 12
          ENDIF
          DA = dperm(MO)
        ENDIF
      ENDIF
      RETURN
      END
C
      SUBROUTINE DTCODE
     I                 ( YR, MO, DA, HR, MN,
     O                   ITIME, ETIME, IRET )
C
C     + + + PURPOSE + + +
C     Encoded date and time in two forms.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER YR, MO, DA, HR, MN, ITIME, ETIME, IRET
C
C     + + + ARGUMENT DEFINITIONS + + +
C     YR     - Year
C     MO     - Month
C     DA     - Day
C     HR     - Hour
C     MN     - Minute
C     ITIME  - Encoded date/time YRJULHRMN
C     ETIME  - Elapsed time in minutes from Jan 1 of year
C     IRET   - Return code
C
C     + + + LOCAL VARIABLES + + +
      INTEGER jdayn, dperm(12)
C
C     + + + EXTERNALS + + +
      INTEGER DAYFEB, JULDAY
      EXTERNAL JDCODE, JULDAY, DAYFEB
C
C     + + + DATA + + +
      SAVE dperm
      DATA dperm/31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
C
C     + + + END SPECIFICATIONS + + +
C
      dperm(2) = DAYFEB(YR)
      IF (HR+MN.EQ.0 .OR. YR.LT.0 .OR. YR.GT.99 .OR. MO.LT.1 .OR. 
     &    MO.GT.12 .OR. DA.LT.1 .OR. DA.GT.dperm(MO) .OR. HR.LT.0 .OR. 
     &    HR.GT.24 .OR. MN.LT.0 .OR. MN.GT.59) THEN
        IRET = 1
      ELSE
        jdayn = JULDAY(YR,MO,DA)
        CALL JDCODE(YR,jdayn,HR,MN,ITIME,ETIME)
        IRET = 0
      ENDIF
      RETURN
      END
C
      SUBROUTINE DECODE
     I                 ( ITIME,
     O                   YR, MO, DA, HR, MN, JDAYN, ETIME )
C
C     + + + PURPOSE + + +
C     Decode date and time from YRJULHRMN to Julian Day and elapsed time
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER ITIME, YR, MO, DA, HR, MN, JDAYN, ETIME
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ITIME  - Encoded date/time YRJULHRMN
C     YR     - Year 
C     MO     - Month
C     DA     - Day
C     HR     - Hour
C     MN     - Minute
C     JDAYN  - Julian day number
C     ETIME  - Elapsed time in minutes from Jan 1 of year
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, isv, dperm(12)
C
C     + + + EXTERNALS + + +
      INTEGER DAYFEB
      EXTERNAL DAYFEB
C
C     + + + DATA + + +
      SAVE dperm
      DATA dperm/31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
C
C     + + + END SPECIFICATIONS + + +
C
      YR = ITIME/10000000
      isv = ITIME - YR*10000000
      JDAYN = isv/10000
      isv = isv - JDAYN*10000
      HR = isv/100
      MN = isv - HR*100
      dperm(2) = DAYFEB(YR)
      isv = JDAYN
      DO 10 i = 1, 12
        MO = i
        IF (isv.LE.dperm(i)) GOTO 20
        isv = isv - dperm(i)
   10 CONTINUE
   20 DA = isv
      ETIME = (JDAYN-1)*1440 + HR*60 + MN
      RETURN
      END
C
      SUBROUTINE JDCODE
     I                 ( YR, JDAYN, HR, MN,
     O                   ITIME, ETIME )
C
C     + + + PURPOSE + + +
C     Encode date and time to YRJULHRMN
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER YR, JDAYN, HR, MN, ITIME, ETIME
C
C     + + + ARGUMENT DEFINITIONS + + +
C     YR     - Year
C     JDAYN  - Julian day number
C     HR     - Hour
C     MN     - Minute
C     ITIME  - Encoded date/time YRJULHRMN
C     ETIME  - Elapsed time in minutes from Jan 1 of year
C
C     + + + END SPECIFICATIONS + + +
C
      ITIME = YR*10000000 + JDAYN*10000 + HR*100 + MN
      ETIME = (JDAYN-1)*1440 + HR*60 + MN
      RETURN
      END
C
      SUBROUTINE TIMER
     M                ( SEC )
C
C     + + + PURPOSE + + +
C     THIS SUBROUTINE IS USED TO RETRIEVE ELAPSED SYSTEM TIME.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL SEC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     SEC    - NUMBER OF SECONDS ELAPSED SINCE LAST CALL TO TIMER.
C
C     + + + LOCAL VARIABLES + + +
      REAL time0, timen
      INTEGER dd, mm, yy, init, yr, mo, da, ndays, i, juldd, julda
      INTEGER DAYNYR, JULDAY
      EXTERNAL SYSDATE, SYSSEC, JULDAY, DAYNYR
      INTRINSIC FLOAT
C
C     + + + DATA INITIALIZATIONS + + +
      SAVE time0, yy, mm, dd, init
      DATA init/0/
C
C     + + + END SPECIFICATIONS + + +
C
      IF (init.EQ.0) THEN
        init = 1
        CALL SYSDATE(yy,mm,dd)
        CALL SYSSEC(i)
        time0 = FLOAT(i/100)
        SEC = time0
        RETURN
      ENDIF
      CALL SYSSEC(i)
      timen = FLOAT(i/100)
      SEC = timen - time0
      time0 = timen
      CALL SYSDATE(yr,mo,da)
      IF (da.NE.dd .OR. mo.NE.mm .OR. yr.NE.yy) THEN
        julda = JULDAY(yr,mo,da)
        juldd = JULDAY(yy,mm,dd)
C        THE NEXT STATEMENT IS A CHECK FOR CENTURY SPANNING (1999-2000)
        IF (yr.LT.yy) yr = yr + 100
        DO 10 i = yy, yr - 1
          julda = DAYNYR(i)
   10   CONTINUE
        ndays = julda - juldd
        SEC = SEC + ndays*86400
      ENDIF
      RETURN
      END
C
      INTEGER FUNCTION JULDAY
     I                       ( YR, MO, DY)
C
C     + + + PURPOSE + + +
C     THIS FUNCTION DETERMINES THE JULIAN DAY NUMBER.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER YR, MO, DY
C
C     + + + ARGUMENT DEFINITIONS + + +
C     YR     - YEAR
C     MO     - MONTH
C     DY     - DAY
C
C     + + + LOCAL VARIABLES + + +
      INTEGER daypmo(12), i
C
C     + + + EXTERNALS + + +
      INTEGER DAYFEB
      EXTERNAL DAYFEB
C
C     + + + DATA INITIALIZATIONS + + +
      DATA daypmo/31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
C
C     + + + END SPECIFICATIONS + + +
C
      daypmo(2) = DAYFEB(YR)
      JULDAY = DY
      DO 10 i = 1, MO - 1
        JULDAY = JULDAY + daypmo(i)
   10 CONTINUE
      RETURN
      END
C
      SUBROUTINE GTMNTH
     M                 ( CMONTH, IMO )
C
C     + + + PURPOSE + + +
C     DETERMINES A FOUR CHARACTER MONTH STRING IF GIVEN A MONTH NUMBER
C     OR DETERMINES THE MONTH NUMBER GIVEN A FOUR CHARACTER STRING.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*4 CMONTH
      INTEGER IMO
C
C     + + + ARGUMENT DEFINITIONS + + +
C     CMONTH - THREE CHARACTER MONTH NAME (" JAN", " FEB", etc.)
C     IMO    - MONTH
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
C
C     + + + DATA INITIALIZATIONS + + +
      CHARACTER*4 month(12)
      DATA month/' JAN', ' FEB', ' MAR', ' APR', ' MAY', ' JUN', ' JUL', 
     &     ' AUG', ' SEP', ' OCT', ' NOV', ' DEC'/
C
C     + + + END SPECIFICATIONS + + +
C
      IF (IMO.GT.0 .AND. IMO.LT.13) THEN
        CMONTH = month(IMO)
      ELSE
        DO 10 i = 1, 12
          IF (CMONTH.EQ.month(i)) THEN
            IMO = i
            GOTO 20
          ENDIF
   10   CONTINUE
        STOP '*ERROR* Month not found in GTMNTH'
      ENDIF
   20 RETURN
      END
C
      SUBROUTINE INCELT
     I                 ( IYR, EYR,
     M                   ELTIME )
C
C     + + + PURPOSE + + +
C     INCREMENT ELAPSED TIME WHEN THE ENDING TIME IS FROM A
C     DIFFERENT YEAR THAN BEGINNING TIME.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IYR, EYR, ELTIME
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IYR    - BEGINNING YEAR
C     EYR    - ENDING YEAR
C     ELTIME - ELAPSED TIME IN MINUTES FROM 1/1/IYR TO ENDING TIME
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, iy, iey
C
C     + + + EXTERNALS + + +
      INTEGER MNYEAR
      EXTERNAL MNYEAR
C
C     + + + END SPECIFICATIONS + + +
C
      iy = IYR
      iey = EYR
      IF (iy.GT.90 .AND. EYR.LT.10) iey = EYR + 100
      IF (iey.GT.iy) THEN
        DO 10 i = 1, iey-iy
          ELTIME = ELTIME + MNYEAR(iy)
          iy = iy + 1
   10   CONTINUE
      ELSEIF (iey.LT.iy) THEN
        DO 20 i = 1, iy-iey
          ELTIME = ELTIME - MNYEAR(iy)
          iy = iy - 1
   20   CONTINUE
      ENDIF
      RETURN
      END
C
      INTEGER FUNCTION MNYEAR
     I                       ( JYR )
C
C     + + + PURPOSE + + +
C     COMPUTE THE NUMBER OF MINUTES IN A YEAR WHILE CHECKING FOR
C     LEAP YEAR.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER JYR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     JYR    - YEAR
C
C     + + + LOCAL VARIABLES + + +
      INTEGER iy
C
C     + + + EXTERNALS + + +
      INTEGER DAYNYR
      EXTERNAL DAYNYR
C
C     + + + END SPECIFICATIONS + + +
C
      iy = JYR
      MNYEAR = DAYNYR(iy)*1440
      RETURN
      END
C
      INTEGER FUNCTION DAYNYR
     I                       ( JYR )
C
C     + + + PURPOSE + + +
C     COMPUTE THE NUMBER OF DAYS IN A YEAR WHILE CHECKING FOR LEAP YEAR.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER JYR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     JYR    - YEAR
C
C     + + + LOCAL VARIABLES + + +
      INTEGER iy
C
C     + + + EXTERNAL + + +
      INTEGER DAYFEB
      EXTERNAL DAYFEB
C
C     + + + END SPECIFICATIONS + + +
C
      iy = JYR
      DAYNYR = 337 + DAYFEB(iy)
      RETURN
      END
C
      INTEGER FUNCTION DAYFEB
     I                       ( JYR )
C
C     + + + PURPOSE + + +
C     COMPUTE THE NUMBER OF DAYS IN FEBRUARY WHILE CHECKING FOR LEAP YEAR.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER JYR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     JYR    - YEAR
C
C     + + + LOCAL VARIABLES + + +
      INTEGER iy
C
C     + + + INTRINSICS + + +
      INTRINSIC MOD
C
C     + + + END SPECIFICATIONS + + +
C
      iy = JYR
      DAYFEB = 28 + (4-MOD(iy,4))/4
      IF (MOD(iy,100).EQ.0) DAYFEB = 28
      RETURN
      END
