      SUBROUTINE SETFNM
     I                 ( INFILE, SUFFX,
     O                   OUTFIL )
C
C     + + + PURPOSE + + +
C     Name a file based on the file name of another file minus any up to
C     three letter suffix (base name) and the supplied file name suffix.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) INFILE, OUTFIL
      CHARACTER*4 SUFFX
C
C     + + + ARGUMENT DEFINITIONS + + +
C     INFILE - File name to determine base name.
C     SUFFX  - Suffix to append to generated file name.
C     OUTFIL - File name generated.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER namlen
C
C     + + + EXTERNALS + + +
      EXTERNAL FILPRE
C
C     + + + END SPECIFICATIONS + + +
C
      CALL FILPRE(INFILE,namlen)
      OUTFIL = INFILE
      OUTFIL(namlen+1:namlen+4) = SUFFX
      RETURN
      END
C
C
C
      SUBROUTINE FILPRE
     I                 ( INFILE,
     O                   NAMLEN )
C     + + + PURPOSE + + +
C     Determine the number of characters of a file name, minus any
C     up to three letter suffix (the base name).
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) INFILE
      INTEGER NAMLEN
C
C     + + + ARGUMENT DEFINITIONS + + +
C     INFILE - File name to determine number of characters in base name.
C     NAMLEN - Number of characters in base name.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, idot, j, numch
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + END SPECIFICATIONS + + +
C
      numch = LEN(INFILE)
      DO 20 i = numch, 1, -1
        IF (INFILE(i:i).NE.' ') THEN
          NAMLEN = i
C---determine location of last "." in name, if present
          DO 10 j = NAMLEN, 1, -1
            IF (INFILE(j:j).EQ.'.') THEN
              idot = j - 1
              GOTO 30
            ENDIF
   10     CONTINUE
          idot = 0
          GOTO 30
        ENDIF
   20 CONTINUE
      NAMLEN = numch
C---determine if an up to 3 letter suffix was found
   30 IF (idot.GT.0 .AND. NAMLEN-idot.LT.5) NAMLEN = idot
C---be sure base name allows addition of a 4 character suffix 
C---such as ".prt".
      IF (NAMLEN.GT.numch-4) NAMLEN = numch - 4
      RETURN
      END
C
C
C
      SUBROUTINE MOVEI0
     I                 ( IARRY, NN )
C
C     + + + PURPOSE + + +
C     Initialize an integer array to zero, used primarily to avoid
C     indexing of doubly subscripted arrays, by just passing the first
C     element of the arrays and the number of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN, IARRY(NN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IARRY  - Address of array to be initialized.
C     NN     - The number of elements in array to be initialized.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, left
C
C     + + + END SPECIFICATIONS + + +
C
      IF (NN.GT.7) THEN
        left = (NN/8)*8
        DO 10 i = 1, left, 8
          IARRY(i) = 0
          IARRY(i+1) = 0
          IARRY(i+2) = 0
          IARRY(i+3) = 0
          IARRY(i+4) = 0
          IARRY(i+5) = 0
          IARRY(i+6) = 0
          IARRY(i+7) = 0
   10   CONTINUE
        DO 20 i = left + 1, NN
          IARRY(i) = 0
   20   CONTINUE
      ELSE
        DO 30 i = 1, NN
          IARRY(i) = 0
   30   CONTINUE
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE MOVE0
     I                 ( ARRY, NN )
C
C     + + + PURPOSE + + +
C     Initialize an real array to zero, used primarily to avoid
C     indexing of doubly subscripted arrays, by just passing the first
C     element of the arrays and the number of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN
      REAL ARRY(NN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ARRY   - Address of array to be initialized.
C     NN     - The number of elements in array to be initialized.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, left
C
C     + + + END SPECIFICATIONS + + +
C
      IF (NN.GT.7) THEN
        left = (NN/8)*8
        DO 10 i = 1, left, 8
          ARRY(i) = 0.0
          ARRY(i+1) = 0.0
          ARRY(i+2) = 0.0
          ARRY(i+3) = 0.0
          ARRY(i+4) = 0.0
          ARRY(i+5) = 0.0
          ARRY(i+6) = 0.0
          ARRY(i+7) = 0.0
   10   CONTINUE
        DO 20 i = left + 1, NN
          ARRY(i) = 0.0
   20   CONTINUE
      ELSE
        DO 30 i = 1, NN
          ARRY(i) = 0.0
   30   CONTINUE
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE MOVEVL
     I                 ( ARRY, NN, VAL )
C
C     + + + PURPOSE + + +
C     Initialize an real array to a value, used primarily to avoid
C     indexing of doubly subscripted arrays, by just passing the first
C     element of the arrays and the number of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN
      REAL ARRY(NN), VAL
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ARRY   - Address of array to be initialized.
C     NN     - The number of elements in array to be initialized.
C     VAL    - Value to initialize array to.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, left
C
C     + + + END SPECIFICATIONS + + +
C
      IF (NN.GT.7) THEN
        left = (NN/8)*8
        DO 10 i = 1, left, 8
          ARRY(i) = VAL
          ARRY(i+1) = VAL
          ARRY(i+2) = VAL
          ARRY(i+3) = VAL
          ARRY(i+4) = VAL
          ARRY(i+5) = VAL
          ARRY(i+6) = VAL
          ARRY(i+7) = VAL
   10   CONTINUE
        DO 20 i = left + 1, NN
          ARRY(i) = VAL
   20   CONTINUE
      ELSE
        DO 30 i = 1, NN
          ARRY(i) = VAL
   30   CONTINUE
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE MOVEIA
     I                  ( NN, IARIN,
     O                    IAROUT )
C
C     + + + PURPOSE + + +
C     Move values from one integer array to another, used primarily to
C     avoid indexing of doubly subscripted arrays, by just passing
C     the first element of the arrays and the number of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN
      INTEGER IARIN(NN), IAROUT(NN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NN     - The number of elements in array to be moved.
C     IARIN  - Address of array to be moved.
C     IAROUT - Address of array to be set.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, num
C
C     + + + END SPECIFICATIONS + + +
C
      num = NN
      DO 10 i = 1, num
        IAROUT(i) = IARIN(i)
   10 CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE MOVERA
     I                  ( NN, ARIN,
     O                    AROUT )
C
C     + + + PURPOSE + + +
C     Move values from one real array to another, used primarily to
C     avoid indexing of doubly subscripted arrays, by just passing
C     the first element of the arrays and the number of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN
      REAL ARIN(NN), AROUT(NN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NN     - The number of elements in array to be moved.
C     ARIN   - Address of array to be moved.
C     AROUT  - Address of array to be set.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, num
C
C     + + + END SPECIFICATIONS + + +
C
      num = NN
      DO 10 i = 1, num
        AROUT(i) = ARIN(i)
   10 CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE MOVECA
     I                  ( NN, CARIN,
     O                    CAROUT )
C
C     + + + PURPOSE + + +
C     Move values from one character*1 array to another, used primarily
C     to avoid indexing of doubly subscripted arrays, by just passing
C     the first element of the arrays and the number of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN
      CHARACTER*1 CARIN(NN), CAROUT(NN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NN     - The number of elements in array to be moved.
C     CARIN  - Address of array to be moved.
C     CAROUT - Address of array to be set.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, num
C
C     + + + END SPECIFICATIONS + + +
C
      num = NN
      DO 10 i = 1, num
        CAROUT(i) = CARIN(i)
   10 CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE MOVAVE
     I                  ( NN, ARIN1, ARIN2,
     O                    AROUT )
C
C     + + + PURPOSE + + +
C     Move the average value of two real arrays to another, used 
C     primarily to avoid indexing of doubly subscripted arrays, by 
C     just passing the first element of the arrays and the number 
C     of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN
      REAL ARIN1(NN), ARIN2(NN), AROUT(NN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NN     - The number of elements in array to be moved.
C     ARIN1  - Address of first array to be averaged.
C     ARIN2  - Address of second array to be averaged.
C     AROUT  - Address of array to be set.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, num
C
C     + + + END SPECIFICATIONS + + +
C
      num = NN
      DO 10 i = 1, num
        AROUT(i) = (ARIN1(i)+ARIN2(i))*0.5
   10 CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE SWITCH
     I                  ( NN, 
     M                    ARRY1, ARRY2 )
C
C     + + + PURPOSE + + +
C     Switch the values in two real arrays, used primarily to avoid 
C     indexing of doubly subscripted arrays, by just passing the
C     first element of the arrays and the number of elements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NN
      REAL ARRY1(NN), ARRY2(NN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NN     - The number of elements in array to be moved.
C     ARRY1  - Address of first array to switch.
C     ARRY2  - Address of second array to switch.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, num
      REAL temp
C
C     + + + END SPECIFICATIONS + + +
C
      num = NN
      DO 10 i = 1, num
        temp = ARRY1(i)
        ARRY1(i) = ARRY2(i)
        ARRY2(i) = temp
   10 CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE REAL2LCHAR
     I                     ( FPN, NDEC,
     O                       STRING, NCHAR )
C
C     + + + PURPOSE + + +
C     Converts a real number to a left justified character string.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NDEC, NCHAR
      REAL FPN
      CHARACTER*(*) STRING
C
C     + + + ARGUMENT DEFINITIONS + + +
C     FPN    - real number to be converted.
C     NDEC   - number of digits to be included to the right of the
C              decimal point.  if <= 0, then an integer is returned.
C     STRING - character string representation of the number.
C     NCHAR  - number of characters in the string.
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER*20 hold, hold2
      INTEGER i, j
      REAL fpv
      CHARACTER cdec*2, fmt*10
C
C
C     + + + END SPECIFICATIONS + + +
C
      STRING = ' '
      NCHAR = 0
      hold2 = ' '
      fpv = FPN
      j = NDEC
      IF (j.LT.1) THEN
        WRITE (hold,'(I20)') INT(fpv)
      ELSE
        WRITE (cdec,'(i2)') j
        fmt = '(F20.'//cdec//')'
        WRITE (hold,fmt) fpv
      ENDIF
C
      DO 10 i = 1, 20
        IF (hold(i:i).NE.' ') THEN
          hold2 = hold(i:20)
          NCHAR = 20 - i + 1
          DO 20 j = 1, NCHAR
            STRING(j:j) = hold2(j:j)
   20     CONTINUE
          RETURN
       ENDIF
   10 CONTINUE
C
      STOP 'error in real2lchar'
      END
C
C
C
      SUBROUTINE INT2LCHAR
     I                    ( INUM,
     O                      STRING, NCHAR )
C
C     + + + PURPOSE + + +
C     Converts an integer number to a left justified character string.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER INUM, NCHAR
      CHARACTER*(*) STRING
C
C     + + + ARGUMENT DEFINITIONS + + +
C     INUM   - integer number to be converted.
C     STRING - character string representation of the number.
C     NCHAR  - number of characters in the string.
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER*20 hold, hold2
      INTEGER i, j
C
C     + + + END SPECIFICATIONS + + +
C
      STRING = ' '
      NCHAR = 0
      hold2 = ' '
      WRITE (hold,'(I20)') INUM
C
      DO 10 i = 1, 20
        IF (hold(i:i).NE.' ') THEN
          hold2 = hold(i:20)
          NCHAR = 20 - i + 1
          DO 20 j = 1, NCHAR
            STRING(j:j) = hold2(j:j)
   20     CONTINUE
          RETURN
       ENDIF
   10 CONTINUE
C
      STOP 'error in real2lchar'
      END
C
C
C
      INTEGER FUNCTION LENCHR
     I                       ( STRING )
C
C     + + + PURPOSE + + +
C     Get length of a character string
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) STRING
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRING - Input string to determine length of
C
C     + + + LOCAL VARIABLES + + +
      INTEGER max, i
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
      max = LEN(STRING)
      DO 10 i = max, 1, -1
        IF (STRING(i:i).NE.' ') THEN
          LENCHR = i
          GOTO 20
        ENDIF
   10 CONTINUE
      LENCHR = 0
C
   20 RETURN
      END
C
C
C
      SUBROUTINE UPCASE
     I                 ( STRING )
C
C     + + + PURPOSE + + +
C     To convert a character string from lower case to upper case
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) STRING
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRING - character string to be made upper case
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, ich, lowa, lowz, n, nn
C
C     + + + INTRINSICS + + +
      INTRINSIC ICHAR, CHAR, LEN
C
C     + + + END SPECIFICATIONS + + +
C
      lowa = ICHAR('a')
      lowz = ICHAR('z')
      nn = lowa - ICHAR('A')
      n = LEN(STRING)
      DO 10 i = 1, n
        ich = ICHAR(STRING(i:i))
C       character is lower case
        IF (ich.GE.lowa .AND. ich.LE.lowz) STRING(i:i) = CHAR(ich-nn)
   10 CONTINUE
      RETURN
      END
C
C
C
      CHARACTER*1 FUNCTION UPCS
     I                         ( CHR )
C
C     + + + PURPOSE + + +
C     To convert a character to upper case
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*1 CHR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     CHR    - character to be made upper case
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ich
C
C     + + + INTRINSICS + + +
      INTRINSIC ICHAR, CHAR
C
C     + + + END SPECIFICATIONS + + +
C
      ich = ICHAR(CHR)
      IF (ich.GE.ICHAR('a') .AND. ich.LE.ICHAR('z')) THEN
        UPCS = CHAR(ich-ICHAR('a')+ICHAR('A'))
      ELSE
        UPCS = CHR
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE CHKSTA
     M                 ( STANUM )
C
C     + + + PURPOSE + + +
C     Checks for old format (8-digit) of station identifiers, removes
C     leading zeros, and right justifies digits in 16-digit field
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STANUM*16
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STANUM - 16-digit station identifier
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, j
      CHARACTER blk*1, fmt*9, ctmp*16
C
C     + + + DATA + + +
      DATA blk/' '/
C
C     + + + END SPECIFICATIONS + + +
C
      IF (STANUM(6:6).NE.blk .AND. STANUM(7:7).NE.blk .AND. STANUM(8:8)
     &    .NE.blk .AND. STANUM(9:10).EQ.'  ') THEN
        READ (STANUM,'(I8)',ERR=10) i
        WRITE (STANUM,'(8X,I8)') i
        GOTO 60
      ENDIF
   10 IF (STANUM(1:1).EQ.'0') STANUM(1:1) = blk
      DO 20 i = 1, 15
        IF (STANUM(i:i).NE.blk) GOTO 30
        IF (STANUM(i+1:i+1).EQ.'0') STANUM(i+1:i+1) = blk
   20 CONTINUE
   30 j = 0
      DO 40 i = 16, 1, -1
        IF (STANUM(i:i).NE.blk) GOTO 50
        j = j + 1
   40 CONTINUE
   50 IF (j.NE.0 .AND. j.NE.16) THEN
        i = 16 - j
        WRITE (fmt,9005) j, i
        WRITE (ctmp,fmt) STANUM(1:i)
        STANUM = ctmp
      ENDIF
   60 RETURN
 9005 FORMAT ('(',I2,'X,A',I2,')')
      END
C
C
C
      SUBROUTINE STRLFT
     M                 ( STRING )
C
C     + + + PURPOSE + + +
C     Left justifies string, limit of 256 characters
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STRING*(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRING - String to justify
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, nl
      CHARACTER strloc*256
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + END SPECIFICATIONS + + +
C
      strloc = ' '
      nl = LEN(STRING)
      IF (nl.GT.256) STOP '*error* in strrht'
      DO 10 i = 1, nl
        IF (STRING(i:i).NE.' ') THEN
          strloc(1:nl-i+1) = STRING(i:nl)
          STRING = strloc(1:nl)
          GOTO 20
        ENDIF
   10 CONTINUE
   20 RETURN
      END
C
C
C
      SUBROUTINE STRRHT
     M                 ( STRING )
C
C     + + + PURPOSE + + +
C     Right justifies string, limit of 256 characters
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STRING*(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRING - String to justify
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, nl
      CHARACTER strloc*256
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + END SPECIFICATIONS + + +
C
      strloc = ' '
      nl = LEN(STRING)
      IF (nl.GT.256) STOP '*error* in strrht'
      DO 10 i = nl, 1, -1
        IF (STRING(i:i).NE.' ') THEN
          strloc(nl-i+1:nl) = STRING(1:i)
          STRING = strloc(1:nl)
          GOTO 20
        ENDIF
   10 CONTINUE
   20 RETURN
      END
