C   *******************************************************************
C                              DIGSGKS/DIGSDISSPLA
C     LAST MODIFIED Sep 26, 1996 -- RSR
C     MODIFIED June 17, 1993 -- RSR
C     MODIFIED May 25, 1993 -- RSR
C     MODIFIED March 4, 1993 -- RSR
C   DIGS-GKS V930601, June 1, 1993
C   -------------------------------------------------------------------
C   FORTRAN 77 collection of graphics routines which can be substituted
C   for regular calcomp calls.  These routines use calls to the GKS
C   ANSI/ISO standard FORTRAN binding, minimum level 0a.  At the moment,
C   16 colors, or 'pen numbers' are supported, but more can be added or
C   current pen color assignments changed.
C
C   Originally modified by ARLEN HARBAUGH, from HOUSTON INSTRUMENTS
C      source, using calls to DISSPLA (originally called DIGSDISSPLA)
C
C   Modified to use GKS July 1990 by Gary Granger, USGS Reston, WRD
C   Modified to use on Data General, with Prior GKS by Steve Regan 9/91
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   Modified for PC use by Tony Bouret, December 1990.
C
C   In general, modifications are in lower case, and
C   digsdisspla code is in upper case and has been deleted
C   where appropriate.
C
C   The following modifications have been made:
C
C   * Source code has been divided from one file into a separate file
C     for each routine.
C
C   * Source code has been annotated.
C
C   * User defined attributes for plotting parameters have been added
C     following routines I developed called PCPLOT, which were
C     IBM PC/AT-dependent and Microsoft FORTRAN 5.0-dependent Calcomp
C     emulation routines for screen and HPGL plotting.
C
C   * Effort has been made to ensure that all code follows standard
C     FORTRAN 77 conventions, as well as standard GKS calls.
C     This code should compile and run on any computer which has
C     GKS libraries and device drivers installed.  However, some minor
C     modification may be needed when porting to a different type of
C     computer, particularly for the initiation and shutdown of the GKS
C     system in use.
C
C   This has been tested using the Graphics Software Systems GSS*GKS
C   Kernel System (GSS-TOOLKIT and GSS*CGI Device Drivers) Version 2.15
C   for DOS and Microsoft FORTRAN 5.0 on a 80286.  These routines must be
C   compiled,C   (optionally LIBed,) and linked according to the
C   requirements of both the compiler and the GKS system in use.
C
C   No GKS libraries or device drivers are included with these routines.
C   The user must obtain this necessary GKS software in order to use
C   these routines, as well as the necessary device drivers in order
C   to run any executable programs created using these routines.
C
C   Required file:  A .CFG file which is created by these routines
C   if it does not exist.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   The following calcomp routines are included:
C
C      SUBROUTINE PLOT
C      SUBROUTINE DIGLINE
C      SUBROUTINE NEWPEN
C      SUBROUTINE PLOTS
C      SUBROUTINE SYMBOL
C      SUBROUTINE FACTOR
C      SUBROUTINE WHERE
C      SUBROUTINE ORIGIN
C  axis, scale, number, arohd, hatch are in digssubs.f
C
C   ...where some have been modified to handle special uses.
C   In general, these follow  the behavior described in a CALCOMP
C   manual except where noted.
C
C   The NEWPEN routine simply changes colors.  These routines were
C   modified as little as possible by me (GJG) except for graphics
C   and attribute setting-calls (DISSPLA calls replaced by GKS).
C   Only routines PLOT, NEWPEN, PLOTS, SYMBOL, and FACTOR were
C   modified this way.  Line labels in AXIS were moved away from
C   left margin, which may have introduced some errors or incorrect
C   actions.
C
C   The following routines are used internally to these source files to
C   handle GKS:
C
C      SUBROUTINE clrtbl
C      SUBROUTINE clsdigs
C      SUBROUTINE endplot
C      SUBROUTINE gksetplt
C      SUBROUTINE opndigs
C      SUBROUTINE setmark
C      SUBROUTINE setnrm
C      SUBROUTINE settext
C
C   NOTE: These routines expect GKS to be in a specific state at all
C   times.  This means GKS should be in state GKSCL (GKS closed) before
C   calling the plot initializing routine PLOTS.  Also, the Aspect Source
C   Flags should remain INDIVIDUAL, and the color table should not be
C   changed for any of the primitives used by digsgks.
C
C   Primitives used: Polymarker, Polyline, and Text.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   GKS calls made:
C
C      Control Functions:
C
C         Subroutine gacwk (wkid):     Activate workstation.
C                                      Integer wkid:  Workstation identifier.
C
C         Subroutine gclks:            Close GKS.
C
C         Subroutine gclrwk (wkid, cofl):  Clear workstation.
C                                      Integer wkid:  Workstation identifier.
C                                      Integer cofl:  Control flag:
C                                         0 = Conditionally,
C                                         1 = Always.
C
C         Subroutine gclwk (wkid):     Close workstation.
C                                      Integer wkid:  Workstation identifier.
C
C         Subroutine gdawk (wkid):     Deactivate workstation.
C                                      Integer wkid:  Workstation identifier.
C
C         Subroutine gopks (errfil, size):  Open GKS.
C                                      Integer errfil:  Error message file:
C                                         A logical unit number from a
C                                         standard FORTRAN OPEN statement,
C                                         which must be opened prior to
C                                         this call.
C                                      Integer size:  Size of a buffer area
C                                         in words allocated for use by
C                                         workstation-independent
C                                         segment storage.
C
C         Subroutine gopwk (wkid, conid, wtype):  Open workstation.
C                                      Integer wkid:  Workstation identifier.
C                                      Integer conid: Connection identifier
C                                         (not used with GSS*GKS on PCs).
C                                      Integer wtype: Workstation type
C                                         (same as wkid in GSS*GKS;
C                                         defined by the files KERNEL.SYS
C                                         and CGI.CFG).
C
C         Subroutine guwk (wkid, regfl):  Update Workstation
C                                      Integer wkid:  Workstation identifier.
C                                      Integer regfl: Regeneration flag:
C                                         0 = Postpone,
C                                         1 = Perform.
C
C      Output Functions:
C
C         Subroutine gpl (n, px, py):  Polyline.
C                                      Integer n:  Number of points.
C                                      Real px(n), py(n):  Coordinates of
C                                         points in world coordinates (WC).
C
C         Subroutine gpm (n, px, py):  Polymarker.
C                                      Integer n:  Number of points.
C                                      Real px(n), py(n):  Coordinates of
C                                         points in world coordinates (WC).
C
C         Subroutine gtx (px, py, chars):  Text.
C                                      Real px, py:  Alignment point of
C                                         string in world coordinates (WC).
C                                      character*(*) chars:  String.
C
C      Attribute Functions:
C
C         Subroutine gschh (chh):      Set character height.
C                                      Real chh:  Character height in
C                                         world coordinates (WC).
C
C         Subroutine gschup (chux, chuy):  Set character up vector.
C                                      Real chux, chuy:  Character up
C                                         vector in world coordinates (WC).
C
C         Subroutine gscr (wkid, ci, cr, cg, cb):  Set color representation.
C                                      Integer wkid:  Workstation identifier.
C                                      Integer ci:    Color index.
C                                      Real cr, cg, cb:  Red/green/blue
C                                         intensities.
C
C         Subroutine gsln (ltype):     Set linetype.
C                                      Integer ltype:  Linetype.
C
C         Subroutine gsmksc (mszsf):   Set marker size scale factor.
C                                      Real mszsf:  Marker size scale factor.
C
C         Subroutine gsmk (mtype):     Set marker type.
C                                      Integer mtype:  Marker type.
C
C         Subroutine gsplci (coli):    Set polyline color index.
C                                      Integer coli:  Color index.
C
C         Subroutine gspmci (coli):    Set polymarker color index.
C                                      Integer coli:  Color index.
C
C         Subroutine gstxci (coli):    Set text color index.
C                                      Integer coli:  Color index.
C
C         Subroutine gstxfp (font, fnt_prec):  Set text font and precision.
C                                      Integer font:  Text font
C                                         (device-specific).
C                                      Integer fnt_prec:  Text precision:
C                                         0 = String fnt_precision, low
C                                         1 = Character fnt_precision, medium
C                                         2 = Stroke fnt_precision. high
C
C      Transformation Functions:
C
C         Subroutine gselnt (tnr):     Set normalization transformation.
C                                      Integer tnr:  Normalization
C                                         transformation number.
C
C         Subroutine gsclip (clsw):    Set clipping indicator.
C                                      Integer clsw:  Clipping indicator.
C
C         Subroutine gsvp (tnr, xmin, xmax, ymin, ymax):  Set viewport.
C                                      Integer tnr:  Transformation number.
C                                      Real xmin, xmax, ymin, ymax:
C                                         Viewport limits (lower-left,
C                                         upper-right) in normalized device
C                                         coordinate (NDC) units.
C
C         Subroutine gswkwn (wkid, xmin, xmax, ymin, ymax):
C                                      Set workstation window.
C                                      Integer wkid:  Workstation identifier.
C                                      Real xmin, xmax, ymin, ymax:
C                                         Workstation window limits in NDC
C                                         units.
C
C         Subroutine gswn (tnr, xmin, xmax, ymin, ymax):  Set window.
C                                      Integer tnr:  Transformation number.
C                                      Real xmin, xmax, ymin, ymax:
C                                         Window limits in NDC units.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   IN SOME CASES YOU MAY WANT TO MODIFY SUBROUTINE PLOTS.  FOR
C   EXAMPLE, YOU MIGHT CHANGE THE DEFAULT PAGE SIZE (AS DESCRIBED
C   ABOVE) TO FIT YOUR PARTICULAR GRAPHICS DEVICE.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   PLOTS
C
C   Purpose:  Initializes for plotting.
C
C---- Arguments:
C
C   UNITSPEC is the unit number of the plotter output file, named within
C          the calling main program.  This file is opened and closed
C          for screen output, creating an empty file.  For HPGL output,
C          the HPGL instructions are written to this file.  If this
C          filename is identical to the port assignment of the plotting
C          device (PRN, LPT1, LPT2, LPT3, COM1, or COM2), the plot
C          is plotted directly and no file is created.
C
C   ID1 is used here to differentiate between applications, in order to
C          assign the plot configuration file name (contains assignable
C          DIGS parameters).
C          IF ID1=0, config file = DIGSPLT.CFG, else = DISGPLT(ID1).CFG,
C          e.g. if ID1=1, config file = DIGSPLT1.CFG
C
C   ID2 is unused
C
      SUBROUTINE PLOTS(UNITSPEC,ID1,ID2)
C
      INCLUDE 'digsblk.cmn'
C
C Declarations (arguments):
C
      INTEGER ID1, ID2, UNITSPEC
C
C Declarations (local variables):
C
      INTEGER iframe
      CHARACTER*72 idflag
      SAVE iframe
      DATA iframe/0/
C
C Initialize common variables; also reset in between multiple plots:
C
      idflag = '@(#) libcalgk - CalComp/Zeta to GKS graphics library'
      idflag = '@(#) libcalgk - Contact: h2osoft@usgs.gov'
      idflag = '@(#) libcalgk - Version: 6.0x 1997/02/05'
      idflag = 
     & '$Id: digsgks.f,v 6.0 1997/02/05 20:13:34 rsregan Exp rsregan $'
      Xcur = 0.0
      Ycur = 0.0
      Xorg = 0.0
      Yorg = 0.0
      Isessn = 0
C
C Make calls and initialize common variables
C to be set only before the first plot:
C
      IF (iframe.EQ.0 .OR. Mode.EQ.999) THEN
        iframe = 1
        Isessn = 1
        Otunit = ABS(UNITSPEC)
        IF (Otunit.GT.6) CLOSE (Otunit)
        Cfg = ID1
        Digfact = 1.0
        Gkfact = 1.0
        Gkxorg = 0.0
        Gkyorg = 0.0
        Colordev = .TRUE.
        Errunit = 82
        CALL GKSETPLT()
        CALL OPNDIGS()
        Last_ltype = 0
        Last_color = 1
      ENDIF
C
C Make calls and initialize variables to be set before the first plot
C and also in between multiple plots
C
      CALL NEWPEN(1)
      CALL DSETPEN(2)
      Gkxorg = Xorg + Pxoff
      Gkyorg = Yorg + Pyoff
      Gkfact = Digfact*Pltfact
      CALL PLOT(Gkxorg,Gkyorg,-3)
      RETURN
      END
C
C     SYMBOL.FOR
C
      SUBROUTINE SYMBOL(X1,Y1,HEIGHT,STRING,ANGLE,NCHAR)
C
 
C     + + + PURPOSE + + +
C     Plots annotation and special symbols.
C     Called by the main program and routines AXIS, NUMBER, and DIGLINE.
C     There are three calls to subroutine SYMBOL:
C     (1) The call to draw text such as titles and legends.
C     (2) The call to draw a single character referenced by its integer
C         index.
C     (3) The call to draw a plot, with a marker centered at X,Y.
C
C     + + + HISTORY + + +
C       AUTHOR: Steve Regan
C       SYSTEM:
C     LANGUAGE: Fortran 77
C      VERSION: June 17, 1993
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NCHAR
      REAL X1, Y1, HEIGHT, ANGLE
      CHARACTER*(*) STRING
C
C     + + +INPUT ARGUMENT DEFINITIONS + + +
C     X1,Y1    - If NCHAR > 0, these are the coordinates in inches of the
C                              lower left hand corner (before rotation) of
C                              the first character being plotted.
C              - If NCHAR <= 0, these are the coordinates of the geometric
C                               center of the marker being plotted.
C                To continue annotation from the current X and/or Y position,
C                the corresponding X1 and/or Y1 coordinates should equal 999.
C     HEIGHT   - The height in inches of the character(s) being plotted.
C                The width of the character, including spacing, is normally
C                the same as the height.
C     STRING   - If NCHAR > 0, this contains a string array of NCHAR
C                              character(s) being plotted.
C                If NCHAR <= 0, the integer index of the marker to be plotted
C                               is contained in STRING.
C     ANGLE    - The angle, in degrees from the x-axis, at which the
C                character(s) are to be plotted.  At ANGLE=0.0, the
C                character(s) will be plotted right side up and parallel
C                to the x-axis.
C     NCHAR    - If > 0,   NCHAR is the number of charcaters in STRING to be
C                          plotted, imncluding any trailing blanks.
C                If = 0/-1,A marker with integer index contained in
C                          STRING is centered at X1,Y1.
C                If < -1,  A marker is produced, where the move to X1,Y1
C                          is made with the line status the same as if
C                          the absolute value of NCHAR were the IPEN
C                          argument of subroutine PLOT.
C
      INCLUDE 'digsblk.cmn'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ipen, intgr, nc
      REAL c, s, ra, x, y, px1(1), py1(1), px, py, xm, ym, hgt
      CHARACTER*1 sym
      EQUIVALENCE (px,px1(1)), (py,py1(1))
C
C     + + + INTRINSICS + + +
      INTRINSIC ABS, IABS, COS, SIN, CHAR, ICHAR
C
C     + + + EXTERNALS + + +
      EXTERNAL PLOT, GMARK1, GMKSTR, GSCHUP, GSCHH, GTX
C
C   GKS calls made:
C
C      Output Functions:
C         Subroutine gtx (px, py, chars):  Text.
C                                      Real px, py:  Alignment point of
C                                         string in world coordinates (WC).
C                                      character*(*) chars:  String.
C      Attribute Functions:
C         Subroutine gschh (chh):      Set character height.
C                                      Real chh:  Character height in
C                                         world coordinates (WC).
C         Subroutine gschup (chux, chuy):  Set character up vector.
C                                      Real chux, chuy:  Character up
C                                         vector in world coordinates (WC).
C
C     + + + END SPECIFICATIONS + + +
C
C X,Y is the current unfactored location:
      IF (ABS(X1-999.).LT.0.0001) THEN
        x = Xcur
      ELSE
        x = X1
      ENDIF
C
      IF (ABS(Y1-999.).LT.0.0001) THEN
        y = Ycur
      ELSE
        y = Y1
      ENDIF
C
C Factor the height:
C
      hgt = HEIGHT*Gkfact
C
C Determine whether or not a line is to be drawn
C
      IF (NCHAR.GT.-2) THEN
        ipen = 3
      ELSE
        ipen = IABS(NCHAR)
      ENDIF
      nc = NCHAR
C
C Move to character location
C
      CALL PLOT(x,y,ipen)
C
C Compute location of symbol:
C
      px = (Gkxorg+x)*Gkfact
      py = (Gkyorg+y)*Gkfact
C
C Set absolute direction for character plotting (convert to radians):
C
      ra = .0174533*ANGLE
      c = COS(ra)
      s = SIN(ra)
C
C GKS Set character up vector (angle):
C
      CALL GSCHUP(-s,c)
C
C GKS Set character height:
C
      CALL GSCHH(hgt)
C
C Plot text string:
C
      IF (nc.EQ.0) THEN
C
C A single right-justified character produced.
C This character may be passed from the calling routine as a single
C character in quotes, or as the "char", the ASCII integer index of
C the character; for example, char(65) is the same as 'A'. (NCHAR is 0):
C
        IF (Mode.EQ.2) THEN
          sym = CHAR(ICHAR(STRING(4:4)))
        ELSE
          sym = STRING(1:1)
        ENDIF
        IF (Font.NE.10) THEN
C GKS Text:
          CALL GTX(px,py,sym)
        ELSE
C DIGS software generated character
          xm = hgt*0.5
          ym = xm*s
          xm = xm*c
          CALL GMARK1(ICHAR(sym),hgt,px-xm,py-ym)
        ENDIF
        xm = hgt*0.5
C
      ELSEIF (nc.GT.0) THEN
C NCHAR is greater than 0 -- use standard characters stored in STRING:
        IF (Font.NE.10) THEN
          CALL GTX(px,py,STRING(1:nc))
        ELSE
          CALL GMKSTR(STRING,nc,hgt,px,py,ANGLE)
        ENDIF
        xm = hgt*nc
      ELSE
C
C Special call to SYMBOL for a centered symbol (NCHAR is less than 0):
C The calling routine should pass the index of the symbol desired
C through STRING as the "char" of the symbol index number desired.
C For example, for symbol number 1 (a dot), pass char(1);
C this is then converted back to the integer 0 here using "ichar".
C (See MARKERS.FOR, a demonstration program of these special symbols.):
C
C Determine marker based on value of mark (reference subroutine setmark):
C
        IF (Mark.NE.0) THEN
          intgr = Mark
        ELSEIF (Mode.EQ.2) THEN
          intgr = ICHAR(STRING(4:4))
        ELSE
          intgr = ICHAR(STRING)
        ENDIF
C
        IF (Mark.NE.256) CALL GMARK1(intgr,hgt,px,py)
        xm = hgt*0.5
C
      ENDIF
C
C Calculate the new current location (end of string):
C
      ym = xm*s
      xm = xm*c
      Xcur = x + xm/Gkfact
      Ycur = y + ym/Gkfact
C
      RETURN
      END
C
C
C
      SUBROUTINE NXTPLT()
C
C     + + + PURPOSE + + +
C     Beeps to signal the end of screen plotting, and then waits for a
C     carriage return from the user to continue.  Finally, this routine
C     clears the screen.
C
C     + + + EXTERNALS + + +
      EXTERNAL PLOT, FACTOR
C
C     + + + END SPECIFICATIONS + + +
C
C Clear screen
      CALL PLOT(0.0,0.0,1000)
      CALL FACTOR(1.0)
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   PLOT
C
C   Purpose:  This subroutine is primarily called from a main program,
C             and is also called by subroutines AXIS, DIGLINE, and SYMBOL.
C             This is the fundamental plotting subroutine.
C
C---- Arguments:
C
C   X,Y are the coordinates in inches from the current origin to which
C       the imaginary pixel cursor or pen moves.
C
C   IPEN:  Determines the type of move to be made:
C
C          +2         draws solid line from previous current position.
C                     to X,Y, which becomes the new current position.
C          +3         moves current position to X,Y.
C          +4 thru +9 same as +2, but draws with a special GKS line type
C                     chosen by the user in subroutine gkSETPLT.
C          -2 thru -9 same as +IPEN except that a new origin is set at X,Y.
C          999        terminates plotting and shuts down GKS.
C          1000       clears the workstation in between multiple plots.
C          1001       clears and deactivates workstations in between
C                     multiple plots, and then reactivates workstations.
C
C---- GKS subroutines called:
C
C      Output Functions:
C
C         Subroutine gpl (n, px, py):  Polyline.
C                                      Integer n:  Number of points.
C                                      Real px(n), py(n):  Coordinates of
C                                         points in world coordinates (WC).
C
      SUBROUTINE PLOT(X,Y,IPEN)
C
      INCLUDE 'digsblk.cmn'
C
C Declarations (arguments)
C
      INTEGER IPEN
      REAL X, Y
C
C Declarations (local variables):
C
      INTEGER ip
      REAL px(2), py(2), xnew, ynew
C
C Need to use absolute value of IPEN for plotting:
C
      ip = IABS(IPEN)
C
C Factor the point to be moved to:
C
      xnew = X
      ynew = Y
C
C If a line is to be drawn, the actual movement will be from
C ORG+CUR to ORG+NEW, times the hpfact value chosen by the user
C in subroutine gkSETPLT:
C
      IF (ip.GT.1 .AND. ip.LT.10) THEN
C
        IF (ip.NE.3) THEN
          px(1) = (Gkxorg+Xcur)*Gkfact
          py(1) = (Gkyorg+Ycur)*Gkfact
C
          px(2) = (Gkxorg+xnew)*Gkfact
          py(2) = (Gkyorg+ynew)*Gkfact
C
C Set color chosen by the user in subroutine gksetplt:
C Set linestyle chosen by the user in subroutine gksetplt:
C
          CALL DSETPEN(ip)
C GKS Polyline:
          CALL GPL(2,px,py)
C
        ENDIF
        IF (IPEN.GT.0) THEN
C
C The new point is now the current location:
C
          Xcur = xnew
          Ycur = ynew
        ELSE
C
C Move origin of plot
C
          Xorg = Xorg + xnew
          Yorg = Yorg + ynew
          Gkxorg = Xorg + Pxoff
          Gkyorg = Yorg + Pyoff
          Xcur = 0.
          Ycur = 0.
        ENDIF
C
C Determine and do other actions based on value of IPEN:
C
      ELSEIF (IPEN.EQ.999) THEN
        CALL CLSDIGS()
      ELSEIF (IPEN.EQ.1000) THEN
        CALL UPDATEPLOT()
        CALL NXTPLOT()
C
        CALL PLOTS(Otunit,Cfg,IPEN)
C Delete Segment
        CALL GDSG(1)
C Create Segment
        CALL GCRSG(1)
        CALL GSVIS(1,0)
      ELSEIF (IPEN.EQ.1001) THEN
        CALL ENDPLOT()
        CALL PLOTS(Otunit,Cfg,IPEN)
      ELSE
        CALL CLSDIGS()
        WRITE (*,9005) IPEN
        STOP
      ENDIF
C
      RETURN
 9005 FORMAT (/' Invalid IPEN number: ',i5)
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  Name:       FACTOR
C
C  Purpose:    This subroutine can be called form a main program to enable
C              the user to enlarge or reduce the size of the entire plot.
C              It should be called immediately after calling subroutine
C              PLOTS.
C
C  Usage:      FACTOR (F)
C
C  Argument:
C
C     F -- The scale factor used to multiply all plotting dimensions.
C          (Input)
C
C  Remarks:   The DIGFACT variable in common /DIGSBLK/ is initially set to 1.0
C             in subroutine PLOTS.
C             This procedure has a side effect: it changes the values of
C             FACT, XCUR, YCUR, XORG, and YORG, which are passed through
C             common block /DIGSBLK/.
C
C  Keywords:  FACTOR, scaling
C
C-----------------------------------------------------------------------
C
      SUBROUTINE FACTOR(F)
C
      INCLUDE 'digsblk.cmn'
C
C Declaration (argument):
C
      REAL F
C
C Replace the old factor with the new one:
C
      Digfact = F
      Gkfact = Digfact*Pltfact
C
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  Name:       WHERE
C
C  Purpose:    This subroutine is called by subroutines AXIS and others,
C              when information concerning the current position
C              of the pixel cursor or the pen and the current scale factor
C              is needed.  Retrieves values of three variables in common
C              block /digsblk/.
C
C  Usage:      WHERE (X, Y, F)
C
C  Arguments:
C
C     X -- the current x-coordinate in inches.  (Output)
C     Y -- the current y-coordinate in inches.  (Output)
C     F -- the scale factor used to multiply all plotting dimensions.
C          (Output)
C
C  Remarks:  The arguments are the same as the XCUR, YCUR, and DIGFACT
C            variables defined in common block /DIGSBLK/ in the variable
C            dictionary at the beginning of this file.  The DIGFACT variable
C            in common /DIGSBLK/ is initially set to 1.0 in subroutine PLOTS.
C            This procedure has no side effects:
C
C  Keywords:  Where; factor
C
C-----------------------------------------------------------------------
C
      SUBROUTINE WHERE(X,Y,F)
C
      INCLUDE 'digsblk.cmn'
C
C---- Declarations (arguments):
C
      REAL X, Y, F
C
C---- Get and return the values:
C
      X = Xcur
      Y = Ycur
      F = Gkfact
C
      RETURN
      END
C
      SUBROUTINE ORIGIN(X,Y,IORIG)
C
      INTEGER IORIG
      REAL X, xold, Y, yold
C
      INCLUDE 'digsblk.cmn'
C
      IF ( IORIG.LT.0 ) THEN
        X = Gkxorg
        Y = Gkyorg
        RETURN
      ELSEIF ( IORIG.EQ.0 ) THEN
        xold = xold + Gkxorg - X
        yold = yold + Gkyorg - Y
        Gkxorg = X + Pxoff
        Gkyorg = Y + Pyoff
        RETURN
      ENDIF
      xold = xold - X
      yold = yold - Y
      Gkxorg = Gkxorg + X - Pxoff
      Gkyorg = Gkyorg + Y - Pyoff
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   NEWPEN
C
C   Purpose:  This subroutine is used to change colors.
C             Color example, 0=background, 1=foreground, 2=red, 3=green, 4=blue
C
C---- Argument:
C
C   color = color index number.
C
C---- GKS subroutines called:
C
C      Attribute Functions:
C
C         Subroutine gsplci (coli):    Set polyline color index.
C                                      Integer coli:  Color index.
C
C         Subroutine gspmci (coli):    Set polymarker color index.
C                                      Integer coli:  Color index.
C
C         Subroutine gstxci (coli):    Set text color index.
C                                      Integer coli:  Color index.
C
      SUBROUTINE NEWPEN(COLOR)
C
C Declaration (argument):
C
      INTEGER COLOR
C
C Declarations (local variables):
C
      INTEGER icolor
      INCLUDE 'digsblk.cmn'
C
C Specifications for intrinsics:
C
      INTRINSIC IABS
C
C If the user has chosen not to use colors, return:
C
      IF (Colordev) THEN
C
        icolor = IABS(COLOR)
        IF (icolor.GT.9) icolor = 1
        icolor = Linecolor(icolor)
        IF (Last_color.NE.icolor) THEN
C
C Set text color index:
C
          CALL GSTXCI(icolor)
C
C Set polyline color index:
C
          CALL GSPLCI(icolor)
C
C Set polymarker color index:
C
          CALL GSPMCI(icolor)
          Last_color = icolor
        ENDIF
      ENDIF
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  Name:      DIGLINE
C
C  Purpose:   This subroutine, called by a main program,  produces a curve
C             using the pairs of data values in the two arrays X and Y.
C             Data points may be represented by markers, connected by lines,
C             or both.
C
C  Usage:     DIGLINE (X, Y, N, INC, LINTYP, INTEQ)
C
C  Arguments:
C
C   X      -- The array of X coordinates to be plotted,
C             usually including the scaling factors.  (Input)
C
C   Y      -- The array of Y coordinates to be plotted against the
C             corresponding elements in X, usually including the scaling
C             factors.  (Input)
C
C   N      -- The number of points to pick from X and Y, not including the
C             two extra locations for scaling parameters, if present.
C             (Input)
C
C   INC    -- If N > 0, the increment for picking points from X and Y
C             (if INC is 4, every 4th point Is plotted).  (Input)
C
C             If N < 0, the increment used in gathering data is assumed to
C             be 1, INC assumes the same meaning as the IPEN argument of
C             subroutine PLOT, and the sign of LINTYP is ignored.
C
C   LINTYP -- Describes the type of curve being drawn through the data
C             points:
C
C             If < 0, Only markers are drawn, with no connecting lines.
C
C             If = 0, points are connected by straight lines and no markers
C             are drawn.
C
C             If > 0, a straight line connects every data point, and markers
C             are drawn at a frequency determined by the value of LINTYP.
C
C             If = 2, every other point has a symbol.
C
C             If = 5, points 1, 6, 11, etc. have symbols.
C
C   INTEQ  -- If LINTYP does not equal 0, this is the index of the marker
C             to be centered at each data point.
C
C             If LINTYP = 0 and N > 0, INTEQ has the same meaning as
C             IPEN argument of subroutine PLOT.
C
C
C  Remarks:   No side effects.
C
C  Keywords:  Line; symbol; marker; straight line segments.
C
C-----------------------------------------------------------------------
C
      SUBROUTINE DIGLINE(X,Y,N,INC,LINTYP,INTEQ)
C
C---- Declarations (arguments):
C
      INTEGER N, INC, LINTYP, INTEQ
      REAL X(*), Y(*)
      INCLUDE 'digsblk.cmn'
C
C---- Declarations (local variables):
C
      INTEGER l, j, incr, i, npts, ipen, linemrk, mrk
      REAL dx, dy, xmin, ymin, x1(250), y1(250), hgt
C
C---- Setup symbols and line type:
C
      IF ( (N.GT.0.AND.N/INC.GT.250) .OR. (N.LT.0.AND.N.GT.250) )
     $      STOP 'exceeded arrays in DIGS DIGLINE routine'
      IF ( Mark.EQ.0 ) THEN
        mrk = INTEQ
      ELSE
        mrk = Mark
      ENDIF
      npts = IABS(N)
      IF ( N.GT.0 ) THEN
        incr = IABS(INC)
        linemrk = LINTYP
        ipen = INTEQ
      ELSE
        incr = 1
        linemrk = IABS(LINTYP)
        ipen = INC
      ENDIF
      xmin = X(npts+1)
      dx = 1.0/X(npts+incr+1)
      ymin = Y(npts+1)
      dy = 1.0/Y(npts+incr+1)
C
      CALL DSETPEN(ipen)
C
C---- Move pen to first position
C
      x1(1) = (X(1)-xmin)*dx
      y1(1) = (Y(1)-ymin)*dy
      CALL PLOT(x1(1),y1(1),3)
C
C---- Draw the segments and symbols:
C
      CALL GQCHH(i,hgt)
      l = 0
      i = 0
      DO 100 j = 1, npts, incr
        i = i + 1
        x1(i) = ((X(j)-xmin)*dx+Gkxorg)*Gkfact
        y1(i) = ((Y(j)-ymin)*dy+Gkyorg)*Gkfact
        IF ( linemrk.GT.0 .AND. Mark.NE.256 .AND. MOD(l,linemrk).EQ.0 )
     $       CALL GMARK1(mrk,hgt*Facmrk,x1(i),y1(i))
        l = l + 1
 100  CONTINUE
      IF ( linemrk.GT.-1 ) THEN
        CALL GPL(i,x1,y1)
      ELSE
        IF ( Mark.NE.256 ) CALL GMARKN(mrk,hgt*Facmrk,x1,y1,i,0.0)
      ENDIF
C
C---- Leave the pen up:
C
      CALL PLOT((X(npts)-xmin)*dx,(Y(npts)-ymin)*dy,3)
C
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   gksetplt
C
C   Purpose:  This routine, called by subroutine PLOTS, initializes
C   user-defined variables for GKS plotting.  These are stored in
C   a .CFG file which should exist in the same directory as the executable
C   file being run.  The user is asked whether or not to modify the
C   plotting parameters.  If the user answers yes, or if the appropriate
C   .CFG file does not exist or is incomplete, the user is then prompted
C   for all the variables.  A blank carriage return in response to any
C   question retains the previous value, unless the previous value
C   did not exist.
C
C---- Arguments:  None.
C
      SUBROUTINE GKSETPLT()
C
      INCLUDE 'digsblk.cmn'
C
C Declarations (local variables):
C
      INTEGER cfgunit, cdev, iup, i, iflg, retcod
      CHARACTER cfgfile*14, msg*21
      LOGICAL fopened
C
C Initial message:
C
      WRITE (*,9040)
C
C Select the appropriate .CFG file:
C
      cfgfile = 'DIGSPLT.CFG'
      IF (Cfg.GT.0 .AND. Cfg.LT.10) WRITE (cfgfile,9030) Cfg
      IF (Cfg.GT.9 .AND. Cfg.LT.100) WRITE (cfgfile,9035) Cfg
C
C Find an unopened file unit:
C
      DO 60 cfgunit = 30, 100
        INQUIRE (cfgunit,OPENED=fopened)
        IF (.NOT.fopened) THEN
          iflg = 1
          CALL OPNRDR(cfgunit,cfgfile,iflg,retcod)
          IF (retcod.LT.0) STOP
     &                     '**error opening plotting configuration file'
C
C Set default values for .cfg file:
C
          Mode = 1
          Numws = 2
          DO 10 i = 1, 9
            Device(i) = 0
            Conid(i) = 0
            Wtype(i) = 0
            Filenm(i) = '                '
            Linecolor(i) = i
            Fileout(i) = 0
   10     CONTINUE
          Device(1) = 1
          Conid(1) = -32768
          Wtype(1) = 1100
          Device(2) = 2
          Conid(2) = 1
          Wtype(2) = 100
          Filenm(2) = 'pscript.dpo   '
          Xpage = 10.0
          Ypage = 10.0
          Pxoff = 0.0
          Pyoff = 0.0
          Irotate = 0
          Pltfact = 1.0
          Plinefact = 1.0
          cdev = 1
          Bgclr = 0
          DO 20 i = 3, 9
            Linetype(i) = i - 1
   20     CONTINUE
          Linetype(1) = 1
          Linetype(2) = 1
          Mark = 0
          Facmrk = 1.0
          Font = 1
          Fnt_prec = 2
C
C Read the .CFG file:
C
          IF (retcod.EQ.0) THEN
            READ (cfgunit,9005,end=30,err=30) Mode, Numws
            READ (cfgunit,9005,end=30,err=30) Device
            READ (cfgunit,9005,end=30,err=30) Conid
            READ (cfgunit,9005,end=30,err=30) Wtype
            READ (cfgunit,9015,end=30,err=30) Filenm
            READ (cfgunit,9025,end=30,err=30) Xpage, Ypage, Irotate
            READ (cfgunit,9020,end=30,err=30) Pxoff, Pyoff, Pltfact, 
     &                                        Plinefact
            READ (cfgunit,9005,end=30,err=30) cdev, Bgclr
            READ (cfgunit,9005,end=30,err=30) Linecolor
            READ (cfgunit,9005,end=30,err=30) Linetype
            READ (cfgunit,9010,end=30,err=30) Mark, Facmrk
            READ (cfgunit,9005,end=30,err=30) Font, Fnt_prec
            GOTO 40
   30       WRITE (*,*) 
     &             '*ERROR* reading plotting config file-verify values.'
   40       REWIND (cfgunit)
            IF (Facmrk.EQ.0.0) Facmrk = 1.0
            IF (Plinefact.EQ.0.0) Plinefact = 1.0
            IF (Pltfact.EQ.0.0) Pltfact = 1.0
          ENDIF
C
C Find out whether or not to recreate the the .CFG file:
C
C update selectively
   50     iup = 0
          WRITE (*,9045)
C
          msg = 'SELECTION CODE (0-10)'
          CALL GETIVL(msg,0,10,iup)
          IF (iup.EQ.0) THEN
C
C The new .CFG file is written:
C
            WRITE (cfgunit,9050) Mode, Numws, Device, Conid, Wtype, 
     &                           Filenm, Xpage, Ypage, Irotate, Pxoff, 
     &                           Pyoff, Pltfact, Plinefact, cdev, Bgclr, 
     &                           Linecolor, Linetype, Mark, Facmrk, 
     &                           Font, Fnt_prec
            CLOSE (cfgunit)
            Colordev = .FALSE.
            IF (cdev.EQ.1) Colordev = .TRUE.
            RETURN
          ELSEIF (iup.EQ.10) THEN
            CALL GKSETDV(Mode)
            CALL GKSETWS(Mode,Numws,Device,Wtype,Conid,Filenm,Colordev)
            CALL GKSETPG(Xpage,Ypage,Irotate)
            IF (Irotate.NE.0) THEN
              IF (Pxoff.EQ.0.0) Pxoff = -6.0
              IF (Pyoff.EQ.0.0) Pyoff = -6.0
            ENDIF
            CALL GKSETOFF(Pxoff,Pyoff,Pltfact,Plinefact)
            CALL GKSETCLR(cdev,Bgclr,Linecolor,Colordev)
            CALL GKSETLIN(Linetype)
            CALL GKSETMRK(Mark,Facmrk)
            CALL GKSETTXT(Mode,Font,Fnt_prec)
          ELSEIF (iup.EQ.1) THEN
            CALL GKSETDV(Mode)
          ELSEIF (iup.EQ.2) THEN
            CALL GKSETWS(Mode,Numws,Device,Wtype,Conid,Filenm,Colordev)
          ELSEIF (iup.EQ.3) THEN
            CALL GKSETPG(Xpage,Ypage,Irotate)
            IF (Irotate.NE.0) THEN
              IF (Pxoff.EQ.0.0) Pxoff = -6.0
              IF (Pyoff.EQ.0.0) Pyoff = -6.0
            ENDIF
          ELSEIF (iup.EQ.4) THEN
            CALL GKSETOFF(Pxoff,Pyoff,Pltfact,Plinefact)
          ELSEIF (iup.EQ.5) THEN
            CALL GKSETCLR(cdev,Bgclr,Linecolor,Colordev)
          ELSEIF (iup.EQ.6) THEN
            CALL GKSETLIN(Linetype)
          ELSEIF (iup.EQ.7) THEN
            CALL GKSETMRK(Mark,Facmrk)
          ELSEIF (iup.EQ.8) THEN
            CALL GKSETTXT(Mode,Font,Fnt_prec)
          ELSEIF (iup.EQ.9) THEN
            WRITE (*,9055) Mode, Numws, (Device(i),i=1,Numws)
            WRITE (*,9060) (Conid(i),i=1,Numws)
            WRITE (*,9065) (Wtype(i),i=1,Numws)
            WRITE (*,9070) (Filenm(i),i=1,Numws)
            WRITE (*,9075) Xpage, Ypage, Irotate, Pxoff, Pyoff, Pltfact, 
     &                     Plinefact, cdev, Bgclr, (Linecolor(i),i=1,9)
            WRITE (*,9080) (Linetype(i),i=1,9)
            WRITE (*,9085) Mark, Facmrk, Font, Fnt_prec
          ENDIF
          GOTO 50
        ENDIF
   60 CONTINUE
      STOP 'no available FORTRAN unit number'
C
C Read/Write formats:
C
 9005 FORMAT (20X,9I6)
 9010 FORMAT (20X,I6,F6.2)
 9015 FORMAT (20X,9A16)
 9020 FORMAT (20X,4F6.2)
 9025 FORMAT (20X,2F6.2,I6)
 9030 FORMAT ('DIGSPLT',I1,'.CFG')
 9035 FORMAT ('DIGSPLT',I2,'.CFG')
 9040 FORMAT (/,15x,'Welcome to the interactive graphics subsystem',/,
     &' You may enter "Q" for any prompt to discontinue. To accept the d
     &isplayed',/,
     &' or default value given in a prompt, enter a carriage return, oth
     &erwise',/,
     &' a value must be entered. The default response to a "Y/N" query i
     &s "Y".',/,
     &' Default responses are set to appropriate values.  After a plot f
     &rame is',/,
     &' completed, a beep is sounded and the program waits for a left mo
     &use click',/,
     &' (if a mouse device is available) or carriage return to continue.
     &',/)
 9045 FORMAT (/,6X,'** PLOT CONFIGURATION UPDATE MENU **'//4X,'Code',
     &        T25,'Activity',/,6X,
     &        '0  --  Finished, save current values',/,6X,
     &        '1  --  GKS software type',/,6X,
     &        '2  --  GKS workstation types',/,6X,
     &        '3  --  Page size and orientation',/,6X,
     &        '4  --  Offset and scale and line-width factors',/,6X,
     &        '5  --  Background and line colors',/,6X,
     &        '6  --  Line types'/6X,'7  --  Marker symbol/fact',/,6X,
     &        '8  --  Font name and precision',/,6X,
     &        '9  --  Display plot configuration values',/,6X,
     &        '10 --  Update all values')
 9050 FORMAT (' GKS type/no. of WS ',2I6,/' Workstation ids    ',9I6,/,
     &        ' WS connection ids  ',9I6,/' Workstation types  ',9I6,/,
     &        ' Output file names  ',9A16,/' Page size/orient   ',2F6.2,
     &        I6,/,' Offset/scale/linwid',4F6.2,/,
     &        ' Color device/backgr',2I6,/' Line colors        ',9I6,/,
     &        ' Line styles        ',9I6,/' Marker symbol/fact ',I6,
     &        F6.2,/,' Font # & precision ',2I6)
 9055 FORMAT (//,4X,'GKS type/no. of WS ',2I6,/4X,'Workstation ids    ',
     &        9I6)
 9060 FORMAT (4X,'WS connection ids  ',9I6)
 9065 FORMAT (4X,'Workstation types  ',9I6)
 9070 FORMAT (4X,'Output file names  ',9A16)
 9075 FORMAT (4X,'Page size/orient   ',2F6.2,I6,/,4X,
     &        'Offset/scale/linwid',4F6.2,/4X,'Color device/backgr',2I6,
     &        /,4X,'Line colors',I14,8I6)
 9080 FORMAT (4X,'Line styles',I14,8I6)
 9085 FORMAT (4X,'Marker symbol/fact',I7,F6.2,/,4X,
     &        'Font # & precision ',2I6,/)
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   gksetdv
C
C   Purpose:  This routine, called by subroutine gksetplt, initializes
C   user-defined variable for GKS software
C
C---- Arguments:  mode -- GKS software type 1=Prior GKS
C                                                2=DISSPLA GKS
C                                                3=GSS-GKS
C                                                4=XGKS
C                                                5=other
C
      SUBROUTINE GKSETDV(MODE)
C
      INTEGER MODE
C
C Declarations (local variables):
C
      CHARACTER msg*50
C
C Inquire GKS software type:
C
      WRITE (*,9005)
      msg = 'SELECTION CODE FOR GKS SOFTWARE TYPE (1-5)'
      CALL GETIVL(msg,1,5,MODE)
C
      RETURN
 9005 FORMAT (/,4X,'Code',5X,'GKS software',/,6X,'1  --  Prior (DG)',/,
     &        6X,'2  --  DISSPLA',/,6X,'3  --  GSS',/,6X,'4  --  XGKS',
     &        /,6X,'5  --  Other')
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   gksetws
C
C   Purpose:  This routine, called by subroutine gksetplt, initializes
C   user-defined variables for GKS software type, number of open
C   workstations, and workstation types.
C
C---- Arguments:  mode -- GKS software type 1=Prior
C                                           2=Disspla
C                                           3=GSS
C                                           4=XGKS
C                                           5=other
C                 device -- workstation identifier array
C                 wtype  -- workstation type array
C
      SUBROUTINE GKSETWS(MODE,NUMWS,DEVICE,WTYPE,CONID,FILENM,COLORDEV)
C
      INTEGER MODE, DEVICE(*), WTYPE(*), NUMWS, CONID(*), iwmax
      CHARACTER*16 FILENM(*)
      LOGICAL COLORDEV
C
C Declarations (local variables):
C
      INTEGER i, j, k
      CHARACTER msg*50, suffix*3
C
      COLORDEV = .FALSE.
C
C Determine workstation device type(s) to open:
C
      WRITE (*,9005)
C
      msg = 'NUMBER OF GKS WORKSTATIONS TO OPEN  '
      CALL GETIVL(msg,1,9,NUMWS)
C
C Inquire workstation types:
C
      WRITE (*,9010)
      iwmax = 9999
      IF (MODE.EQ.2) THEN
        WRITE (*,9015)
      ELSEIF (MODE.EQ.3) THEN
        WRITE (*,9020)
        iwmax = 6
      ELSEIF (MODE.EQ.1) THEN
        WRITE (*,9025)
        iwmax = 14
      ELSEIF (MODE.EQ.4) THEN
        WRITE (*,9040)
        iwmax = 4
      ELSE
        WRITE (*,9045)
      ENDIF
      DO 10 i = 1, NUMWS
        IF (MODE.EQ.1) THEN
          IF (WTYPE(i).EQ.1100) THEN
            WTYPE(i) = 11
          ELSEIF (WTYPE(i).EQ.1101) THEN
            WTYPE(i) = 12
          ELSEIF (WTYPE(i).EQ.4105) THEN
            WTYPE(i) = 13
          ELSEIF (WTYPE(i).EQ.7475) THEN
            WTYPE(i) = 14
          ELSEIF (WTYPE(i).GT.4) THEN
            WTYPE(i) = WTYPE(i) - 19
            IF (WTYPE(i).GT.26) WTYPE(i) = WTYPE(i) - 73
          ENDIF
        ENDIF
        WRITE (msg,'(A,I2,5X)') 'WORKSTATION TYPE OF WORKSTATION', i
        CALL GETIVL(msg,0,iwmax,WTYPE(i))
        IF (MODE.EQ.1) THEN
          IF (WTYPE(i).EQ.11 .OR. WTYPE(i).EQ.13 .OR. WTYPE(i)
     &        .EQ.14 .OR. WTYPE(i).EQ.9) COLORDEV = .TRUE.
          IF (WTYPE(i).EQ.11) THEN
            WTYPE(i) = 1100
          ELSEIF (WTYPE(i).EQ.12) THEN
            WTYPE(i) = 1101
          ELSEIF (WTYPE(i).EQ.13) THEN
            WTYPE(i) = 4105
          ELSEIF (WTYPE(i).EQ.14) THEN
            WTYPE(i) = 7475
          ELSEIF (WTYPE(i).GT.4) THEN
            WTYPE(i) = WTYPE(i) + 19
            IF (WTYPE(i).GT.26) WTYPE(i) = WTYPE(i) + 73
          ENDIF
        ENDIF
        IF (MODE.EQ.3) THEN
          DEVICE(i) = WTYPE(i)
        ELSE
          DEVICE(i) = i
        ENDIF
   10 CONTINUE
      DO 30 i = 1, NUMWS
        IF (MODE.EQ.3) THEN
          CONID(i) = 0
        ELSE
          CONID(i) = 1
          IF (MODE.EQ.1) THEN
            IF (WTYPE(i).EQ.1100) CONID(i) = -32768
            IF (WTYPE(i).EQ.1) CONID(i) = -32768
          ELSE
            WRITE (*,9030)
            WRITE (msg,'(A,I2,5X)') 
     &                     'WORKSTATION CONNECTION ID OF WORKSTATION', i
            CALL GETIVL(msg,-32768,256,CONID(i))
          ENDIF
          IF (CONID(i).EQ.1) THEN
            FILENM(i) = 'gksout.dpo'
            IF (WTYPE(i).EQ.24) FILENM(i) = 'cgmbin.dpo'
            IF (WTYPE(i).EQ.25) FILENM(i) = 'cgmtxt.dpo'
            IF (WTYPE(i).EQ.26) FILENM(i) = 'cgmchr.dpo'
            IF (WTYPE(i).EQ.3) FILENM(i) = 'gksmetatxt.dpo'
            IF (WTYPE(i).EQ.4) FILENM(i) = 'gksmetabin.dpo'
            IF (WTYPE(i).EQ.7475) FILENM(i) = 'hpgl7475.dpo'
            IF (WTYPE(i).GT.99 .AND. WTYPE(i).LT.103)
     &          FILENM(i) = 'pscript.dpo'
            DO 20 j = 1, i - 1
              IF (FILENM(j).EQ.FILENM(i)) THEN
                k = INDEX(FILENM(i),'dpo')
                WRITE (suffix,9035) 'dp', i
                FILENM(i)(k:k+2) = suffix
              ENDIF
   20       CONTINUE
          ELSE
            FILENM(i) = '      '
          ENDIF
        ENDIF
   30 CONTINUE
      RETURN
 9005 FORMAT (/,
     &' Up to 9 GKS output devices (workstations) may be selected, if av
     &ailable',/,
     &' on your computer.  For example, this package allows the simultan
     &eous',/,
     &' production of graphics image(s) on your terminal and stored in a
     & CGM file.',/,
     &' Other workstation types can include POSTSCRIPT and HPGL output f
     &iles.',/,
     &' Note, the ratio of X to Y dimensions of the first workstation is
     & used for',/,
     &' all workstations.  This ratio is used to scale the specified pag
     &e size',/,
     & ' dimensions such that the image fills the entire output area and 
     & is not',/,
     &' distorted in either dimension.  Hint, if a hardcopy is one of th
     &e desired',/,' outputs, specify it as the first workstation.')
 9010 FORMAT (/,
     &' The type for each workstation that is to be opened must be defin
     &ed.')
 9015 FORMAT (
     &' Supported CA-GKS/DISSPLA devices and workstation type numbers ar
     &e:',//,
     &' DEC VT241 (Regis): 9014   TEK 4006: 9005      TEK 4016: 9006'/
     &' HP 2623:           9010   TEK 4010,4012,4013,4051:      9001'/
     &' HP 2647, 2648:     9011   TEK 4014,4015,4052,4054,4081: 9002'/
     &' HP 7475A:          9012   TEK 4025: 9003      TEK 4027: 9004'/
     &' IBM 3179:          3179   TEK 4106,4107,4109,4115,4125: 4107'/
     &' IBM 3270 PC/G:     5279   TEK 4112: 4112      TEK 4113: 4113'/
     &' IBM 3270 PC/GX:    5379   TEK 4114, 4116:               4114'/
     &' IBM 3279:          3279   TEK 4226: 9008      TEK 4663: 9009'/
     &' ISSCO-GKS DISSPOP Metafile:'/
     &' For software characters and area fill: 102'/
     &' For hardware characters and area fill: 103'/
     &' ISSCO-GKS Output Metafile:             1')
 9020 FORMAT (' Supported GSS*GKS devices on a PC are:',//,2X,
     &        2(2X,'Code',5X,'Device Type'),8X,'Code',5X,'Device Type',
     &        /,6X,
     &'0  --  WISS           4  --  Input GKS metafile   7  --  Plotter'
     &,/,6X,
     &'1  --  Terminal       5  --  Output GKS metafile  8  --  Output C
     &GM',/,6X,'2,3 -  Input devices  6  --  Printer')
 9025 FORMAT (' Supported Prior GKS workstation types for AViiONs are:',
     &        //,4X,2('Code',5X,'Device Type',16X),/,6X,
     & '1  --  WS Independent Seg. (WISS)  8  --  PostScript-grey scale'
     & /6X,'2  --  Input generic metafile      9  --  PostScript-color',
     & /,6X,
     & '3  --  Output GKS metafile-ASCII  10  --  PostScript-monochrome'
     & ,/,6X,
     &'4  --  Output GKS metafile-BIN    11  --  X-terminal (X11) color'
     &,/,6X,
     &'5  --  Output CGM-binary          12  --  X-terminal (X11) mono',
     &/,6X,'6  --  Output CGM-clear text      13  --  Tektronix 4105',/,
     &6X,
     &'7  --  Output CGM-character       14  --  HP 7475 plotter (HPGL)'
     &)
 9030 FORMAT (/
     &' A connection id must be specified for each workstation that is t
     &o be opened.',/,
     &' For example, the connection id for an X11 window is -32768 or NU
     &LL, for others',/,
     &' it may be 0.  If the workstation produces an output file such as
     & a CGM or',/,
     &' PostScript file, enter 1 to the id query.  For NCAR GKS, enter 2
     &.  For XGKS,',/,' enter 5 for an X window and 1 for a CGM file.')
 9035 FORMAT (a2,a1)
 9040 FORMAT (/,
     &       ' For XGKS, enter 4 for an X window and 2 for a CGM file.')
 9045 FORMAT (/,
     &' For NCAR GKS, enter 1, for others please supply type numbers as 
     &appropriate.')
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   gksetpg
C
C   Purpose:  This routine, called by subroutine gksetplt, initializes
C   user-defined variables for x and y page size.
 
C---- Arguments:  xpage -- page width in inches.
C                 ypage -- page height in inches.
C                 irotate -- page orientation (0=portrait, 1=landscape)
C
      SUBROUTINE GKSETPG(XPAGE,YPAGE,IROTATE)
C
      REAL XPAGE, YPAGE
      INTEGER IROTATE
C
C---- Declarations (local variables):
      CHARACTER msg*60, str*28
C
C Inquire the page size in inches for each workstation:
C
      str = 'IN UNITS OF PLOTTING PROGRAM'
      WRITE (*,9005)
      WRITE (*,9010)
      msg = 'PAGE WIDTH '//str
      CALL GETRVL(msg,2,0.5,100.0,XPAGE)
      msg = 'PAGE HEIGHT '//str
      CALL GETRVL(msg,2,0.5,100.0,YPAGE)
      msg = 'PLOT ROTATION IN INTEGER DEGREES'
      CALL GETIVL(msg,-360,360,IROTATE)
      IF (IROTATE.NE.0) WRITE (*,9015)
C
      RETURN
 9005 FORMAT (/
     &' GKS requires that the dimensions of the graphics image be define
     &d.  These',/,
     &' dimensions (X and Y) are used to convert from image units to phy
     &sical',/,
     &' locations on the output device.  For example, a sheet of paper i
     &s 8.5 by',/,
     &' 11 inches, specification of the page size dimensions allows this
     & physical',/,
     &' space to represent 20 X 20 inches of graphics image.  Note, decr
     &easing the',/,
     &' X and Y values gives the effect of zooming the graphics image, i
     &ncreasing',/,
     &' X and Y gives the effect of panning (shrinking) the graphics ima
     &ge.  These',/,
     &' values will affect the output to all selected workstations.')
 9010 FORMAT (/,
     & ' Note, the aspect ratio (ratio of X to Y dimensions of the first 
     & workstation)',/,
     &' is used for all workstations.  This ratio is used to scale the s
     &pecified',/,
     & ' dimensions such that the image fills the entire output area and 
     & is not',/,
     &' distorted in either dimension.  Hint, if a hardcopy is one of th
     &e desired',/,' outputs, specify it as the first workstation.')
 9015 FORMAT (/,
     &' Note, in general you will need to change the X and Y offset when
     & rotating',/,
     &' the plot.  The default offset is -6.0 in each direction for a ro
     &tated plot.')
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   gksetoff
C
C   Purpose:  This routine, called by subroutine gksetplt, initializes
C   user-defined variables for plot margin offset and scaling factor.
 
C---- Arguments:  pxoff  -- left margin offset in inches.
C                 pyoff  -- bottom margin offset in inches.
C                 pltfact-- user-defined plot scaling factor.
C
      SUBROUTINE GKSETOFF(PXOFF,PYOFF,PLTFACT,PLINEFACT)
C
      REAL PXOFF, PYOFF, PLTFACT, PLINEFACT
C
C Declarations (local variables):
      CHARACTER*32 msg
C User sets the additional plotting origin, hpxorg and hpyorg
C
      WRITE (*,9005)
      msg = 'LEFT MARGIN OFFSET IN INCHES '
      CALL GETRVL(msg,2,-100.0,100.0,PXOFF)
      msg = 'BOTTOM MARGIN OFFSET IN INCHES '
      CALL GETRVL(msg,2,-100.0,100.0,PYOFF)
C
C User enters the additional plotting factor, pltfact:
C
      WRITE (*,9010)
C
      msg = 'PLOT SIZE FACTOR'
      CALL GETRVL(msg,2,0.0,100.0,PLTFACT)
      IF (PLTFACT.EQ.0.0) PLTFACT = 1.0
C
C User enters polyline width scale factor
C
      msg = 'LINE-WIDTH SCALE FACTOR (0.1-10)'
      CALL GETRVL(msg,2,0.0,10.0,PLINEFACT)
      IF (PLINEFACT.EQ.0.0) PLINEFACT = 1.0
      RETURN
 9005 FORMAT (/
     &' You may override the size of the left and bottom margins from ho
     &w they are',/,
     &' defined in the application (offset the graphics origin).  (Note,
     & values',/,' may be positive or negative.)')
 9010 FORMAT (/,
     &' The scale factor enables the enlargement or reduction of the gra
     &phics',/,
     &' image. The value entered here will be multiplied by any plot siz
     &e factor',/,
     &' inherent to the application program. Therefore, a value of 1.0 w
     &ill produce',/,
     &' a plot exactly as determined by the application. Values > 1.0 wi
     &ll enlarge',/,
     &' the graphics image with respect to the page/window size.  Values
     & < 1.0 will',/,' reduce the image.')
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   gksetclr
C
C   Purpose:  This routine, called by subroutine gksetplt, initializes
C   user-defined variables for whether color is used and the color table.
C
C---- Arguments:  cdev -- flag to indicate if a color device is being used.
C                 BGCLR  -- BACKGROUND COLOR INDEX
C                 linecolor -- color table.
C                 colordev -- output, set to true if cdev=1
C
      SUBROUTINE GKSETCLR(CDEV,BGCLR,LINECOLOR,COLORDEV)
C
      INTEGER CDEV, LINECOLOR(*), BGCLR
      LOGICAL COLORDEV
C
C Declarations (local variables):
      INTEGER i, iopt
      CHARACTER*58 msg, msga(2)
      EQUIVALENCE (msga(1),msg)
C
C Inquire the use of color.
C
      IF (.NOT.COLORDEV) THEN
        msg = 'DO ANY OF YOUR CHOSEN WORKSTATIONS SUPPORT COLOR OUTPUT'
        CALL GETYES(msg,COLORDEV)
        CDEV = 0
        IF (COLORDEV) CDEV = 1
      ENDIF
      IF (COLORDEV) THEN
        WRITE (*,9005)
C color table defined in routine CLRTBL
        WRITE (*,9010)
C
        msg = 'THE NUMBER OF A COLOR TO USE AS THE BACKGROUND COLOR'
        CALL GETIVL(msg,0,15,BGCLR)
C
C User chooses colors:
C
        iopt = 0
        WRITE (*,9015)
   10   IF (iopt.EQ.0) THEN
          WRITE (*,9020) (i,LINECOLOR(i),i=1,9)
        ELSEIF (iopt.EQ.10) THEN
          WRITE (*,9010)
        ELSE
          WRITE (msg,'(a,i3)') 'THE COLOR INDEX NUMBER FOR PEN:', iopt
          CALL GETIVL(msg,0,15,LINECOLOR(iopt))
        ENDIF
        WRITE (*,9025)
        iopt = 11
        msg = 'SELECTION CODE (0-11)'
        CALL GETIVL(msg,0,11,iopt)
        IF (iopt.NE.11) GOTO 10
      ENDIF
      RETURN
 9005 FORMAT (/
     &' Colors are selected by number.  You may choose to use the defaul
     &t color',/,
     &' definitions of your output device (refer to the device driver de
     &scriptions',/,
     &' in your GKS manual for valid numbers and the corresponding color
     &s) or use',/,
     &' the color table supplied. Note, this choice will affect the outp
     &ut to all',/,' chosen workstations.')
C
 9010 FORMAT (/,' The following is the supplied color table.'//,
     &        ' 0 - White   4 - Blue       8 - Orange   12 - Light Red'/
     &       ' 1 - Black   5 - Cyan       9 - Brown    13 - Light Green'
     &       /' 2 - Red     6 - Magenta   10 - Violet   14 - Light Blue'
     &       /' 3 - Green   7 - Yellow    11 - Grey     15 - Pink')
 9015 FORMAT (/
     &' Application programs normally select colors by "pen" number, you
     & may specify',/,
     &' the association between "pen" number and "color" number. The fol
     &lowing',/,' table shows the current values.'/)
 9020 FORMAT (6X,'Pen    Color',/,9(I8,'  --',I4,/))
 9025 FORMAT (
     & ' Enter 0 to display the current pen/color table, 1-9 to change a 
     & pen color,',/,
     &' 10 to display the supplied color table, or 11 when finished with
     & colors.')
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   gksetlin
C
C   Purpose:  This routine, called by subroutine gksetplt, initializes
C   user-defined variables for line styles.
C
C---- Arguments:  linetype -- line style array.
C
      SUBROUTINE GKSETLIN(LINETYPE)
C
      INTEGER LINETYPE(*)
C
C Declarations (local variables):
      INTEGER i, iopt
      CHARACTER*52 msg
C
C User indicates desired line styles:
C
      WRITE (*,9005)
      WRITE (*,9010)
C
      iopt = 0
      WRITE (*,9015)
   10 IF (iopt.EQ.0) THEN
        WRITE (*,9020) (i,LINETYPE(i),i=1,9)
      ELSEIF (iopt.EQ.10) THEN
        WRITE (*,9010)
      ELSE
        WRITE (msg,'(a,i3)') 'LINE TYPE NUMBER FOR LINE:', iopt
        CALL GETIVL(msg,-5,5,LINETYPE(iopt))
      ENDIF
      WRITE (*,9025)
      iopt = 11
      msg = 'SELECTION CODE (0-11)'
      CALL GETIVL(msg,0,11,iopt)
      IF (iopt.NE.11) GOTO 10
      RETURN
 9005 FORMAT (/,
     &' Line types/styles are selected by number.  You may specify the l
     &ine type',/,
     &' associated with each curve.  (Note, these choices have no effect
     & if the',/,
     &' application program generates its own line types internally.)',
     &/)
 9010 FORMAT (' The following are the valid GKS line types:',/5x,
     &        '<-2  --  Device dependent'/6x,
     &        '-2  --  Dashed with two dots'/6x,
     &        '-1  --  Medium dashed'/7x,'1  --  Solid'/7x,
     &        '2  --  Dashed'/7x,'3  --  Dotted'/7x,
     &        '4  --  Dashed-Dotted'/5x,
     &        '>=5  --  Reserved for future standardization',/)
 9015 FORMAT (
     &' The following table shows the current association between line n
     &umber and type.')
C
 9020 FORMAT (5X,'Line    Type',/,9(I8,'  --',I3,/))
 9025 FORMAT (
     &' Enter 0 to display the line/type table, 1-9 to change a line typ
     &e,',/,
     &' 10 to display valid GKS line types, or 11 when finished with lin
     &es.')
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C gksetmrk
C
C Purpose:  This subroutine is called by subroutine gksetplt
C           to allow the user to override the coordinate marker symbol
C           chosen within a main program.  GKS symbols are referenced
C           by the integers -1 and 1 through 5.  Additionally, the user may
C           choose to use an ASCII character, referenced by its decimal
C           ASCII number, numbers 33 through 126 (the low ASCII
C           characters associated with escape codes, 0-32, will not print
C           at all; nor will the extended ASCII character set, numbers
C           127-255).  A choice of 0 will plot no coordinate marker symbol,
C           and a choice of 999 will allow the the marker chosen within the
C           main program to be plotted.  This subroutine only has an effect
C           when coordinate markers are called for within a main program,
C           by the call to SYMBOL with an NCHAR value of less than 0.
C
C           Refer to the demonstration program MARKERS.FOR to see these
C           symbols.
C
C Argument:
C
C    mark - The integer value as described above.  This value is input
C           from the value saved in the appropriate .CFG file, and
C           output as chosen by the user here.
C
      SUBROUTINE GKSETMRK(MARK,FACMRK)
C
C Declaration (argument):
C
      INTEGER MARK
      REAL FACMRK
C
C Declaration (local variable):
C
      CHARACTER*56 msg
C
C Prompt the user for a coordinate marker symbol number:
C
      WRITE (*,9005)
C
      msg = 'SELECTION CODE FROM ABOVE MESSAGE '
      CALL GETIVL(msg,-5,256,MARK)
      msg = 'MARKER SIZE SCALE FACTOR '
      CALL GETRVL(msg,2,0.001,10.0,FACMRK)
      RETURN
 9005 FORMAT (/,' Coordinate marker symbols are selected by number.  You
     & can select an',/,' alternate symbol from the one specified in the
     & application program, if any.',/,
     &' Note, this maker symbol will be used in place of all other marke
     &r symbols.',//,' The following are the valid GKS marker symbols:',
     &/,6X,'<-1  --  Device implementation dependent',/,8X,
     &'1  --  bullet',/,8X,'2  --  plus',/,8X,'3  --  asterisk',/,8X,
     &'4  --  circle',/,8X,'5  --  an "X"',/,7X,
     &'>5  --  Reserved for future standardization.'//
     &' ASCII characters can also be used as coordinate markers; these a
     &re referenced',/,
     & ' by integer codes, numbered 33 to 126 (36=$,48=0,A=65,Z=90,a=97, 
     & etc).',/,
     &' Codes 6-32 have defined markers (6=box,7=triangle,8=diamond,9=oc
     &tagon,',/,' 17-20=crossed markers,21=water-surface,22=arrow,25-31=
     &filled markers).',/,' Codes 129-255 will not produce a marker.  Se
     &lection of "0" below signals to'/,
     &' retain markers as determined by the application, selection of 25
     &6 signals',/,' to not produce markers.')
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C gksettxt
C
C   Purpose:  This subroutine, called by subroutine gksetplt, inquires
C             the user for desired font and precision.
C
C   Arguments (both are input and output):
C
C      mode:  GKS software type.
C
C      font:  Integer index of desired font.
C
C      fnt_prec:  Integer index of desired precision:
C
C   GKS calls made:
C
C      Attribute Functions:
C
C         Subroutine gksettxt (mode, font, fnt_prec):  Set text font and precision.
C                                      Integer font:  Text font
C                                         (device-specific).
C                                      Integer fnt_prec:  Text precision:
C                                         0 = String precision, low
C                                         1 = Character precision, medium
C                                         2 = Stroke precision. high
C
      SUBROUTINE GKSETTXT(MODE,FONT,FNT_PREC)
C
C---- Declarations (arguments):
C
      INTEGER FONT, FNT_PREC, MODE
C
C Declarations (local variables):
C
      CHARACTER*40 msg
C
C Inquire if the current font and precision is OK:
C
C Get new font:
      WRITE (*,9005)
      CALL GETCRT()
      IF (MODE.EQ.2 .OR. MODE.EQ.3) THEN
        WRITE (*,9010)
      ELSE
        WRITE (*,9015)
      ENDIF
C
      msg = 'A FONT NUMBER FOR YOUR OUTPUT DEVICE '
      CALL GETIVL(msg,-3000,50,FONT)
C
C Get new precision:
C
      IF (FONT.NE.0 .AND. FONT.NE.10) THEN
        WRITE (*,9020)
        msg = 'THE FONT PRECISION '
        CALL GETIVL(msg,0,3,FNT_PREC)
      ENDIF
      RETURN
 9005 FORMAT (/,
     &' All fonts are GKS-implementation and/or device dependent.  Fonts
     & are',/,
     &' referenced by a GKS-assigned number. The following table shows p
     &ossible',/,
     &' font selections, see the GKS workstation description manual for 
     &valid choices',/,
     &' for your device. If you would like the default font for your cho
     &sen output',/,
     &' device(s) enter a 0. Note, the most common default font is monos
     &paced Simplex',/,
     &' Roman and is usually designated as font number 1. If you would l
     &ike a',/,' device independent font, enter a 10.'/)
 9010 FORMAT (7x,'-106 - Triplex italic software font (PC)'/7x,
     &        '-105 - Triplex software font (PC)'/7x,
     &        '-104 - Duplex software font (PC)'/7x,
     &        '-103 - Complex italic software font (PC)'/7x,
     &        '-102 - Complex software font (PC)'/7x,
     &        '-101 - Simplex software font (PC) or Hardware characters'
     &        /9x,'-1 - Proportionally spaced software characters'/10x,
     &        '0 - Device default font',/,10x,
     &        '1 - Monospaced software characters',/,10X,
     &        '2 - HPGL Bold or PostScript Courier Bold',/,10X,
     &        '3 - HPGL Italics or PostScript Courier Oblique',/,10X,
     &        '4 - PostScript Times Roman',/,10X,
     &        '5 - PostScript Times Bold',/,10X,
     &        '6 - PostScript Times Italics',/,10X,
     &        '7 - PostScript Helvetica',/,10X,
     &        '8 - PostScript Helvetica Bold',/,10X,
     &        '9 - PostScript Helvetica Oblique'/,10X,
     &        '10 - DIGS internal font',/,
     &        ' All other numbers:  A device dependent hardware font')
 9015 FORMAT (' (Note, fonts -16 through 1 are Hershey fonts)',/,
     &  4X,'-2809 - PostScript Helvetica-bold   -15 - Gothic English',/,
     &  4X,'-2808 - PostScript Helvetica        -11 - Duplex Roman',/,
     &4X,'-2805 - PostScript Times-Roman-bold -10 - Complex Cyrillic',/,
     &  4X,'-2804 - PostScript Times-Roman       -9 - Complex Italic',/,
     &  4X,'-2801 - PostScript Courier-bold      -8 - Complex Script',/,
     &  4X,'-2800 - PostScript Courier           -6 - Complex Roman',/,
     &  4X,'-2604 - X11 Courier',18X,'-5 - Simplex Script',/,
     &  4X,'-2603 - X11 char-landscape-bold      -4 - Simplex Greek',/,
     &  4X,'-2602 - X11 char-landscape           -3 - Roman-proport.',/,
     &  4X,'-2601 - X11 char-bold',16X,'-2 - Carto. Greek',/,
     &  4X,'-2600 - X11 char',21X,'-1 - Carto. Roman',/,
     &  4X,'-2000 - Tektronix 4105',16X,'0 - Device Default',/,
     &  4X,'-1200 - HPGL',26X,'1 - Simplex Roman-mono',/,
     &  7X,'10 - Device Independent')
 9020 FORMAT (/,
     &' Enter font precision. Refer to GKS manual for a description of t
     &ext precision.',/,4X,'Code',12X,'Definition',/,6X,
     &'0  --  String    (low precision)',/,6X,
     &'1  --  Character (medium precision)',/,6X,
     &'2  --  Stroke    (high precision)')
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  opndigs
C
C  Purpose:  Opens the GKS system, opens and activates all
C            requested workstations, and sets the maximum viewable area
C            for all devices.  Called by subroutines PLOTS and PLOT.
C
C  Arguments:  None.
C
C  GKS subroutines used:
C
C      Control Functions:
C
C         Subroutine gacwk (wkid):     Activate workstation.
C                                      Integer wkid:  Workstation identifier.
C
C         Subroutine gopks (errfil, size):  Open GKS.
C                                      Integer errfil:  Error message file:
C                                         A logical unit number from a
C                                         standard FORTRAN OPEN statement,
C                                         which must be opened prior to
C                                         this call.
C                                      Integer size:  Size of a buffer area
C                                         in words allocated for use by
C                                         workstation-independent
C                                         segment storage.
C
C         Subroutine gopwk (wkid, conid, wtype):  Open workstation.
C                                      Integer wkid:  Workstation identifier.
C                                      Integer conid: Connection identifier
C                                         (not used with GSS*GKS on PCs).
C                                      Integer wtype: Workstation type
C                                         (same as wkid in GSS*GKS;
C                                         defined by the files KERNEL.SYS
C                                         and CGI.CFG).
C
C      Attribute Functions:
C
C         Subroutine gstxfp (font, fnt_prec):  Set text font and precision.
C                                      Integer font:  Text font
C                                         (device-specific).
C                                      Integer fnt_prec:  Text precision:
C                                         0 = String fnt_precision, low
C                                         1 = Character fnt_precision, medium
C                                         2 = Stroke fnt_precision. high
C
C      Transformation Functions:
C
C         Subroutine gswn (tnr, xmin, xmax, ymin, ymax):  Set window.
C               Input arguments:
C                                   Integer tnr:  Transformation number.
C                                   Real xmin, xmax, ymin, ymax:
C                                      Window limits in NDC units
C                                      (lower left, upper right).
C
C         Subroutine gsvp (tnr, xmin, xmax, ymin, ymax):  Set viewport.
C               Input arguments:
C                                   Integer tnr:  Transformation number.
C                                   Real xmin, xmax, ymin, ymax:
C                                      Viewport limits (lower-left,
C                                      upper-right) in normalized device
C                                      coordinate (NDC) units.
C
C         Subroutine gswkwn (wkid, xmin, xmax, ymin, ymax):
C               Input arguments:
C                                   Set workstation window.
C                                   Integer wkid:  Workstation identifier.
C                                   Real xmin, xmax, ymin, ymax:
C                                      Workstation window limits in NDC
C                                      units (lower left, upper right).
C
C         Subroutine gsclip (clsw): Set clipping indicator.
C               Input argument:
C                                   Integer clsw:  Clipping indicator:
C                                      0 = No clip,
C                                      1 = Clip.
C
C         Subroutine gselnt (tnr):  Set normalization transformation.
C               Input argument:
C                                   Integer tnr:  Normalization
C                                      transformation number.
C
      SUBROUTINE OPNDIGS()
C
      INCLUDE 'digsblk.cmn'
C
C Declarations (local variables):
C
      LOGICAL fopened, otopen, exists
      INTEGER i, iconid, j, k
      INTEGER iclcap, idcunt, ierrin, lxx, lyy, ncol, ndef
      REAL rxx, ryy, scale, tmp, wsasp, xndc, xwc, yndc, ywc
      CHARACTER otnam*16, ctmp*4, erfil*12
C
      IF (Isessn.EQ.1) THEN
        erfil = 'GKS.errors  '
C
C Find an unopened file unit:
C
        DO 10 Errunit = 70, 100
          INQUIRE (Errunit,OPENED=fopened)
          IF (.NOT.fopened) GOTO 20
   10   CONTINUE
C
C Open Graphic Kernel System for business:
C
   20   CALL OPNFIL(Errunit,erfil)
        CALL GOPKS(Errunit,1024)
        WRITE (*,*) 'The GKS errors file is named: ', erfil
C open WISS workstation
        CALL GOPWK(13,-32768,1)
        CALL GACWK(13)
C Turn off GKS viewport clipping:
        CALL GSCLIP(0)
C GKS Set text font and fnt_precision:
        IF (Font.EQ.10 .OR. Font.EQ.0) THEN
          CALL GSTXFP(1,Fnt_prec)
        ELSE
          CALL GSTXFP(Font,Fnt_prec)
        ENDIF
C Set polyline width to PLINEFACT (orig. set to 1.5)
        CALL GSLWSC(Plinefact)
C
      ENDIF
C
C Open workstations:
C
      otopen = .TRUE.
      DO 60 i = 1, Numws
        iconid = Conid(i)
        IF (iconid.EQ.1 .AND. Isessn.EQ.1) THEN
          IF (Filenm(i).EQ.'  ') THEN
            otnam = 'GKSOUT.    '
            WRITE (ctmp,'(I4)') Wtype(i)
            j = 8
            DO 30 k = 1, 4
              IF (ctmp(k:k).NE.' ') THEN
                otnam(j:j) = ctmp(k:k)
                j = j + 1
              ENDIF
   30       CONTINUE
          ELSE
            otnam = Filenm(i)
          ENDIF
          IF (Otunit.GT.6 .AND. otopen) THEN
            iconid = Otunit
            otopen = .FALSE.
            INQUIRE (iconid,OPENED=fopened)
          ELSE
            DO 40 iconid = 70, 100
              INQUIRE (iconid,OPENED=fopened)
              IF (.NOT.fopened) GOTO 50
   40       CONTINUE
          ENDIF
   50     IF (.NOT.fopened) THEN
            INQUIRE (FILE=otnam,EXIST=exists)
            IF (exists) THEN
              OPEN (iconid,FILE=otnam,STATUS='OLD',FORM='FORMATTED',
     &              ACCESS='SEQUENTIAL')
C Truncate file
              ENDFILE iconid
              REWIND iconid
            ELSE
              WRITE (*,*) 'output file does not exist'
C Position to last line in file
              OPEN (iconid,FILE=otnam,STATUS='NEW',FORM='FORMATTED',
     &              ACCESS='SEQUENTIAL')
            ENDIF
          ENDIF
          Fileout(i) = iconid
          WRITE (*,*) 'Graphical output can be found in file: ', otnam
        ENDIF
        IF (Wtype(i).EQ.1 .AND. iconid.EQ.2) Fileout(i) = iconid
C Open workstations, if first time in OPNDIGS or X-window
        IF (Isessn.EQ.1 .OR. Fileout(i).EQ.0)
     &      CALL GOPWK(Device(i),iconid,Wtype(i))
C Activate workstations:
        CALL GACWK(Device(i))
C set deferral state to ASAP with implicit regeneration suppressed
        CALL GSDS(Device(i),0,0)
C Index colors:
        CALL GQCF(Wtype(i),ierrin,ncol,iclcap,ndef)
        IF (iclcap.NE.0) CALL CLRTBL(Device(i),Bgclr)
C
C Calculate the aspect ratio of display surface:
C
        CALL GQDSP(Wtype(i),ierrin,idcunt,rxx,ryy,lxx,lyy)
        ywc = Ypage
        xwc = Xpage
        IF (lyy.EQ.0 .OR. lxx.EQ.0) THEN
          rxx = 1500.0
          ryy = 1500.0
          lxx = 1500
          lyy = 1500
        ENDIF
        wsasp = rxx/ryy
        IF (Irotate.EQ.1) Irotate = 90
        Irotate = MOD(Irotate,360)
        IF ((Irotate.GT.45.AND.Irotate.LT.135) .OR. (Irotate.LT.-45.AND.
     &      Irotate.GT.-135)) THEN
          tmp = ywc
          ywc = xwc
          xwc = tmp
        ENDIF
C
        IF (ywc*wsasp.LT.xwc) THEN
          ywc = xwc/wsasp
        ELSE
          xwc = ywc*wsasp
        ENDIF
C
        WRITE (*,9005) Device(i), wsasp, xwc, ywc
C
C GKS Set Window (world coordinates), no border:
C
        CALL GSWN(Device(i),0.0,xwc,0.0,ywc)
        scale = AMAX1(xwc,ywc)
        xndc = xwc/scale
        yndc = ywc/scale
C
C GKS Set ViewPort in plotting coordinates:
C
        CALL GSVP(Device(i),0.0,xndc,0.0,yndc)
        Xshift(i) = xndc
C
C GKS Set Workstation window:
C
        CALL GSWKWN(Device(i),0.0,xndc,0.0,yndc)
        CALL GSWKVP(Device(i),0.0,rxx,0.0,ryy)
C       CALL GSWKWN(Device(i),0.0,1.0,0.0,1.0)
C
   60 CONTINUE
C create segment 1
      CALL GCRSG(1)
      CALL GSVIS(1,0)
C Select Normalization Transformation defined above:
      CALL GSELNT(Device(1))
C GKS Set marker type:
      IF (Mark.NE.0 .AND. Mark.NE.256) CALL GSMK(Mark)
C GKS Set marker size scale factor:
      CALL GSMKSC(Facmrk)
C
      RETURN
 9005 FORMAT (/,' Device:',I3,' aspect ratio x to y:',F8.4,/,
     &        ' X dimension (in image units):',F9.4,' Y dimension:',
     &        F9.4,/)
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C clrtbl
C
C   Purpose:  Defines colors for subsequent calls to subroutine NEWPEN;
C             called by subroutine opngks.
C
C             Colors are indexed as follows:
C
C                 0:  white
C                 1:  black
C                 2:  red
C                 3:  green
C                 4:  blue
C                 5:  cyan
C                 6:  magenta
C                 7:  yellow
C                 8:  orange
C                 9:  brown
C                10:  violet
C                11:  grey
C                12:  light red
C                13:  light green
C                14:  light blue
C                15:  pink
C
C   Arguments:
C
C      dev:  Workstation number.
C
C   GKS subroutines called:
C
C      Attribute Functions:
C
C         Subroutine gscr (wkid, ci, cr, cg, cb):  Set color representation.
C                         Integer wkid:  Workstation identifier.
C                         Integer ci:    Color index.
C                         Real cr, cg, cb:  Red/green/blue intensities.
C
C
C
      SUBROUTINE CLRTBL(DEV,BGCLR)
C
C Declaration (argument):
C
      INTEGER DEV, BGCLR
C
C Declarations (local variables):
C
      INTEGER j
      REAL rgb(3,16)
C
C Data:
C
      DATA rgb/1.00, 1.00, 1.00, 0.00, 0.00, 0.00, 1.00, 0.00, 0.00, 
     &     0.00, 1.00, 0.00, 0.00, 0.00, 1.00, 0.00, 1.00, 1.00, 1.00, 
     &     0.00, 1.00, 1.00, 1.00, 0.00, 1.00, 0.50, 0.00, 0.60, 0.30, 
     &     0.10, 0.50, 0.10, 0.67, 0.50, 0.50, 0.50, 1.00, 0.40, 0.40, 
     &     0.40, 1.00, 0.40, 0.40, 0.40, 1.00, 1.00, 0.38, 0.38/
C
C GKS Set Color Representation:
C
      rgb(1,1) = rgb(1,BGCLR+1)
      rgb(2,1) = rgb(2,BGCLR+1)
      rgb(3,1) = rgb(3,BGCLR+1)
      DO 10 j = 1, 16
        CALL GSCR(DEV,j-1,rgb(1,j),rgb(2,j),rgb(3,j))
   10 CONTINUE
C
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C clsdigs
C
C   Purpose:  This subroutine, called by subroutine PLOT when the IPEN
C             argument of subroutine PLOT equals 999, deactivates
C             and closes all open workstations, and closes GKS.
C
C   Arguments:  None.
C
C   GKS subroutines called:
C
C      Control Functions:
C
C         Subroutine gclks:         Close GKS.
C
C         Subroutine gclwk (wkid):  Close workstation.
C                                   Integer wkid:  Workstation identifier.
C
C         Subroutine gdawk (wkid):  Deactivate workstation.
C                                   Integer wkid:  Workstation identifier.
C
      SUBROUTINE CLSDIGS()
C
      INCLUDE 'digsblk.cmn'
C
C Declarations (local variables):
C
      INTEGER i, ierr, istate
C
C Close and  Delete Segment
      CALL GQOPS(istate)
      IF (istate.EQ.4) THEN
        CALL GCLSG()
        CALL GDSG(1)
      ENDIF
C
C GKS Deactivate Workstation:
C
      DO 10 i = 1, Numws
        CALL GQWKS(Device(i),ierr,istate)
        IF (istate.EQ.1) CALL GDAWK(Device(i))
   10 CONTINUE
      CALL GQWKS(13,ierr,istate)
      IF (istate.EQ.1) THEN
        CALL GDAWK(13)
        CALL GCLWK(13)
      ENDIF
C
C GKS Close Workstation:
C
      DO 20 i = 1, Numws
        CALL GQWKS(Device(i),ierr,istate)
        IF (ierr.NE.25) CALL GCLWK(Device(i))
   20 CONTINUE
C
C GKS Close GKS:
C
      CALL GCLKS()
      CLOSE (Errunit)
C set flag to allow tdds to reopen gks from begining (allows iframe
C to be set to 0 in PLOTS.
      Mode = 999
C
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   endplot
C
C   Purpose:  This subroutine, called from subroutine PLOT, beeps to signal
C             the end of screen plotting, and then waits for a carriage
C             return from the user to continue.  Finally, this routine
C             clears the screen in the case of display, or advances the
C             page in the case of a printer or plotter, and deactivates.
C
C---- Arguments:  None.
C
C   GKS subroutines called:
C
C      Control Functions:
C
C         Subroutine gwawk (wkid): Deactive workstation
C            Input arguments:
C            Integer wkid:  Workstation identifier.
C
C         Subroutine gclwk (wkid):  Close Workstation
C            Integer wkid:  Workstation identifier.
C
      SUBROUTINE ENDPLOT()
C
      INCLUDE 'digsblk.cmn'
C
C Declaration (local variable):
C
      INTEGER i, ierr, istate
C
C Update Workstation and wait for continue:
      CALL UPDATEPLOT()
      CALL NXTPLOT()
C
C GKS deactivate Workstation:
C
      DO 10 i = 1, Numws
        CALL GQWKS(Device(i),ierr,istate)
        IF (istate.EQ.1) CALL GDAWK(Device(i))
C       if terminal output, close workstation
        IF (Fileout(i).EQ.0) CALL GCLWK(Device(i))
   10 CONTINUE
C Delete Segment
      CALL GDSG(1)
      IF (Mode.EQ.2) PRINT *, '  '
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   updateplot
C
C   Purpose:  This subroutine, called from subroutine PLOT, to be sure
C             all plotting instructions are cleared from buffers.
C
C---- Arguments:  none
C
C   GKS subroutines called:
C
C      Control Functions:
C
C         Subroutine guwk (wkid, regfl):  Update Workstation
C            Integer wkid:  Workstation identifier.
C            Integer regfl: Regeneration flag:
C               0 = Postpone,
C               1 = Perform.
C
      SUBROUTINE UPDATEPLOT
C
      INCLUDE 'digsblk.cmn'
C
C Declaration (local variable):
C
      INTEGER i, ierrind, iwkcat
      REAL tout(2,3), angle
      EXTERNAL GCLSG, GSVIS, GEVTM, GSSGT, GUWK, GQWKCA
C
C Update Workstation (make sure plotting has finished):
C
C     wait is set to false if terminal output
C     do not want to wait for a reponse if only outputting to a file.
C     This could cause problems on some machines.
      Wait = .FALSE.
      CALL GCLSG()
      CALL GSVIS(1,1)
      IF (Irotate.NE.0) THEN
        angle = FLOAT(Irotate)*3.1415927/180.0
        CALL GEVTM(0.0,0.0,Xshift(1),0.0,angle,1.0,1.0,1,tout)
        CALL GSSGT(1,tout)
      ENDIF
      Ixw = 0
      DO 10 i = 1, Numws
C Update workstation
        CALL GUWK(Device(i),1)
        IF (Fileout(i).EQ.0) Wait = .TRUE.
        CALL GQWKCA(Wtype(i),ierrind,iwkcat)
        IF (Ixw.EQ.0 .AND. ierrind.EQ.0 .AND. iwkcat.EQ.2)
     &      Ixw = Device(i)
   10 CONTINUE
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C   nxtplot
C
C   Purpose:  This subroutine, called from subroutine PLOT, beeps to signal
C             the end of screen plotting, and then waits for a carriage
C             return from the user to continue.  Finally, this routine
C             clears the screen in the case of display, or advances the
C             page in the case of a printer or plotter.
C
C---- Arguments:  wait logical (input)
C
C   GKS subroutines called:
C
C      Control Functions:
C
C         Subroutine gclrwk (wkid, cofl):  Clear workstation.
C            Input arguments:
C            Integer wkid:  Workstation identifier.
C            Integer cofl:  Control flag:
C               0 = Conditionally:  The display surface is cleared only if
C                                   an object appears on it.
C               1 = Always:         The display surface is always cleared.
C            If the device is a CRT or a printer-plotter with auto-paper
C            advance, the surface is cleared without any user interaction.
C            If the device is a plotter that does not have auto-paper
C            advance, the user will be prompted to change the paper and
C            then press ENTER.
C
C         Subroutine grqlc (wkid,lcdnr,stat,tnr,px,py) Request Locator
C            Integer wkid:  Workstation identifier.
C            Integer lcdnr: Locator device number.
C            Integer stat:  Status of input request.
C            Integer tnr:   Number of normalization transformations used.
C            Real px:       X-value of locator in world coordinates.
C            Real py:       Y-value of locator in world coordinates.
C
      SUBROUTINE NXTPLOT()
C
      INCLUDE 'digsblk.cmn'
C
C Declaration (local variable):
C
      INTEGER i, istat, itnr
      REAL px, py
      CHARACTER*1 dummy
      INTRINSIC CHAR
      EXTERNAL GRQLC, GCLRWK, GETCRT
C
C wait is set to false if terminal output
C do not want to wait for a reponse if only outputting to a file.
C This could cause problems on some machines.
C
C Beep
      WRITE (*,*) CHAR(7)
C
      IF ((Mode.EQ.1.OR.Mode.EQ.4) .AND. Ixw.NE.0) THEN
C Wait for mouse click
        itnr = 0
        CALL GRQLC(Ixw,1,istat,itnr,px,py)
      ELSEIF (Wait) THEN
C If window without mouse, wait for carriage return
        CALL GETCRT
      ENDIF
C
C GKS Clear Workstation:
C
      DO 10 i = 1, Numws
        IF (Fileout(i).EQ.0) CALL GCLRWK(Device(i),0)
   10 CONTINUE
C
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  Name:      DSETPEN
C
C  Purpose:   This subroutine, called internally to digsgks, gets the proper
C             index for colors and line styles from array linedat in common
C             block DIGSBLK and call NEWPEN to set proper color.
C
C  Usage:     DSETPEN (IPEN)
C
C  Arguments:
C
C   IPEN   -- Calcomp IPEN variable,
C          +2         draws solid line from previous current position.
C                     to X,Y, which becomes the new current position.
C          +3         moves current position to X,Y.
C          +4 thru +9 same as +2, but draws with a special line type
C                     chosen by the user in subroutine gkSETPLT.
C          -2 thru -9 same as +IPEN except that a new origin is set at X,Y.
C             factors.  (Input)
C---- Argument:
C
C   ltype = a GKS linetype number, entered by the user in subroutine
C           gkSETPLT and stored in common array linestyle.
C
C---- GKS subroutines called:
C
C      Attribute Functions:
C
C         Subroutine gsln (ltype):  Set linetype.
C                                   Integer ltype:  Linetype.
C                                   Valid GKS line types are:
C                                  <-2 = Device dependent
C                                   -2 = Dashed with two dots
C                                   -1 = Medium dashed
C                                    1 = Solid
C                                    2 = Dashed
C                                    3 = Dotted
C                                    4 = Dashed-Dotted
C                                  >=5 = Reserved for future standardization
C
C  Remarks:   side effects: can change current color.
C
C  Keywords:  Line; color; straight line segments.
C
C-----------------------------------------------------------------------
C
      SUBROUTINE DSETPEN(IPEN)
C
C Declarations (arguments):
C
      INTEGER IPEN
      INCLUDE 'digsblk.cmn'
C
C Declarations (local variables):
C
      INTEGER indx, ltype
C
      indx = IABS(IPEN) - 1
      IF (indx.GT.1) indx = indx - 1
      IF (indx.GT.9) indx = 1
      ltype = Linetype(indx)
      IF (Last_ltype.NE.ltype) THEN
        CALL GSLN(ltype)
        Last_ltype = ltype
      ENDIF
C
      RETURN
      END
